package com.persagy.dmp.auth;


import java.lang.reflect.Method;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import com.persagy.dmp.entity.JWTUserDto;

import cn.hutool.core.util.StrUtil;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.ExpiredJwtException;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class AuthenticationInterceptor implements HandlerInterceptor {

	@Value("${auth.token.enable:false}")
	private boolean authEnable;
	
    @Override
    public boolean preHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object object) {
    	/*String requestURI = httpServletRequest.getRequestURI();
    	if (requestURI.contains("userRole/token")) {
			return true;
		}*/
    	
        if (!(object instanceof HandlerMethod)) {
            return true;
        }
        HandlerMethod handlerMethod = (HandlerMethod) object;
        Method method = handlerMethod.getMethod();
        boolean anonymousInterface = method.isAnnotationPresent(Anonymous.class);
        if (anonymousInterface) {
            return true;
        }
        
        String token = httpServletRequest.getHeader("Authorization");
        try {
        	// 为了兼容之前的逻辑，这里只要token不为空，就解析，并设置值，方便后续日志记录使用，authEnable 只是控制是否允许访问，不控制是否进行解析
        	if (StrUtil.isNotBlank(token)) {
        		Claims claims = JwtHelper.verifyJwt(token);
                setUser(claims);
			} else if (authEnable) {
				throw new RuntimeException("用户登录信息不可为空");
			}
        } catch (ExpiredJwtException e) {
        	log.warn("token已过期，token：{}", token);
        	if (authEnable) {
        		throw new RuntimeException("用户登录信息已过期");
			}
        } catch (Exception e) {
        	log.warn("token校验失败，token：{}", token);
        	if (authEnable) {
        		throw new RuntimeException("用户登录信息鉴定失败");
			}
        }

        return true;
    }


    private void setUser(Claims claims) {
    	String groupCode = claims.get("groupCode", String.class);
    	String projectId = claims.get("projectId", String.class);
        String userId = claims.get("userId", String.class);
        String username = claims.get("username", String.class);
        String name = claims.get("name", String.class);
        String roleName = claims.get("roleName", String.class);
        String ip = claims.get("ip", String.class);
        JWTUserDto jwtUserDto = new JWTUserDto(userId, username, name, roleName, ip);
        jwtUserDto.setGroupCode(groupCode);
        jwtUserDto.setProjectId(projectId);
        AuthManager.setUser(jwtUserDto);
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) {
        AuthManager.remove();
    }
}