package com.persagy.dmp.auth;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.JwtBuilder;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;

import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.io.UnsupportedEncodingException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public class JwtHelper {

    public static final String JWT_ID = "tokenId";
    /**
     * token 过期时间, 单位: 毫秒. 不过期
     */
    private static final long TOKEN_EXPIRED_TIME = 300000000000L;
    /**
     * jwt 加密解密密钥   ibms --> md5
     */
    private static final String JWT_SECRET = "c200b99a3c0a49028c7d1fda239ae341";

    private static SecretKey KEY = null;

    static {
        try {
            KEY = new SecretKeySpec(JWT_SECRET.getBytes("UTF-8"), 0, JWT_SECRET.length(), "AES");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
    }

    public static String createJWT(Map<String, Object> claims, Long time) {
        SignatureAlgorithm signatureAlgorithm = SignatureAlgorithm.HS256;
        Date now = new Date(System.currentTimeMillis());

        long nowMillis = System.currentTimeMillis();

        JwtBuilder builder = Jwts.builder()
                .setClaims(claims)
                .setId(JWT_ID)
                .setIssuedAt(now)
                .signWith(signatureAlgorithm, KEY);
        if (time >= 0) {
            long expMillis = nowMillis + time;
            Date exp = new Date(expMillis);
            builder.setExpiration(exp);
        }
        return builder.compact();
    }

    /**
     * 验证jwt
     */
    public static Claims verifyJwt(String token) {
        return Jwts.parser()
                .setSigningKey(KEY)
                .parseClaimsJws(token).getBody();
    }

    /**
     * 根据userId和openid生成token
     */
    public static String generateToken(String userId, String name, String roleName, String ip) {
        Map<String, Object> map = new HashMap<>();
        map.put("userId", userId);
        map.put("name", name);
        map.put("roleName", roleName);
        map.put("ip", ip);
        return createJWT(map, TOKEN_EXPIRED_TIME);
    }
    
    /**
     * 
     * @param userId
     * @param name
     * @param roleName
     * @param ip
     * @param expireTime 单位：毫秒，2400000
     * @return
     * @date 2022年4月18日 下午2:55:17
     */
    public static String generateToken(String userId, String name, String roleName, String ip, long expireTime) {
        Map<String, Object> map = new HashMap<>();
        map.put("userId", userId);
        map.put("name", name);
        map.put("roleName", roleName);
        map.put("ip", ip);
        return JwtHelper.createJWT(map, expireTime);
    }
    
    
    /**
     * 根据userId和openid生成token
     */
    public static String generateToken(String groupCode, String projectId, String userId, String username, String name, String roleName, String ip) {
        Map<String, Object> map = new HashMap<>();
        map.put("groupCode", groupCode);
        map.put("projectId", projectId);
        map.put("userId", userId);
        map.put("username", username);
        map.put("name", name);
        map.put("roleName", roleName);
        map.put("ip", ip);
        return createJWT(map, TOKEN_EXPIRED_TIME);
    }
    
    /**
     * 
     * @param userId
     * @param name
     * @param roleName
     * @param ip
     * @param expireTime 单位：毫秒，2400000
     * @return
     * @date 2022年4月18日 下午2:55:17
     */
    public static String generateToken(String groupCode, String projectId, String userId, String username, String name, String roleName, String ip, long expireTime) {
        Map<String, Object> map = new HashMap<>();
        map.put("groupCode", groupCode);
        map.put("projectId", projectId);
        map.put("userId", userId);
        map.put("username", username);
        map.put("name", name);
        map.put("roleName", roleName);
        map.put("ip", ip);
        return JwtHelper.createJWT(map, expireTime);
    }
    
}