package com.persagy.dmp.cache;

import com.alibaba.fastjson.JSONArray;
import com.google.common.base.Joiner;
import com.persagy.dmp.constant.RedisConstant;
import com.persagy.dmp.dto.InfoPointDictDTO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;
import java.util.Random;
import java.util.concurrent.TimeUnit;

/**
 * @description: 数据字典缓存
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/12/29 16:09
 * @version: V1.0
 **/
@Component
public class DataDictCache {

    private final static Logger logger = LoggerFactory.getLogger(DataDictCache.class);

    private static final Joiner joiner = Joiner.on(":").skipNulls();

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    public void putDataDict(String groupCode, String projectId, String classCode, List<InfoPointDictDTO> infoPointDictDTOList) {
        try {
            if (!CollectionUtils.isEmpty(infoPointDictDTOList)) {
                Random random = new Random();
                String key = joiner.join(RedisConstant.DATA_DICT_PREFIX, groupCode, projectId, classCode);
                //随机设定60到120分钟的过期时间
                redisTemplate.opsForValue().set(key, infoPointDictDTOList, random(random, 120, 60), TimeUnit.MINUTES);
            }
        } catch (Exception e) {
            logger.error("数据字典缓存异常", e);
        }
    }

    public List<InfoPointDictDTO> getDataDict(String groupCode, String projectId, String classCode) {
        try {
            String key = joiner.join(RedisConstant.DATA_DICT_PREFIX, groupCode, projectId, classCode);
            Object object = redisTemplate.opsForValue().get(key);
            if (object != null) {
                return JSONArray.parseArray(object.toString()).toJavaList(InfoPointDictDTO.class);
            }
        } catch (Exception e) {
            logger.error("获取数据字典缓存异常", e);
        }
        return null;
    }

    /**
     * @description: 产生随机数
     * @param: random
     * @param: max 随机数最大值
     * @param: min 随机数最小值
     * @return: int
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/29 15:53
     * @version: V1.0
     */
    private int random(Random random, int max, int min) {
        return random.nextInt(max - min + 1) + max;
    }
}
