package com.persagy.dmp.cache;

import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Joiner;
import com.persagy.dmp.constant.RedisConstant;
import com.persagy.dmp.dto.EquipBelongModularDTO;
import com.persagy.dmp.dto.EquipLocationDTO;
import com.persagy.dmp.dto.SpaceLocationDTO;
import com.persagy.dmp.dto.SystemLocationDTO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @description: 对象位置缓存
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/12/23 15:12
 * @version: V1.0
 **/
@Component
public class LocationCache {

    private final static Logger logger = LoggerFactory.getLogger(LocationCache.class);

    private static final Joiner joiner = Joiner.on(":").skipNulls();

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    public void putEquipLocation(String groupCode, String projectId, List<EquipLocationDTO> equipLocationDTOList) {
        try {
            Random random = new Random();

            if (!CollectionUtils.isEmpty(equipLocationDTOList)) {
                Map<String, EquipLocationDTO> map = new HashMap<>();

                equipLocationDTOList.forEach(
                        o -> {
                            String key = joiner.join(RedisConstant.EQUIP_LOCATION_PREFIX, groupCode, projectId, o.getEquipId());
                            map.put(key, o);
                        });
                redisTemplate.opsForValue().multiSet(map);
                equipLocationDTOList.forEach(
                        o -> {
                            String key = joiner.join(RedisConstant.EQUIP_LOCATION_PREFIX, groupCode, projectId, o.getEquipId());
                            //随机设定60到120分钟的过期时间
                            redisTemplate.expire(key, random(random, 120, 60), TimeUnit.MINUTES);
                        });
            }
        } catch (Exception e) {
            logger.error("设备位置缓存异常", e);
        }
    }
    public void putModularBox(String groupCode, String projectId, List<EquipBelongModularDTO> equipLocationDTOList) {
        try {
            Random random = new Random();

            if (!CollectionUtils.isEmpty(equipLocationDTOList)) {
                Map<String, EquipBelongModularDTO> map = new HashMap<>();

                equipLocationDTOList.forEach(
                        o -> {
                            String key = joiner.join(RedisConstant.EQUIP_MODULAR_BOX_PREFIX, groupCode, projectId, o.getEquipId());
                            map.put(key, o);
                        });
                redisTemplate.opsForValue().multiSet(map);
                equipLocationDTOList.forEach(
                        o -> {
                            String key = joiner.join(RedisConstant.EQUIP_MODULAR_BOX_PREFIX, groupCode, projectId, o.getEquipId());
                            //随机设定60到120分钟的过期时间
                            redisTemplate.expire(key, random(random, 120, 60), TimeUnit.MINUTES);
                        });
            }
        } catch (Exception e) {
            logger.error("设备位置缓存异常", e);
        }
    }

    public List<EquipLocationDTO> getEquipLocation(String groupCode, String projectId, List<String> equipId) {
        List<EquipLocationDTO> equipLocationDTOList = new ArrayList<>();
        try {
            List<String> list = new ArrayList<>();
            equipId.forEach(o -> list.add(joiner.join(RedisConstant.EQUIP_LOCATION_PREFIX, groupCode, projectId, o)));
            List<Object> objectList = redisTemplate.opsForValue().multiGet(list);
            if (!CollectionUtils.isEmpty(objectList)) {
                objectList.forEach(o -> {
                    if (o != null) {
                        equipLocationDTOList.add(JSONObject.parseObject(o.toString()).toJavaObject(EquipLocationDTO.class));
                    }
                });
            }
        } catch (Exception e) {
            logger.error("获取设备位置缓存异常", e);
        }
        return equipLocationDTOList;
    }
    public List<EquipBelongModularDTO> getEquipBelongModular(String groupCode, String projectId, List<String> equipId) {
        List<EquipBelongModularDTO> equipLocationDTOList = new ArrayList<>();
        try {
            List<String> list = new ArrayList<>();
            equipId.forEach(o -> list.add(joiner.join(RedisConstant.EQUIP_MODULAR_BOX_PREFIX, groupCode, projectId, o)));
            List<Object> objectList = redisTemplate.opsForValue().multiGet(list);
            if (!CollectionUtils.isEmpty(objectList)) {
                objectList.forEach(o -> {
                    if (o != null) {
                        equipLocationDTOList.add(JSONObject.parseObject(o.toString()).toJavaObject(EquipBelongModularDTO.class));
                    }
                });
            }
        } catch (Exception e) {
            logger.error("获取设备位置缓存异常", e);
        }
        return equipLocationDTOList;
    }


    public void putSpaceLocation(String groupCode, String projectId, List<SpaceLocationDTO> spaceLocationDTOList) {
        try {
            Random random = new Random();
            if (!CollectionUtils.isEmpty(spaceLocationDTOList)) {
                Map<String, SpaceLocationDTO> map = new HashMap<>();

                spaceLocationDTOList.forEach(
                        o -> {
                            String key = joiner.join(RedisConstant.SPACE_LOCATION_PREFIX, groupCode, projectId, o.getSpaceId());
                            map.put(key, o);
                        });
                redisTemplate.opsForValue().multiSet(map);
                spaceLocationDTOList.forEach(
                        o -> {
                            String key = joiner.join(RedisConstant.SPACE_LOCATION_PREFIX, groupCode, projectId, o.getSpaceId());
                            //随机设定60到120分钟的过期时间
                            redisTemplate.expire(key, random(random, 120, 60), TimeUnit.MINUTES);
                        });

            }
        } catch (Exception e) {
            logger.error("空间位置缓存异常", e);
        }
    }


    public List<SpaceLocationDTO> getSpaceLocation(String groupCode, String projectId, List<String> spaceId) {
        List<SpaceLocationDTO> spaceLocationDTOList = new ArrayList<>();
        try {
            List<String> list = new ArrayList<>();
            spaceId.forEach(o -> list.add(joiner.join(RedisConstant.SPACE_LOCATION_PREFIX, groupCode, projectId, o)));
            List<Object> objectList = redisTemplate.opsForValue().multiGet(list);
            if (!CollectionUtils.isEmpty(objectList)) {
                objectList.forEach(o -> {
                    if (o != null) {
                        spaceLocationDTOList.add(JSONObject.parseObject(o.toString()).toJavaObject(SpaceLocationDTO.class));
                    }
                });
            }
        } catch (Exception e) {
            logger.error("获取空间位置缓存异常", e);
        }
        return spaceLocationDTOList;
    }


    public void putSystemLocation(String groupCode, String projectId, List<SystemLocationDTO> systemLocationDTOList) {
        try {
            Random random = new Random();
            if (!CollectionUtils.isEmpty(systemLocationDTOList)) {
                Map<String, SystemLocationDTO> map = new HashMap<>();

                systemLocationDTOList.forEach(
                        o -> {
                            String key = joiner.join(RedisConstant.SYSTEM_LOCATION_PREFIX, groupCode, projectId, o.getSystemId());
                            map.put(key, o);
                        });
                redisTemplate.opsForValue().multiSet(map);
                systemLocationDTOList.forEach(
                        o -> {
                            String key = joiner.join(RedisConstant.SYSTEM_LOCATION_PREFIX, groupCode, projectId, o.getSystemId());
                            //随机设定60到120分钟的过期时间
                            redisTemplate.expire(key, random(random, 120, 60), TimeUnit.MINUTES);
                        });
            }
        } catch (Exception e) {
            logger.error("系统位置缓存异常", e);
        }
    }


    public List<SystemLocationDTO> getSystemLocation(String groupCode, String projectId, List<String> systemId) {
        List<SystemLocationDTO> systemLocationDTOList = new ArrayList<>();
        try {
            List<String> list = new ArrayList<>();
            systemId.forEach(o -> list.add(joiner.join(RedisConstant.SYSTEM_LOCATION_PREFIX, groupCode, projectId, o)));
            List<Object> objectList = redisTemplate.opsForValue().multiGet(list);
            if (!CollectionUtils.isEmpty(objectList)) {
                objectList.forEach(o -> {
                    if (o != null) {
                        systemLocationDTOList.add(JSONObject.parseObject(o.toString()).toJavaObject(SystemLocationDTO.class));
                    }
                });
            }
        } catch (Exception e) {
            logger.error("获取系统位置缓存异常", e);
        }
        return systemLocationDTOList;
    }

    /**
     * @description: 产生随机数
     * @param: random
     * @param: max 随机数最大值
     * @param: min 随机数最小值
     * @return: int
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/29 15:53
     * @version: V1.0
     */
    private int random(Random random, int max, int min) {
        return random.nextInt(max - min + 1) + max;
    }
}
