package com.persagy.dmp.cache;

import com.alibaba.fastjson.JSON;
import com.google.common.base.Joiner;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.Collections;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * @author : ZhangWenTao
 * @version : 1.0
 * @since : 2022/7/21 14:50
 */
@Slf4j
@Component
public class RedisCache {

    private static final Joiner joiner = Joiner.on(":");

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;


    public void put(String prefix, String key, Object value, long timeout, TimeUnit unit) {

        try {
            redisTemplate.opsForValue().set(joiner.join(prefix, key), toJSONString(value), timeout, unit);
        } catch (Exception e) {
            log.warn("set fail", e);
        }
    }

    public <T> T get(String prefix, String key, Class<T> clazz) {
        Object value = null;

        try {
            value = redisTemplate.opsForValue().get(joiner.join(prefix, key));
        } catch (Exception e) {
            log.warn("get fail", e);
        }

        if (Objects.isNull(value)) {
            return null;
        }

        return parseObject(value, clazz);
    }

    private String toJSONString(Object object) {
        return JSON.toJSONString(object);
    }

    private <T> T parseObject(Object object, Class<T> clazz) {
        return JSON.parseObject(object.toString(), clazz);
    }

    public String joinKeys(String... strings) {
        if (ArrayUtils.isEmpty(strings)) {
            return StringUtils.EMPTY;
        }

        return joiner.join(strings);
    }

    public String md5(Collection<String> source) {
        return DigestUtils.md5Hex(joinKeys(
                ObjectUtils.defaultIfNull(source, Collections.<String>emptyList())
                        .stream()
                        .sorted()
                        .toArray(String[]::new))
        );
    }
}
