package com.persagy.dmp.cache;

import com.alibaba.fastjson.JSONArray;
import com.google.common.base.Joiner;
import com.persagy.dmp.constant.RedisConstant;
import com.persagy.dmp.service.ObjectService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.data.redis.connection.RedisConnection;
import org.springframework.data.redis.core.Cursor;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ScanOptions;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * @description: 关系缓存
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2021/1/4 18:04
 * @version: V1.0
 **/
@Component
public class RelationCache {

    private final static Logger logger = LoggerFactory.getLogger(RelationCache.class);

    private static final Joiner joiner = Joiner.on(":").skipNulls();

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private ObjectService objectService;

    public void putRelation(String prefix, String groupCode, String projectId, String id, String category, JSONArray objArr) {
        try {
            String key = joiner.join(prefix, groupCode, projectId, id, category);
            redisTemplate.opsForValue().set(key, objArr, 60, TimeUnit.MINUTES);
        } catch (Exception e) {
            logger.error("关系对象缓存异常", e);
        }
    }

    public JSONArray getRelation(String prefix, String groupCode, String projectId, String id, String category) {
        try {
            String key = joiner.join(prefix, groupCode, projectId, id, category);
            Object object = redisTemplate.opsForValue().get(key);
            if (object != null) {
                return JSONArray.parseArray(object.toString());
            }
        } catch (Exception e) {
            logger.error("获取关系对象缓存异常", e);
        }
        return null;
    }

    public void putRelation(String groupCode, String projectId, String graphCode, String relCode, String fromId, List<String> toIds) {
        try {
            String key = joiner.join(RedisConstant.RELATION_PREFIX, groupCode, projectId, graphCode, relCode,
                    "from", fromId, "to", toIds);
            Set<String> keys = redisTemplate.keys(key + "*");

        } catch (Exception e) {
            logger.error("获取系统位置缓存异常", e);
        }
    }

    public void putRelation(String groupCode, String projectId, String graphCode, String relCode, String fromId, String toId) {
        try {
            String key = joiner.join(RedisConstant.RELATION_PREFIX, groupCode, projectId, graphCode, relCode,
                    "from", fromId, "to", toId);
            Cursor<Object> cursor = redisTemplate.opsForSet().scan("", ScanOptions.scanOptions().match("").build());

            redisTemplate.opsForSet().scan("setValue", ScanOptions.scanOptions().match("C").build());
            while (cursor.hasNext()) {
                Object object = cursor.next();
                System.out.println("通过scan(K key, ScanOptions options)方法获取匹配的值:" + object);
            }
            redisTemplate.executePipelined(new RedisCallback<Object>() {
                @Override
                public Object doInRedis(RedisConnection redisConnection) throws DataAccessException {
                    //  redisConnection.
                    return null;
                }
            });

        } catch (Exception e) {
            logger.error("获取系统位置缓存异常", e);
        }
    }
}
