package com.persagy.dmp.client;

import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.dmp.common.DmpResult;
import com.persagy.dmp.common.url.FuncIdUrParam;
import com.persagy.dmp.common.url.InstanceUrlParam;
import com.persagy.dmp.constant.RequestUrlConstant;
import com.persagy.dmp.dto.QueryFuncId;

/**
 * 最新的报警类型查询服务
 * @author 易涛
 * @version 1.0
 * @date 2021/6/23 15:58
 */
@FeignClient(name = "PERSAGY-ZKT-DMP", url = "${persagy.zkt.dmp.url:}")
public interface AlarmNewServerClient {
    
    /**
     * 产品模块（场景）下的管理对象类型
     * @param jsonObject 产品模块编码
     * @author 易涛
     * @date 2021/6/23 17:02 
     * @return com.alibaba.fastjson.JSONObject
     */
    @PostMapping("/rwd/def/class")
    JSONObject queryObjectClass(@SpringQueryMap InstanceUrlParam instanceUrlParam,@RequestBody JSONObject jsonObject);

    /**
     * 管理对象类型的信息点
     * @param jsonObject 模块，类型编码
     * @author 易涛
     * @date 2021/6/23 17:07 
     * @return com.alibaba.fastjson.JSONObject
     */
    @PostMapping("/rwd/def/funcid")
    JSONObject queryInfoCode(@SpringQueryMap InstanceUrlParam instanceUrlParam,@RequestBody JSONObject jsonObject);
    
    /**
     * 查询对象
     * @param instanceUrlParam get的对象
     * @param jsonObject post对象
     * @author 易涛
     * @date 2021/6/23 16:26 
     * @return com.alibaba.fastjson.JSONObject
     */
    @PostMapping("/rwd/instance/object/query")
    JSONObject  queryObject(@SpringQueryMap InstanceUrlParam instanceUrlParam, @RequestBody JSONObject jsonObject);

    /**
     * 查询关系
     * @param instanceUrlParam get的对象
     * @param jsonObject post对象
     * @author 易涛
     * @date 2021/6/23 16:42
     * @return com.alibaba.fastjson.JSONObject
     */
    @PostMapping("/rwd/instance/relation/query")
    JSONObject queryRelation(@SpringQueryMap InstanceUrlParam instanceUrlParam, @RequestBody JSONObject jsonObject);

    /**
     * 查询对象关系
     * @param instanceUrlParam get的对象
     * @param jsonObject post对象
     * @author 易涛
     * @date 2021/6/23 16:42
     * @return com.alibaba.fastjson.JSONObject
     */
    @PostMapping("/rwd/instance/relation/object/query")
    JSONObject queryObjectRelation(@SpringQueryMap InstanceUrlParam instanceUrlParam, @RequestBody JSONObject jsonObject);



    /**
     * @description: 查询信息点定义
     * @param: funcIdUrParam
     * @return: com.persagy.dmp.common.DmpResult
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/28 15:51
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.DEF_FUNCID)
    DmpResult<JSONArray> queryFuncId(@SpringQueryMap FuncIdUrParam funcIdUrParam, QueryFuncId queryFuncId);
}
