package com.persagy.dmp.client;

import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import com.persagy.dmp.constant.RequestUrlConstant;
import com.persagy.dmp.dto.collect.BatchPointReportParam;
import com.persagy.dmp.dto.collect.BatchPointReportResult;
import com.persagy.dmp.dto.collect.PointReportParam;
import com.persagy.dmp.dto.collect.PointReportResult;

/**
 * @description: 采集服务
 * @author: shiliqiang
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/9/21 18:15
 * @version: V1.0
 */
@FeignClient(name = "iot-collect", url = "${iot.collect.feign.client.url:}")
public interface EmsCollectClient {

    /**
     * @description: 异步采集-读取点位
     * @param: pointReportParam
     * @return: com.persagy.dmp.dto.collect.PointReportResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:44
     * @version: V1.0
     */
    @PostMapping(value = RequestUrlConstant.POINT_READ_POST)
    PointReportResult pointRead(@RequestBody PointReportParam pointReportParam);

    /**
     * @description: 异步采集-批量读取点位
     * @param: pointReportParam
     * @return: com.persagy.dmp.dto.collect.PointReportResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:44
     * @version: V1.0
     */
    @PostMapping(value = RequestUrlConstant.POINT_READ_BATCH_POST)
    BatchPointReportResult pointReadBatch(@RequestBody BatchPointReportParam batchPointReportParam);

    /**
     * @description: 查询采集当前值
     * @param: pointReportParam
     * @return: com.persagy.dmp.dto.collect.PointReportResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:43
     * @version: V1.0
     */
    @PostMapping(value = RequestUrlConstant.PRESENT_REPORT_POST)
    PointReportResult presentReport(@RequestBody PointReportParam pointReportParam);

    /**
     * @description: 批量查询采集当前值
     * @param: batchPointReportParam
     * @return: com.persagy.dmp.dto.collect.BatchPointReportResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:43
     * @version: V1.0
     */
    @PostMapping(value = RequestUrlConstant.PRESENT_REPORT_BATCH_POST)
    BatchPointReportResult presentReportBatch(@RequestBody BatchPointReportParam batchPointReportParam);
}
