package com.persagy.dmp.client;

import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import com.persagy.dmp.constant.RequestUrlConstant;
import com.persagy.dmp.dto.control.BatchPointSetParam;
import com.persagy.dmp.dto.control.BatchPointSetResult;
import com.persagy.dmp.dto.control.PointSetParam;
import com.persagy.dmp.dto.control.PointSetResult;

/**
 * @description: 控制程序（指令下发）
 * @author: shiliqiang
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/10/30 16:44
 * @version: V1.0
 */
@FeignClient(name = "iot-collect", url = "${iot.collect.feign.client.url:}")
public interface EmsControlClient {

    /**
     * @description: 异步控制-设置点位
     * @param: pointSetParam
     * @return: com.persagy.dmp.dto.control.PointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:41
     * @version: V1.0
     */
    @PostMapping(value = RequestUrlConstant.POINT_SET_POST)
    PointSetResult pointSet(@RequestBody PointSetParam pointSetParam);

    /**
     * @description: 异步控制-批量设置点位
     * @param: batchPointSetParam
     * @return: com.persagy.dmp.dto.control.BatchPointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:41
     * @version: V1.0
     */
    @PostMapping(value = RequestUrlConstant.POINT_SET_BATCH_POST)
    BatchPointSetResult pointSetBatch(@RequestBody BatchPointSetParam batchPointSetParam);

    /**
     * @description: 同步控制-设置点位
     * @param: pointSetParam
     * @return: com.persagy.dmp.dto.control.PointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:41
     * @version: V1.0
     */
    @PostMapping(value = RequestUrlConstant.SYNC_POINT_SET_POST)
    PointSetResult syncPointSet(@RequestBody PointSetParam pointSetParam);

    /**
     * @description: 同步控制-批量设置点位
     * @param: batchPointSetParam
     * @return: com.persagy.dmp.dto.control.BatchPointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:41
     * @version: V1.0
     */
    @PostMapping(value = RequestUrlConstant.SYNC_POINT_SET_BATCH_POST)
    BatchPointSetResult syncPointSetBatch(@RequestBody BatchPointSetParam batchPointSetParam);

    /**
     * @description: 异步控制-批量查询控制结果
     * @param: batchPointSetParam
     * @return: com.persagy.dmp.dto.control.BatchPointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:41
     * @version: V1.0
     */
    @PostMapping(value = RequestUrlConstant.POINT_SET_RESULT_BATCH_POST)
    BatchPointSetResult pointSetResultBatch(@RequestBody BatchPointSetParam batchPointSetParam);

    /**
     * @description: 查询控制当前值
     * @param: pointSetParam
     * @return: com.persagy.dmp.dto.control.PointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:42
     * @version: V1.0
     */
    @PostMapping(value = RequestUrlConstant.PRESENT_SET_POST)
    PointSetResult presentSet(@RequestBody PointSetParam pointSetParam);

    /**
     * @description: 批量查询控制当前值
     * @param: batchPointSetParam
     * @return: com.persagy.dmp.dto.control.BatchPointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:42
     * @version: V1.0
     */
    @PostMapping(value = RequestUrlConstant.PRESENT_SET_BATCH_POST)
    BatchPointSetResult presentSetBatch(@RequestBody BatchPointSetParam batchPointSetParam);
}
