package com.persagy.dmp.client;

import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.dmp.common.DmpResult;
import com.persagy.dmp.common.url.DicUrlParam;
import com.persagy.dmp.constant.RequestUrlConstant;
import com.persagy.dmp.dto.org.GroupInfoDTO;
import com.persagy.dmp.dto.org.ProjectInfoDTO;

/**
 * @author LuoGuangyi
 * @version 1.0.0
 * @Description 数据中台人员组织机构
 * @ClassName OrgClient.java
 * @createTime 2021/01/13 11:54
 * @Since
 */
@FeignClient(name = "dmp-org", url = "${persgay.dmp.org.url:}")
public interface OrgServerClient {

    /**
     * @param dicUrlParam:
     * @param criteria:
     * @description: 查询用户信息
     * @author: LuoGuangyi
     * @createTime: 2021/01/13 12:59
     * @return: com.persagy.dmp.common.DmpResult<com.alibaba.fastjson.JSONArray>
     * @expression
     */
    @PostMapping(RequestUrlConstant.ORG_USER_QUERY)
    DmpResult<JSONArray> queryUser(@SpringQueryMap DicUrlParam dicUrlParam, @RequestBody JSONObject criteria);

    /**
     * @description: 查询集团
     * @param: dicUrlParam:
     * @param: criteria:
     * @return: com.persagy.dmp.common.DmpResult<com.persagy.dmp.dto.org.GroupInfoDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/3/16 10:26
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.ORG_GROUP_QUERY)
    DmpResult<GroupInfoDTO> queryGroup(@SpringQueryMap DicUrlParam dicUrlParam, @RequestBody JSONObject criteria);

    /**
     * @description: 查询项目
     * @param: dicUrlParam:
     * @param: criteria:
     * @return: com.persagy.dmp.common.DmpResult<ProjectInfoDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/3/16 10:26
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.ORG_PROJECT_QUERY)
    DmpResult<ProjectInfoDTO> queryProject(@SpringQueryMap DicUrlParam dicUrlParam, @RequestBody JSONObject criteria);
}
