package com.persagy.dmp.client;

import java.util.List;

import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.dmp.common.DmpResult;
import com.persagy.dmp.common.url.ClassUrParam;
import com.persagy.dmp.common.url.FuncIdUrParam;
import com.persagy.dmp.common.url.IOTUrlParam;
import com.persagy.dmp.common.url.InstanceUrlParam;
import com.persagy.dmp.constant.RequestUrlConstant;
import com.persagy.dmp.dto.DataQueryDTO;
import com.persagy.dmp.dto.HistoryDataQuery;
import com.persagy.dmp.dto.QueryFuncId;
import com.persagy.dmp.dto.relation.CreateRelation;
import com.persagy.dmp.dto.relation.QueryRelation;
import com.persagy.dmp.dto.relation.QueryRelationObject;

/**
 * @description: 数据中台物理世界、数据字典、IOT 接口feign
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/10/28 11:28
 * name = "dmp-rwd"
 * name = "persagy-zkt-dmp"
 * url = "http://39.102.43.179:9976"
 * @version: V1.0
 */
@FeignClient(name = "persagy-zkt-dmp", url = "${persagy.zkt.dmp.url:}")
public interface RwdServerClient {

    /**
     * @description: 查询对象列表
     * @param: objectUrlParam 对象请求url通用参数
     * @param: criteria post请求参数
     * @return: com.persagy.dmp.common.DmpResult
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/21 17:09
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.OBJECT_QUERY)
    DmpResult<JSONArray> queryObject(@SpringQueryMap InstanceUrlParam objectUrlParam, @RequestBody JSONObject criteria);

    /**
     * @description: 创建对象
     * @param: objectUrlParam 对象请求url通用参数
     * @param: criteria post请求参数
     * @return: com.persagy.dmp.common.DmpResult
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/21 17:09
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.OBJECT_CREATE)
    DmpResult createObject(@SpringQueryMap InstanceUrlParam objectUrlParam, @RequestBody JSONArray objectArr);

    /**
     * @description: 更新对象
     * @param: objectUrlParam 对象请求url通用参数
     * @param: criteria post请求参数
     * @return: com.persagy.dmp.common.DmpResult
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/21 17:09
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.OBJECT_UPDATE)
    DmpResult updateObject(@SpringQueryMap InstanceUrlParam objectUrlParam, @RequestBody JSONArray objectArr);

    /**
     * @description: 删除对象
     * @param: objectUrlParam 对象请求url通用参数
     * @param: criteria post请求参数
     * @return: com.persagy.dmp.common.DmpResult
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/21 17:09
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.OBJECT_DELETE)
    DmpResult deleteObject(@SpringQueryMap InstanceUrlParam objectUrlParam, @RequestBody List<String> objectIdList);

    /**
     * @description: 创建关系
     * @param: objectUrlParam
     * @param: relationCriteriaList
     * @return: com.persagy.dmp.common.DmpResult
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/22 16:37
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.RELATION_CREATE)
    DmpResult createRelation(@SpringQueryMap InstanceUrlParam objectUrlParam, @RequestBody List<CreateRelation> createRelationList);

    /**
     * @description: 删除关系
     * @param: objectUrlParam
     * @param: relationCriteriaList
     * @return: com.persagy.dmp.common.DmpResult
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/22 16:37
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.RELATION_DELETE)
    DmpResult deleteRelation(@SpringQueryMap InstanceUrlParam objectUrlParam, @RequestBody JSONObject criteria);

    /**
     * @description: 查询关系
     * @param: objectUrlParam
     * @param: relationCriteriaList
     * @return: com.persagy.dmp.common.DmpResult
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/22 16:37
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.RELATION_QUERY)
    DmpResult<JSONArray> queryRelation(@SpringQueryMap InstanceUrlParam objectUrlParam, @RequestBody QueryRelation criteria);

    /**
     * @description: 查询关系对象
     * @param: objectUrlParam
     * @param: relationCriteriaList
     * @return: com.persagy.dmp.common.DmpResult
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/22 16:37
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.RELATION_OBJECT_QUERY)
    DmpResult<JSONArray> queryRelationObject(@SpringQueryMap InstanceUrlParam objectUrlParam, @RequestBody QueryRelationObject criteria);

    /**
     * @description: 查询iot实时数据
     * @param: iotUrlParam iot请求url通用参数
     * @param: dataQueryDTOList post请求参数
     * @return: com.persagy.dmp.common.DmpResult
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/21 15:19
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.OBJECT_CURRENT_DATA)
    DmpResult<JSONArray> queryIotCurrentData(@SpringQueryMap IOTUrlParam iotUrlParam, @RequestBody List<DataQueryDTO> dataQueryDTOList);

    /**
     * @description: 查询iot历史数据/分精度数据
     * @param: iotUrlParam iot请求url通用参数
     * @param: historyDataQuery post请求参数
     * @return: com.persagy.dmp.common.DmpResult
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/21 15:20
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.OBJECT_HISTORY_DATA)
    DmpResult<JSONArray> queryIotHistoryData(@SpringQueryMap IOTUrlParam iotUrlParam, @RequestBody HistoryDataQuery historyDataQuery);

    /**
     * @description: 查询信息点定义
     * @param: funcIdUrParam
     * @return: com.persagy.dmp.common.DmpResult
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/28 15:51
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.DEF_FUNCID)
    DmpResult<JSONArray> queryFuncId(@SpringQueryMap FuncIdUrParam funcIdUrParam, QueryFuncId queryFuncId);

    /**
     * @description: 查询类型定义
     * @param: classUrParam
     * @param: criteria
     * @return: com.persagy.dmp.common.DmpResult
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/28 15:51
     * @version: V1.0
     */
    @PostMapping(RequestUrlConstant.DEF_CLASS)
    DmpResult<JSONArray> queryClass(@SpringQueryMap ClassUrParam classUrParam, JSONObject criteria);
}
