package com.persagy.dmp.config;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.annotation.WebFilter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpHeaders;

/**
 * @description: 允许跨域 Filter
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2021/4/22 16:03
 * @version: V1.0
 **/
@Order(1)
@WebFilter
public class CrossOriginFilter implements Filter {
    /**
     * true 常量
     */
    public static final String CONST_STRING_TRUE = "true";
    
	@Value("${exclude.cross.origin:false}")
	private boolean excludeCrossOrigin;

    @Override
    public void init(FilterConfig filterConfig) {

    }

    @Override
    public void destroy() {

    }

    public void process(HttpServletRequest request, HttpServletResponse response) {
        String value = request.getHeader(HttpHeaders.ORIGIN);
        if (!excludeCrossOrigin) {
        	response.setHeader(HttpHeaders.ACCESS_CONTROL_ALLOW_ORIGIN, (StringUtils.isBlank(value) || "null".equalsIgnoreCase(value)) ? "*" : value);
		}
        response.setHeader(HttpHeaders.ACCESS_CONTROL_ALLOW_CREDENTIALS, CONST_STRING_TRUE);
        value = request.getHeader(HttpHeaders.ACCESS_CONTROL_REQUEST_HEADERS);
        if (StringUtils.isNotBlank(value)) {
            response.setHeader(HttpHeaders.ACCESS_CONTROL_ALLOW_HEADERS, value);
        }else {
            response.setHeader(HttpHeaders.ACCESS_CONTROL_ALLOW_HEADERS, "*" );
        }
        response.setHeader(HttpHeaders.ACCESS_CONTROL_MAX_AGE, "3600" );
        value = request.getHeader(HttpHeaders.ACCESS_CONTROL_REQUEST_METHOD);
        response.setHeader(HttpHeaders.ACCESS_CONTROL_ALLOW_METHODS, (StringUtils.isBlank(value) || "null".equalsIgnoreCase(value)) ? "GET,POST,OPTIONS,PUT,DELETE" : value);
    }

    @Override
    public void doFilter(ServletRequest req, ServletResponse resp, FilterChain chain) throws IOException, ServletException {
        HttpServletRequest request = (HttpServletRequest) req;
        process(request, (HttpServletResponse) resp);
        chain.doFilter(req, resp);
    }
}
