package com.persagy.dmp.dao;

import com.persagy.dmp.dto.control.BatchPointSetParam;
import com.persagy.dmp.dto.control.BatchPointSetResult;
import com.persagy.dmp.dto.control.PointSetParam;
import com.persagy.dmp.dto.control.PointSetResult;

/**
 * @description: 控制dao接口
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/12/18 18:20
 * @version: V1.0
 **/
public interface ControlDao {

    /**
     * @description: 异步控制-设置点位
     * @param: pointSetParam
     * @return: com.persagy.dmp.dto.control.PointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:20
     * @version: V1.0
     */
    PointSetResult pointSet(PointSetParam pointSetParam);

    /**
     * @description: 异步控制-批量设置点位
     * @param: pointSetParam
     * @return: com.persagy.dmp.dto.control.BatchPointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:20
     * @version: V1.0
     */
    BatchPointSetResult pointSetBatch(BatchPointSetParam batchPointSetParam);

    /**
     * @description: 同步控制-设置点位
     * @param: pointSetParam
     * @return: com.persagy.dmp.dto.control.PointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:20
     * @version: V1.0
     */
    PointSetResult syncPointSet(PointSetParam pointSetParam);

    /**
     * @description: 同步控制-批量设置点位
     * @param: pointSetParam
     * @return: com.persagy.dmp.dto.control.BatchPointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:20
     * @version: V1.0
     */
    BatchPointSetResult syncPointSetBatch(BatchPointSetParam batchPointSetParam);

    /**
     * @description: 异步控制-批量查询控制结果
     * @param: pointSetParam
     * @return: com.persagy.dmp.dto.control.BatchPointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:20
     * @version: V1.0
     */
    BatchPointSetResult pointSetResultBatch(BatchPointSetParam batchPointSetParam);

    /**
     * @description: 查询控制当前值
     * @param: pointSetParam
     * @return: com.persagy.dmp.dto.control.PointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:20
     * @version: V1.0
     */
    PointSetResult presentSet(PointSetParam pointSetParam);

    /**
     * @description: 批量查询控制当前值
     * @param: pointSetParam
     * @return: com.persagy.dmp.dto.control.BatchPointSetResult
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/18 20:20
     * @version: V1.0
     */
    BatchPointSetResult presentSetBatch(BatchPointSetParam batchPointSetParam);
}
