package com.persagy.dmp.dao;

import com.alibaba.fastjson.JSONArray;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.dmp.dto.FuncIdQueryDTO;
import com.persagy.dmp.dto.InfoPointDictDTO;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @description: 字典定义
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/10/28 15:24
 * @version: V1.0
 */
public interface DefDao {

    /***
     * @description: 获取数据字典平台级别信息点属性信息
     * @param groupCode 集团编码
     * @param: projectId
     * @param: classCode
     * @return: java.util.List<com.persagy.huiyun.dto.InfoPointDictDTO>
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/6/5 17:14
     * @version: V1.0
     */
    List<InfoPointDictDTO> queryInfoPointDictList(String groupCode, String projectId, String classCode) throws Exception;

    /**
     * @param groupCode 集团编码
     * @description: 获取数据字典信息点属性信息
     * @param: projectId 项目id
     * @param: funcIdQueryDto 查询信息点参数类
     * @return: java.util.List<com.persagy.common.dto.InfoPointDictDTO>
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/29 10:56
     * @version: V1.0
     */
    List<InfoPointDictDTO> queryInfoPointDictList(String groupCode, String projectId, FuncIdQueryDTO funcIdQueryDto) throws Exception;


    /**
     * @param groupCode                        集团编码
     * @param projectId
     * @param subFlag:true/false,是否按分类信息点展示子分类 可选,默认false
     * @param criteria:参数自定义可传id，code等
     * @description: 查询类信息（可实例化的）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: shiliqiang
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/11 16:44
     * @version: V1.0
     */
    JSONArray queryClassInfoList(String groupCode, @NotNull String projectId, Boolean subFlag, @NotNull JacksonCriteria criteria) throws Exception;

}