package com.persagy.dmp.dao;

import com.persagy.dmp.dto.CurrentDataDTO;
import com.persagy.dmp.dto.DataQueryDTO;
import com.persagy.dmp.dto.HistoryDataDTO;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @description: iot数据接口
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/10/15 11:58
 * @version: V1.0
 **/
public interface IOTDao {

    /**
     * @description: 查询实时数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCode 信息点编码
     * @return: com.persagy.common.dto.CurrentDataDTO
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:12
     * @version: V1.0
     */
    CurrentDataDTO queryCurrentData(String groupCode, @NotNull String projectId, @NotNull String objectId, @NotNull String infoCode) throws Exception;

    /**
     * @description: 查询实时数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIdList 对象列表
     * @param: infoCode 信息点编码
     * @return: java.util.List<com.persagy.common.dto.CurrentDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:12
     * @version: V1.0
     */
    List<CurrentDataDTO> queryCurrentDataList(String groupCode, @NotNull String projectId, @NotNull List<String> objectIdList, @NotNull String infoCode) throws Exception;

    /**
     * @description: 查询实时数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCodeList 信息点编码列表
     * @return: java.util.List<com.persagy.common.dto.CurrentDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:11
     * @version: V1.0
     */

    List<CurrentDataDTO> queryCurrentDataList(String groupCode, @NotNull String projectId, @NotNull String objectId, @NotNull List<String> infoCodeList) throws Exception;

    /**
     * @description: 查询实时数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: dataQueryDtoList 实时数据参数（对象id、信息点）列表
     * @return: java.util.List<com.persagy.common.dto.CurrentDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:11
     * @version: V1.0
     */
    List<CurrentDataDTO> queryCurrentDataList(String groupCode, @NotNull String projectId, @NotNull List<DataQueryDTO> dataQueryDTOList) throws Exception;


    /**
     * @description: 查询历史数据/分精度数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCode 信息点编码
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 时间间隔
     * @return: com.persagy.common.dto.HistoryDataDTO
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:06
     * @version: V1.0
     */
    HistoryDataDTO queryHistoryDataList(String groupCode, @NotNull String projectId, @NotNull String objectId, @NotNull String infoCode, @NotNull String startTime, String endTime, @NotNull String period) throws Exception;

    /**
     * @description: 查询历史数据/分精度数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCodeList 信息点编码列表
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 时间间隔
     * @return: java.util.List<com.persagy.common.dto.HistoryDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:06
     * @version: V1.0
     */
    List<HistoryDataDTO> queryHistoryDataList(String groupCode, @NotNull String projectId, @NotNull String objectId, @NotNull List<String> infoCodeList, @NotNull String startTime, String endTime, @NotNull String period) throws Exception;

    /**
     * @description: 查询历史数据/分精度数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIdList 对象id列表
     * @param: infoCode 信息点编码
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 时间间隔
     * @return: java.util.List<com.persagy.common.dto.HistoryDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:05
     * @version: V1.0
     */
    List<HistoryDataDTO> queryHistoryDataList(String groupCode, @NotNull String projectId, @NotNull List<String> objectIdList, @NotNull String infoCode, @NotNull String startTime, String endTime, String period) throws Exception;

    /**
     * @description: 查询历史数据/分精度数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: dataQueryDtoList 查询历史数据的对象id、信息点信息点编码类列表
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 时间间隔
     * @return: java.util.List<com.persagy.common.dto.HistoryDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:04
     * @version: V1.0
     */
    List<HistoryDataDTO> queryHistoryDataList(String groupCode, @NotNull String projectId, @NotNull List<DataQueryDTO> dataQueryDTOList, @NotNull String startTime, String endTime, @NotNull String period) throws Exception;
}
