package com.persagy.dmp.dao.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.client.DicServerClient;
import com.persagy.dmp.common.DmpResult;
import com.persagy.dmp.common.url.DicUrlParam;
import com.persagy.dmp.constant.RequestUrlConstant;
import com.persagy.dmp.dao.DicDao;
import com.persagy.dmp.dto.dict.DictDataDTO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @description: 字典接口实现类
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/10/28 15:18
 * @version: V1.0
 **/
@Component
public class DicDaoImpl implements DicDao {

    private static final Logger logger = LoggerFactory.getLogger(DicDaoImpl.class);

    @Autowired
    private DicServerClient dicServerClient;

    @Override
    public List<DictDataDTO> queryDictDataList(String groupCode, String projectId, String type) throws Exception {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("type", type);
        return queryDictDataList(groupCode, projectId, criteria);
    }

    @Override
    public List<DictDataDTO> queryDictDataList(String groupCode, String projectId, JacksonCriteria criteria) throws Exception {
        DicUrlParam dicUrlParam = DicUrlParam.builder().groupCode(groupCode).projectId(projectId).build();
        DmpResult<JSONArray> dmpResult = dicServerClient.dataQuery(dicUrlParam, JSONObject.parseObject(JacksonMapper.toFormatJson(criteria)));
        if (dmpResult != null && DmpResult.SUCCESS.equalsIgnoreCase(dmpResult.getResult())) {
            JSONArray dataArray = dmpResult.getData();
            if (!CollectionUtils.isEmpty(dataArray)) {
                return dataArray.toJavaList(DictDataDTO.class);
            }
        } else {
            logger.error("字典请求:{}，参数:{}，返回值:{}", RequestUrlConstant.DICT_QUERY, dmpResult);
        }
        return new ArrayList<>();
    }
}


