package com.persagy.dmp.dao.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.json.JacksonMapper;
import com.persagy.dmp.client.OrgServerClient;
import com.persagy.dmp.common.DmpResult;
import com.persagy.dmp.common.url.DicUrlParam;
import com.persagy.dmp.constant.CommonConstant;
import com.persagy.dmp.dao.OrgDao;
import com.persagy.dmp.entity.User;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author LuoGuangyi
 * @version 1.0.0
 * @Description 人员组织权限
 * @ClassName OrgDaoImpl.java
 * @createTime 2021/01/13 13:03
 * @Since
 */
@Component
@Slf4j
public class OrgDaoImpl implements OrgDao {

    @Autowired
    private OrgServerClient  orgServerClient;

    /**
     * @param dicUrlParam: 中台URL参数
     * @param userIds:     用户Id集合
     * @description: 根据用户id批量查询用户姓名
     * @author: LuoGuangyi
     * @createTime: 2021/01/13 13:19
     * @return: java.util.List<com.persagy.dmp.entity.User>
     * @expression
     */
    @Override
    public List<User> queryUserList(DicUrlParam dicUrlParam, List<String> userIds) throws Exception {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("id", userIds);
        JSONObject param = JSONObject.parseObject(JacksonMapper.toFormatJson(criteria));
        DmpResult<JSONArray> dmpResult = orgServerClient.queryUser(dicUrlParam, param);
        if (dmpResult != null && DmpResult.SUCCESS.equalsIgnoreCase(dmpResult.getResult())) {
            JSONArray dataArray = dmpResult.getData();
            if (CollectionUtil.isNotEmpty(dataArray)) {
                return dataArray.toJavaList(User.class);
            }
        }
        return new ArrayList<>();
    }

    /**
     * @param dicUrlParam: 中台URL参数
     * @param userIds:     用户Id集合
     * @description: 根据用户Id批量查询用户<id, name>的map
     * @author: LuoGuangyi
     * @createTime: 2021/01/13 13:20
     * @return: java.util.Map<java.lang.String, java.lang.String>
     * @expression
     */
    @Override
    public Map<String, String> queryUserMap(DicUrlParam dicUrlParam, List<String> userIds) throws Exception {
        List<User> users = queryUserList(dicUrlParam, userIds);
        Map<String, String> userIdNameMap = users.stream().collect(Collectors.toMap(User::getId, User::getName));
        userIdNameMap.put(CommonConstant.SYSTEM_ID, CommonConstant.SYSTEM_NAME);
        return userIdNameMap;
    }

    /**
     * @param dicUrlParam:
     * @param userIds:
     * @description: 查询用户列表，只保留用户ID和名称
     * @author: LuoGuangyi
     * @createTime: 2021/02/03 15:26
     * @return: java.util.List<com.persagy.dmp.entity.User>
     * @expression
     */
    @Override
    public List<User> queryUserIdNameList(DicUrlParam dicUrlParam, List<String> userIds) throws Exception {
        List<User> users = queryUserList(dicUrlParam, userIds);
        users = users.stream().map(item -> {
            User user = new User();
            user.setId(item.getId());
            user.setName(item.getName());
            return user;
        }).collect(Collectors.toList());
        return users;
    }

}


