package com.persagy.dmp.dto.logicalGrouping;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import com.alibaba.fastjson.JSONObject;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
    * 自定义对象类下的对象组
    */
@ApiModel(value="com-persagy-dmp-entity-LogicalGrouping")
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class LogicalGroupingDTO implements Serializable {
    /**
     * 主键id
     */
    @ApiModelProperty(value="主键id")
    private String logicalGroupingId;

    /**
     * 集团编码
     */
    @ApiModelProperty(value="集团编码")
    private String groupCode;

    /**
     * 项目Id
     */
    @ApiModelProperty(value="项目Id")
    private String projectId;

    /**
     * 父编组id,默认为"0"，0代表顶层
     */
    @ApiModelProperty(value="父编组id,默认为'0'，0代表顶层")
    private String parentId;

    /**
     * 场景
     */
    @ApiModelProperty(value="场景")
    private String ibmsSceneCode;

    /**
     * 中控对象类code
     */
    @ApiModelProperty(value="中控对象类code")
    private String ibmsClassCode;

    /**
     * 编组名称
     */
    @ApiModelProperty(value="编组名称")
    private String logicalGroupingName;

    /**
     * 描述
     */
    @ApiModelProperty(value="描述")
    private String remark;

    /**
     * 一级代码
     */
    @ApiModelProperty(value="一级代码")
    private String firstCode;

    /**
     * 二级代码
     */
    @ApiModelProperty(value="二级代码")
    private String secondCode;

    /**
     * 展示顺序
     */
    @ApiModelProperty(value="展示顺序")
    private Integer sequence;
    /**
     * 对象数量
     */
    @ApiModelProperty(value="对象数量")
    private Integer objNum;

    /**
     * 创建时间:时间格式为yyyyMMddHHmmss
     */
    @ApiModelProperty(value="创建时间:时间格式为yyyyMMddHHmmss")
    private Date createTime;

    /**
     * 更新时间:时间格式为yyyyMMddHHmmss
     */
    @ApiModelProperty(value="更新时间:时间格式为yyyyMMddHHmmss")
    private Date updateTime;

    /**
     * 有效状态(1-有效；0-无效)
     */
    @ApiModelProperty(value="有效状态(1-有效；0-无效)")
    private Integer valid;

    /**
     * 编组扩展字段
     */
    @ApiModelProperty(value="编组扩展字段")
    private JSONObject groupExtra;

    /**
     * 楼层id
     */
    @ApiModelProperty(value="楼层id")
    private String floorId;

    /**
     * 楼层名称
     */
    @ApiModelProperty(value="楼层名称")
    private String floorName;

    /**
     * 楼层序列号
     */
    @ApiModelProperty(value="楼层序列号")
    private Integer floorSequenceId;

    /**
     * 子节点
     */
    @ApiModelProperty(value="子编组")
    private List<LogicalGroupingDTO> children;

    @ApiModelProperty(value="编组对象")
    private List<JSONObject> logicalObjects;
    /**
     * 添加的对象列表
     */
    @ApiModelProperty(value="添加的对象列表")
    private transient List<String> add;

    @ApiModelProperty(value="是否递归查询")
    private transient Boolean recursive = false;
    @ApiModelProperty(value="页码")
    private  Integer page = 1;
    @ApiModelProperty(value="页大小")
    private  Integer size = 1000000;

    @ApiModelProperty(value="关键字")
    private String keyWord;

    @ApiModelProperty(value="关键字")
    private boolean containsNotGrouping = false;
    /**
     * 主键id
     */
    @ApiModelProperty(value="主键id")
    private List<String> logicalGroupingIds;


    /**
     * 照明回路的objId
     */
    @ApiModelProperty(value="对象id")
    private List<String> deleteObjIds;

    /**
     * 照明的一级编组下的二级编组
     */
    @ApiModelProperty(value="照明的一级编组下的二级编组数量")
    private Integer secondLogicNum;

    /** 此一级编组对应的所有模式 */
    private List<JSONObject> models;
    

    private static final long serialVersionUID = 1L;

    public static final String COL_LOGICAL_GROUPING_ID = "logical_grouping_id";

    public static final String COL_GROUP_CODE = "group_code";

    public static final String COL_PROJECT_ID = "project_id";

    public static final String COL_PARENT_ID = "parent_id";

    public static final String COL_IBMS_SCENE_CODE = "ibms_scene_code";

    public static final String COL_IBMS_CLASS_CODE = "ibms_class_code";

    public static final String COL_LOGICAL_GROUPING_NAME = "logical_grouping_name";

    public static final String COL_REMARK = "remark";

    public static final String COL_FIRST_CODE = "first_code";

    public static final String COL_SECOND_CODE = "second_code";

    public static final String COL_SEQUENCE = "sequence";

    public static final String COL_CREATE_TIME = "create_time";

    public static final String COL_UPDATE_TIME = "update_time";

    public static final String COL_VALID = "valid";

    public static final String COL_GROUP_EXTRA = "group_extra";

    public static final String COL_FLOOR_ID = "floor_id";

    public static final String COL_FLOOR_NAME = "floor_name";

    public static final String COL_FLOOR_SEQUENCE_ID = "floor_sequence_id";


    @Override
    public String toString() {
        return JSONObject.toJSONString(this);
    }
}