package com.persagy.dmp.dto.logicalGrouping;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
    * 对象组与数据字典对象关系表
    */
@ApiModel(value="com-persagy-dmp-entity-LogicalObject")
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class LogicalObjectDTO implements Serializable {
    /**
     * 主键id
     */
    @ApiModelProperty(value="主键id")
    private String id;

    /**
     * 集团编码
     */
    @ApiModelProperty(value="集团编码")
    private String groupCode;

    /**
     * 项目Id
     */
    @ApiModelProperty(value="项目Id")
    private String projectId;

    /**
     * 分组id
     */
    @ApiModelProperty(value="分组id")
    private String logicalGroupingId;

    /**
     * 对象id
     */
    @ApiModelProperty(value="对象id")
    private String objId;

    /**
     * 对象类型（building、equipment、component、floor、shaft、space、system）
     */
    @ApiModelProperty(value="对象类型（building、equipment、component、floor、shaft、space、system）")
    private String objType;

    /**
     * 对象名称
     */
    @ApiModelProperty(value="对象名称")
    private String objName;

    /**
     * 创建时间:时间格式为yyyyMMddHHmmss
     */
    @ApiModelProperty(value="创建时间:时间格式为yyyyMMddHHmmss")
    private Date createTime;

    /**
     * 更新时间:时间格式为yyyyMMddHHmmss
     */
    @ApiModelProperty(value="更新时间:时间格式为yyyyMMddHHmmss")
    private Date updateTime;

    /**
     * 扩展字段
     */
    @ApiModelProperty(value="扩展字段")
    private JSONObject groupExtra;


    /**
     * 本地id
     */
    @ApiModelProperty(value="对象本地id")
    private String localId;
    /**
     * 楼层id
     */
    @ApiModelProperty(value="楼层id")
    private String floorId;

    /**
     * 楼层名称
     */
    @ApiModelProperty(value="楼层名称")
    private String floorName;

    /**
     * 楼层序列号
     */
    @ApiModelProperty(value="楼层序列号")
    private Integer floorSequenceId;

    /**
     * 空间id
     */
    @ApiModelProperty(value="空间id")
    private String spaceId;

    /**
     * 空间名称
     */
    @ApiModelProperty(value="空间名称")
    private String spaceName;

    /**
     * 建筑id
     */
    @ApiModelProperty(value="建筑id")
    private String buildId;

    /**
     * 建筑名称
     */
    @ApiModelProperty(value="建筑名称")
    private String buildName;

    /**
     * 有效状态(1-有效；0-无效)
     */
    @ApiModelProperty(value="有效状态(1-有效；0-无效)")
    private transient int valid;

    /**
     * 添加的对象列表
     */
    @ApiModelProperty(value="添加的对象列表")
    private transient List<String> add;

    /**
     * 删除的对象列表
     */
    @ApiModelProperty(value="删除的对象列表")
    private transient List<String> delete;

    /**
     * 更新的对象列表
     */
    @ApiModelProperty(value="更新的对象列表")
    private transient List<String> update;

    /**
     * 更新的对象列表
     */
    @ApiModelProperty(value="更新的对象列表")
    private transient List<LogicalObjectRelationDTO> updateBatch;

    @Builder.Default
    @ApiModelProperty(value="页码")
    private transient Integer page = 1;

    @Builder.Default
    @ApiModelProperty(value="页大小")
    private transient Integer size = 1000;

    @ApiModelProperty(value="编组id集合")
    private List<String> logicalGroupingIds;

    @ApiModelProperty(value="关键字")
    private String keyWord;

    @ApiModelProperty(value="所属模块名称")
    private String belongModularName;

    @ApiModelProperty(value="所属模块名称")
    private String belongPowerBoxName;

    @ApiModelProperty(value="所属电井名称")
    private String belongPowerWellsName;

    @ApiModelProperty(value="所属模块名称")
    private String location;

    private static final long serialVersionUID = 1L;

    public static final String COL_ID = "id";

    public static final String COL_GROUP_CODE = "group_code";

    public static final String COL_PROJECT_ID = "project_id";

    public static final String COL_LOGICAL_GROUPING_ID = "logical_grouping_id";

    public static final String COL_OBJ_ID = "obj_id";

    public static final String COL_OBJ_TYPE = "obj_type";

    public static final String COL_OBJ_NAME = "obj_name";

    public static final String COL_CREATE_TIME = "create_time";

    public static final String COL_UPDATE_TIME = "update_time";

    public static final String COL_GROUP_EXTRA = "group_extra";

    public static final String COL_LOCAL_ID = "local_id";

    public static final String COL_FLOOR_ID = "floor_id";

    public static final String COL_FLOOR_NAME = "floor_name";

    public static final String COL_FLOOR_SEQUENCE_ID = "floor_sequence_id";

    public static final String COL_SPACE_ID = "space_id";

    public static final String COL_SPACE_NAME = "space_name";

    public static final String COL_BUILD_ID = "build_id";

    public static final String COL_BUILD_NAME = "build_name";

    public static final String COL_VALID = "valid";


    @Override
    public String toString() {
        return JSONObject.toJSONString(this);
    }
}