package com.persagy.dmp.entity;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import org.apache.ibatis.type.JdbcType;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.FieldStrategy;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.FastjsonTypeHandler;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
    * 自定义对象类下的对象组
    */
@ApiModel(value="com-persagy-dmp-entity-LogicalGrouping")
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
@TableName(value = "logical_grouping",autoResultMap = true)
public class LogicalGrouping implements Serializable {
    /**
     * 主键id
     */
    @TableId(value = "logical_grouping_id", type = IdType.ASSIGN_ID)
    @ApiModelProperty(value="主键id")
    private String logicalGroupingId;

    /**
     * 集团编码
     */
    @TableField(value = "group_code")
    @ApiModelProperty(value="集团编码")
    private String groupCode;

    /**
     * 项目Id
     */
    @TableField(value = "project_id")
    @ApiModelProperty(value="项目Id")
    private String projectId;

    /**
     * 父编组id,默认为"0"，0代表顶层
     */
    @TableField(value = "parent_id")
    @ApiModelProperty(value="父编组id,默认为'0'，0代表顶层")
    private String parentId;

    /**
     * 场景
     */
    @TableField(value = "ibms_scene_code")
    @ApiModelProperty(value="场景")
    private String ibmsSceneCode;

    /**
     * 中控对象类code
     */
    @TableField(value = "ibms_class_code")
    @ApiModelProperty(value="中控对象类code")
    private String ibmsClassCode;

    /**
     * 编组名称
     */
    @TableField(value = "logical_grouping_name")
    @ApiModelProperty(value="编组名称")
    private String logicalGroupingName;

    /**
     * 描述
     */
    @TableField(value = "remark", updateStrategy = FieldStrategy.NOT_NULL)
    @ApiModelProperty(value="描述")
    private String remark;

    /**
     * 一级代码
     */
    @TableField(value = "first_code")
    @ApiModelProperty(value="一级代码")
    private String firstCode;

    /**
     * 二级代码
     */
    @TableField(value = "second_code")
    @ApiModelProperty(value="二级代码")
    private String secondCode;

    /**
     * 展示顺序
     */
    @TableField(value = "`sequence`")
    @ApiModelProperty(value="展示顺序")
    private Integer sequence;
    /**
     * 对象数量
     */
    @TableField(value = "obj_num")
    @ApiModelProperty(value="对象数量")
    private Integer objNum;

    /**
     * 创建时间:时间格式为yyyyMMddHHmmss
     */
    @TableField(value = "create_time", fill = FieldFill.INSERT)
    @ApiModelProperty(value="创建时间:时间格式为yyyyMMddHHmmss")
    private Date createTime;

    /**
     * 更新时间:时间格式为yyyyMMddHHmmss
     */
    @TableField(value = "update_time", fill = FieldFill.INSERT_UPDATE)
    @ApiModelProperty(value="更新时间:时间格式为yyyyMMddHHmmss")
    @JSONField(format = "yyyyMMddHHmmss")
    private Date updateTime;

    /**
     * 有效状态(1-有效；0-无效)
     */
    @TableField(value = "`valid`", select = false)
    @ApiModelProperty(value="有效状态(1-有效；0-无效)")
    private int valid;

    /**
     * 编组扩展字段
     */
    @TableField(value = "group_extra",jdbcType = JdbcType.VARCHAR, typeHandler = FastjsonTypeHandler.class)
    @ApiModelProperty(value="编组扩展字段")
    private JSONObject groupExtra;

    /**
     * 楼层id
     */
    @TableField(exist = false,value = "floor_id")
    @ApiModelProperty(value="楼层id")
    private String floorId;

    /**
     * 楼层名称
     */
    @TableField(exist = false,value = "floor_name")
    @ApiModelProperty(value="楼层名称")
    private String floorName;

    /**
     * 楼层序列号
     */
    @TableField(exist = false,value = "floor_sequence_id")
    @ApiModelProperty(value="楼层序列号")
    private Integer floorSequenceId;

    /** 此一级编组对应的所有模式 */
    @TableField(exist = false)
    private List<JSONObject> models;
    
    /** 是否默认被选中，true-是，false-否 */
    @TableField(exist = false)
    private Boolean select;
    
    private static final long serialVersionUID = 1L;

    public static final String COL_LOGICAL_GROUPING_ID = "logical_grouping_id";

    public static final String COL_GROUP_CODE = "group_code";

    public static final String COL_PROJECT_ID = "project_id";

    public static final String COL_PARENT_ID = "parent_id";

    public static final String COL_IBMS_SCENE_CODE = "ibms_scene_code";

    public static final String COL_IBMS_CLASS_CODE = "ibms_class_code";

    public static final String COL_LOGICAL_GROUPING_NAME = "logical_grouping_name";

    public static final String COL_REMARK = "remark";

    public static final String COL_FIRST_CODE = "first_code";

    public static final String COL_SECOND_CODE = "second_code";

    public static final String COL_SEQUENCE = "sequence";

    public static final String COL_OBJ_NUM = "obj_num";

    public static final String COL_CREATE_TIME = "create_time";

    public static final String COL_UPDATE_TIME = "update_time";

    public static final String COL_VALID = "valid";

    public static final String COL_GROUP_EXTRA = "group_extra";

    public static final String COL_FLOOR_ID = "floor_id";

    public static final String COL_FLOOR_NAME = "floor_name";

    public static final String COL_FLOOR_SEQUENCE_ID = "floor_sequence_id";

    @Override
    public String toString() {
        return JSONObject.toJSONString(this);
    }

}