package com.persagy.dmp.entity;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.handlers.FastjsonTypeHandler;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.ibatis.type.JdbcType;

import java.io.Serializable;
import java.util.Date;

/**
    * 对象组与数据字典对象关系表
    */
@ApiModel(value="com-persagy-dmp-entity-LogicalObject")
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
@TableName(value = "logical_object",autoResultMap = true)
public class LogicalObject implements Serializable {
    /**
     * 主键id
     */
    @TableId(value = "id", type = IdType.ASSIGN_ID)
    @ApiModelProperty(value="主键id")
    private String id;

    /**
     * 集团编码
     */
    @TableField(value = "group_code")
    @ApiModelProperty(value="集团编码")
    private String groupCode;

    /**
     * 项目Id
     */
    @TableField(value = "project_id")
    @ApiModelProperty(value="项目Id")
    private String projectId;

    /**
     * 分组id
     */
    @TableField(value = "logical_grouping_id")
    @ApiModelProperty(value="分组id")
    private String logicalGroupingId;

    /**
     * 对象id
     */
    @TableField(value = "obj_id")
    @ApiModelProperty(value="对象id")
    private String objId;

    /**
     * 对象类型（building、equipment、component、floor、shaft、space、system）
     */
    @TableField(value = "obj_type")
    @ApiModelProperty(value="对象类型（building、equipment、component、floor、shaft、space、system）")
    private String objType;

    /**
     * 对象名称
     */
    @TableField(value = "obj_name")
    @ApiModelProperty(value="对象名称")
    private String objName;

    /**
     * 创建时间:时间格式为yyyyMMddHHmmss
     */
    @TableField(value = "create_time", fill = FieldFill.INSERT)
    @ApiModelProperty(value="创建时间:时间格式为yyyyMMddHHmmss")
    private Date createTime;

    /**
     * 更新时间:时间格式为yyyyMMddHHmmss
     */
    @TableField(value = "update_time", fill = FieldFill.INSERT_UPDATE)
    @ApiModelProperty(value="更新时间:时间格式为yyyyMMddHHmmss")
    @JSONField(format = "yyyyMMddHHmmss")
    private Date updateTime;

    /**
     * 有效状态(1-有效；0-无效)
     */
    @TableField(value = "`valid`", select = false)
    @ApiModelProperty(value="有效状态(1-有效；0-无效)")
    private int valid;

    /**
     * 扩展字段
     */
    @TableField(value = "group_extra", jdbcType = JdbcType.VARCHAR, typeHandler = FastjsonTypeHandler.class)
    @ApiModelProperty(value="扩展字段")
    private JSONObject groupExtra;

    /**
     * 对象本地id
     */
    @TableField(value = "local_id")
    @ApiModelProperty(value="本地id")
    private String localId;
    /**
     * 楼层id
     */
    @TableField(value = "floor_id")
    @ApiModelProperty(value="楼层id")
    private transient String floorId;

    /**
     * 楼层名称
     */
    @TableField(value = "floor_name")
    @ApiModelProperty(value="楼层名称")
    private transient String floorName;

    /**
     * 楼层序列号
     */
    @TableField(exist = false,value = "floor_sequence_id")
    @ApiModelProperty(value="楼层序列号")
    private Integer floorSequenceId;

    /**
     * 空间id
     */
    @TableField(exist = false,value = "space_id")
    @ApiModelProperty(value="空间id")
    private String spaceId;

    /**
     * 空间名称
     */
    @TableField(exist = false,value = "space_name")
    @ApiModelProperty(value="空间名称")
    private String spaceName;

    /**
     * 建筑id
     */
    @TableField(exist = false,value = "build_id")
    @ApiModelProperty(value="建筑id")
    private String buildId;

    /**
     * 建筑名称
     */
    @TableField(exist = false,value = "build_name")
    @ApiModelProperty(value="建筑名称")
    private String buildName;


    private static final long serialVersionUID = 1L;

    public static final String COL_ID = "id";

    public static final String COL_GROUP_CODE = "group_code";

    public static final String COL_PROJECT_ID = "project_id";

    public static final String COL_LOGICAL_GROUPING_ID = "logical_grouping_id";

    public static final String COL_OBJ_ID = "obj_id";

    public static final String COL_OBJ_TYPE = "obj_type";

    public static final String COL_OBJ_NAME = "obj_name";

    public static final String COL_CREATE_TIME = "create_time";

    public static final String COL_UPDATE_TIME = "update_time";

    public static final String COL_GROUP_EXTRA = "group_extra";

    public static final String COL_LOCAL_ID = "local_id";

    public static final String COL_FLOOR_ID = "floor_id";

    public static final String COL_FLOOR_NAME = "floor_name";

    public static final String COL_FLOOR_SEQUENCE_ID = "floor_sequence_id";

    public static final String COL_SPACE_ID = "space_id";

    public static final String COL_SPACE_NAME = "space_name";

    public static final String COL_BUILD_ID = "build_id";

    public static final String COL_BUILD_NAME = "build_name";

    public static final String COL_VALID = "valid";

    @Override
    public String toString() {
        return JSONObject.toJSONString(this);
    }

}