package com.persagy.dmp.entity;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.handlers.FastjsonTypeHandler;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.ibatis.type.JdbcType;

import java.util.Date;

/**
 * @author : ZhangWenTao
 * @version : 1.0
 * @since : 2022/7/13 10:57
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@TableName("logical_object_equip")
public class LogicalObjectEquip {

    /**
     * 主键id
     */
    @TableId(value = "id", type = IdType.ASSIGN_ID)
    private String id;

    /**
     * 集团编码
     */
    @TableField(value = "group_code")
    private String groupCode;

    /**
     * 项目Id
     */
    @TableField(value = "project_id")
    private String projectId;

    /**
     * 场景码code
     */
    @TableField(value = "ibms_scene_code")
    private String ibmsSceneCode;

    /**
     * 中控对象类code
     */
    @TableField(value = "ibms_class_code")
    private String ibmsClassCode;

    /**
     * 对象id
     */
    @TableField(value = "obj_id")
    private String objId;

    /**
     * 对象类型（building、equipment、component、floor、shaft、space、system）
     */
    @TableField(value = "obj_type")
    private String objType;

    /**
     * 对象名称
     */
    @TableField(value = "obj_name")
    private String objName;

    /**
     * 创建时间:时间格式为yyyyMMddHHmmss
     */
    @TableField(value = "create_time", fill = FieldFill.INSERT)
    private Date createTime;

    /**
     * 更新时间:时间格式为yyyyMMddHHmmss
     */
    @TableField(value = "update_time", fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    /**
     * 有效状态(1-有效；0-无效)
     */
    @TableField(value = "`valid`", select = false)
    @ApiModelProperty(value = "有效状态(1-有效；0-无效)")
    private Integer valid;

    /**
     * 扩展字段
     */
    @TableField(value = "group_extra", jdbcType = JdbcType.VARCHAR, typeHandler = FastjsonTypeHandler.class)
    private JSONObject groupExtra;

    /**
     * 楼层id
     */
    @TableField(value = "floor_id")
    private transient String floorId;

    /**
     * 楼层名称
     */
    @TableField(value = "floor_name")
    private transient String floorName;

    /**
     * 楼层序列号
     */
    @TableField(exist = false, value = "floor_sequence_id")
    private Integer floorSequenceId;

    /**
     * 空间id
     */
    @TableField(exist = false, value = "space_id")
    private String spaceId;

    /**
     * 空间名称
     */
    @TableField(exist = false, value = "space_name")
    private String spaceName;

    /**
     * 建筑id
     */
    @TableField(exist = false, value = "build_id")
    private String buildId;

    /**
     * 建筑名称
     */
    @TableField(exist = false, value = "build_name")
    private String buildName;

    /**
     * 对象本地id
     */
    @TableField(value = "local_id")
    private String localId;

    /**
     * 照明位置
     */
    @TableField(value = "location")
    private String location;

    /**
     * 所属模块ID
     */
    @TableField(value = "modular_id")
    private String modularId;

    /**
     * 所属模块名称
     */
    @TableField(value = "modular_name")
    private String modularName;

    /**
     * 所属配电箱ID
     */
    @TableField(value = "power_box_id")
    private String powerBoxId;

    /**
     * 所属配电箱名称
     */
    @TableField(value = "power_box_name")
    private String powerBoxName;

    /**
     * 所属电井ID
     */
    @TableField(value = "power_wells_id")
    private String powerWellsId;

    /**
     * 所属电井名称
     */
    @TableField(value = "power_wells_name")
    private String powerWellsName;

}
