package com.persagy.dmp.operatorlog;

import javax.servlet.http.HttpServletRequest;

import com.persagy.dmp.auth.AuthManager;
import com.persagy.dmp.entity.JWTUserDto;
import com.persagy.dmp.utils.RequestUtil;

import cn.hutool.core.util.StrUtil;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

/**
 * @author xuhuiyang
 * @Date 2022-04-14
 */
@Setter
@Getter
@NoArgsConstructor
public class OperatorLogDto {

    private String logSource;
    /**
     * 集团code
     */
    private String groupCode;
    /**
     * 项目id
     */
    private String projectId;
    /**
     * 类型
     */
    private String systemType;
    /**
     * 用户id，logType为user时需要
     */
    private String userId;
    /**
     * 用户名，logType为user时需要
     */
    private String userName;
    /**
     * 角色名，logType为user时需要
     */
    private String roleName;
    /**
     * 日志详情，详见：https://thoughts.teambition.com/workspaces/60812f6205669300474fb287/docs/60ffcea641cef60001c20610
     */
    private String details;
    /**
     * 系统,产品模块，logType为sys时需要
     */
    private String module;
    /**
     * ibms产品模块编码
     */
    private String ibmsSceneCode;

    /**
     * ibms产品模块名称
     */
    private String ibmsSceneName;

    /**
     * ibms设备类型，logType为sys时需要
     */
    private String ibmsClassCode;
    /**
     * ibms设备类型名称，logType为sys时需要
     */
    private String ibmsClassName;

    /**
     * 对象编码
     */
    private String classCode;
    /**
     * 设备名称，logType为sys时需要
     */
    private String deviceName;
    /**
     * 设备编码，logType为sys时需要
     */
    private String deviceCode;
    /**
     * 空间id，logType为sys时需要
     */
    private String spaceId;
    /**
     * 楼层，logType为sys时需要
     */
    private String floor;
    /**
     * IP地址，logType为sys时需要
     */
    private String ipAddress;
    /**
     * 扩展字段，存储其它信息，json格式，如有
     */
    private String extraData;

    /**
     * 用户token信息
     */
    //private String token;


    public OperatorLogDto(HttpServletRequest request, String groupCode, String projectId, String userId, String logSource, String systemType, String module, String details) {
        this.groupCode = groupCode;
        this.projectId = projectId;
        this.userId = userId;
        this.logSource = logSource;
        this.systemType = systemType;
        this.module = module;
        this.details = details;
        this.filleUserInfo(request);
    }


    private void filleUserInfo(HttpServletRequest request) {
        JWTUserDto userDto = AuthManager.currentUser();
        if (userDto != null) {
            this.userId = userDto.getUserId();
            this.userName = userDto.getName();
            this.roleName = userDto.getRoleName();
            this.ipAddress = userDto.getIp();
        }
        if (StrUtil.isBlank(ipAddress) && request != null) {
        	this.ipAddress = RequestUtil.getIpAddress(request);
		}
    }

    public void fillIbmsInfo(String ibmsClassCode, String ibmsClassName, String ibmsSceneCode, String ibmsSceneName) {
        this.ibmsClassCode = ibmsClassCode;
        this.ibmsClassName = ibmsClassName;
        this.ibmsSceneCode = ibmsSceneCode;
        this.ibmsSceneName = ibmsSceneName;
    }

    public void fillDeviceInfo(String deviceCode, String deviceName, String spaceId, String floorId) {
        this.deviceCode = deviceCode;
        this.deviceName = deviceName;
        this.spaceId = spaceId;
        this.floor = floorId;
    }

}
