package com.persagy.dmp.request;

import com.persagy.dmp.dto.logicalGrouping.LogicalObjectTabularDTO;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.NotBlank;
import java.util.List;

/**
 * 逻辑对象查询参数
 *
 * @author : ZhangWenTao
 * @version : 1.0
 * @since : 2022/7/12 15:15
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class QueryLogicalObjectTabularRequest {

    /**
     * 名称/编码/位置的关键字进行模糊搜索
     */
    @ApiModelProperty("名称/编码/位置的关键字进行模糊搜索")
    private String keyword;

    /**
     * 筛选条件-逻辑编组
     */
    @ApiModelProperty("筛选条件-逻辑编组")
    private List<String> logicalGroupingIds;

    /**
     * 是否只查询所期望分组的relations数据{@link LogicalObjectTabularDTO#getRelations()}: true-是 false-否  默认: false
     */
    @ApiModelProperty("是否只查询所期望分组的relations数据: true-是 false-否  默认: false")
    private boolean onlyExpectGrouping;

    /**
     * 集团编码
     */
    @NotBlank
    @ApiModelProperty(value = "集团编码", required = true)
    private String groupCode;

    /**
     * 项目Id
     */
    @NotBlank
    @ApiModelProperty(value = "项目Id", required = true)
    private String projectId;

    /**
     * 产品模块编码
     */
    @NotBlank
    @ApiModelProperty(value = "产品模块编码", required = true)
    private String ibmsSceneCode;

    /**
     * 管理对象类型编码
     */
    @NotBlank
    @ApiModelProperty(value = "管理对象类型编码", required = true)
    private String ibmsClassCode;

    /**
     * 楼栋ID
     */
    @ApiModelProperty("楼栋ID")
    private List<String> buildIds;

    /**
     * 楼层ID
     */
    @ApiModelProperty("楼层ID")
    private List<String> floorIds;

    /**
     * 所属模块名称
     */
    @ApiModelProperty("所属模块名称")
    private List<String> modularNames;

    /**
     * 所属配电箱名称
     */
    @ApiModelProperty("所属配电箱名称")
    private List<String> powerBoxNames;


    /**
     * 所属电井名称
     */
    @ApiModelProperty("所属电井名称")
    private List<String> powerWellsNames;

    /**
     * 是否只查询未分组对象, 为true时logicalGroupingIds参数无效: true-是 false-否  默认: false
     */
    @ApiModelProperty("是否只查询未分组对象: true-是 false-否  默认: false")
    private boolean onlyNotGrouping;

    /**
     * 是否排序(按照楼栋名称首字母、楼层从高到低进行排序): true-是 false-否  默认: false
     */
    @ApiModelProperty("是否排序(按照楼栋名称首字母、楼层从高到低进行排序): true-是 false-否  默认: false")
    private boolean sort;

    /**
     * 页码
     */
    @Builder.Default
    @ApiModelProperty("页码")
    private int page = 1;

    /**
     * 分页条数 小于1不进行分页
     */
    @Builder.Default
    @ApiModelProperty("分页条数: 小于1不进行分页")
    private int size = -1;

}
