package com.persagy.dmp.request;

import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.NotBlank;
import java.util.List;

/**
 * 保存或更新逻辑编组请求参数
 *
 * @author : ZhangWenTao
 * @version : 1.0
 * @since : 2022/7/18 15:35
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class UpsertLogicalGroupingRequest {

    /**
     * 集团编码
     */
    @NotBlank
    @ApiModelProperty(value = "集团编码", required = true)
    private String groupCode;

    /**
     * 项目Id
     */
    @NotBlank
    @ApiModelProperty(value = "项目Id", required = true)
    private String projectId;

    /**
     * 产品模块编码
     */
    @NotBlank
    @ApiModelProperty(value = "产品模块编码", required = true)
    private String ibmsSceneCode;

    /**
     * 管理对象类型编码
     */
    @NotBlank
    @ApiModelProperty(value = "管理对象类型编码", required = true)
    private String ibmsClassCode;

    /**
     * 逻辑分组ID, 不为空时执行更新
     */
    @ApiModelProperty("逻辑分组ID, 不为空时执行更新")
    private String logicalGroupingId;

    /**
     * 父逻辑分组ID, 根节点可以不传
     */
    @ApiModelProperty("父逻辑分组ID, 根节点可以不传")
    private String parentLogicalGroupingId;

    /**
     * 编组名称
     */
    @NotBlank
    @ApiModelProperty(value = "编组名称", required = true)
    private String logicalGroupingName;

    /**
     * 描述
     */
    @ApiModelProperty("描述")
    private String remark;

    /**
     * 一级代码
     */
    @ApiModelProperty(value = "一级代码")
    private String firstCode;

    /**
     * 二级代码
     */
    @ApiModelProperty(value = "二级代码")
    private String secondCode;

    /**
     * 楼层id
     */
    @ApiModelProperty("楼层id")
    private String floorId;

    /**
     * 对象ID列表
     */
    @ApiModelProperty("对象ID列表")
    private List<String> objIds;

}
