package com.persagy.dmp.service;

import com.persagy.dmp.dto.CurrentDataDTO;
import com.persagy.dmp.dto.DataQueryDTO;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * @description: 实时数据接口
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/9/22 15:15
 * @version: V1.0
 **/
public interface CurrentDataService {

    /**
     * @description: 查询实时数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCode 信息点编码
     * @return: com.persagy.common.dto.CurrentDataDTO
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:12
     * @version: V1.0
     */
    CurrentDataDTO queryCurrentData(String groupCode, String projectId, String objectId, String infoCode) throws Exception;

    /**
     * @description: 查询实时数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIdList 对象列表
     * @param: infoCode 信息点编码
     * @return: java.util.Map<java.lang.String, java.math.BigDecimal>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/3/30 18:44
     * @version: V1.0
     */
    Map<String, BigDecimal> queryCurrentDataMap(String groupCode, String projectId, List<String> objectIdList, String infoCode) throws Exception;

    /**
     * @description: 查询实时数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIdList 对象列表
     * @param: infoCode 信息点编码
     * @return: java.util.Map<java.lang.String, java.math.BigDecimal>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/3/30 18:44
     * @version: V1.0
     */
    Map<String, BigDecimal> queryCurrentDataMap(String groupCode, String projectId, String objectId, List<String> infoCodes) throws Exception;

    /**
     * @description: 查询实时数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIdList 对象列表
     * @param: infoCode 信息点编码
     * @return: java.util.List<com.persagy.common.dto.CurrentDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:12
     * @version: V1.0
     */
    List<CurrentDataDTO> queryCurrentDataList(String groupCode, String projectId, List<String> objectIdList, String infoCode) throws Exception;

    /**
     * @description: 查询实时数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCodeList 信息点编码列表
     * @return: java.util.List<com.persagy.common.dto.CurrentDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:11
     * @version: V1.0
     */
    List<CurrentDataDTO> queryCurrentDataList(String groupCode, String projectId, String objectId, List<String> infoCodeList) throws Exception;

    /**
     * @description: 查询实时数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: dataQueryDtoList 实时数据参数（对象id、信息点）列表
     * @return: java.util.List<com.persagy.common.dto.CurrentDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:11
     * @version: V1.0
     */
    List<CurrentDataDTO> queryCurrentDataList(String groupCode, String projectId, List<DataQueryDTO> dataQueryDTOList) throws Exception;

    /**
     * @description: 查询控制数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCode 信息点编码
     * @return: com.persagy.common.dto.CurrentDataDTO
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:12
     * @version: V1.0
     */
    CurrentDataDTO queryControlData(String groupCode, String projectId, String objectId, String infoCode) throws Exception;


    /**
     * @description: 查询控制数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIdList 对象列表
     * @param: infoCode 信息点编码
     * @return: java.util.Map<java.lang.String, java.math.BigDecimal>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/3/30 18:44
     * @version: V1.0
     */
    Map<String, BigDecimal> queryControlDataMap(String groupCode, String projectId, List<String> objectIdList, String infoCode) throws Exception;

    /**
     * @description: 查询控制数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIdList 对象列表
     * @param: infoCode 信息点编码
     * @return: java.util.List<com.persagy.common.dto.CurrentDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:12
     * @version: V1.0
     */
    List<CurrentDataDTO> queryControlDataList(String groupCode, String projectId, List<String> objectIdList, String infoCode) throws Exception;

    /**
     * @description: 查询控制数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCodeList 信息点编码列表
     * @return: java.util.List<com.persagy.common.dto.CurrentDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:11
     * @version: V1.0
     */
    List<CurrentDataDTO> queryControlDataList(String groupCode, String projectId, String objectId, List<String> infoCodeList) throws Exception;

    /**
     * @description: 查询控制数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: dataQueryDTOList 控制参数（对象id、信息点）列表
     * @return: java.util.List<com.persagy.dmp.dto.CurrentDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/3/24 18:21
     * @version: V1.0
     */
    List<CurrentDataDTO> queryControlDataList(String groupCode, String projectId, List<DataQueryDTO> dataQueryDTOList) throws Exception;
}


