package com.persagy.dmp.service;

import com.persagy.dmp.dto.CategoryDTO;
import com.persagy.dmp.dto.FuncIdQueryDTO;
import com.persagy.dmp.dto.InfoPointDictDTO;

import java.util.List;
import java.util.Map;

/**
 * @description: 数据字典接口
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/9/22 15:10
 * @version: V1.0
 **/
public interface DictService {


    /**
     * @description: 获取数据字典平台级别信息点属性信息
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: classCode 对象类型编码
     * @return: java.util.Map<java.lang.String, com.persagy.dmp.dto.InfoPointDictDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/22 9:34
     * @version: V1.0
     */
    Map<String, InfoPointDictDTO> queryInfoPointDictMap(String groupCode, String projectId, String classCode) throws Exception;

    /***
     * @description: 获取数据字典平台级别信息点属性信息
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: classCode  对象类型编码
     * @return: java.util.List<com.persagy.huiyun.dto.InfoPointDictDTO>
     * @exception:
     * @author: fenghanchao
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/6/5 17:14
     * @version: V1.0
     */
    List<InfoPointDictDTO> queryInfoPointDictList(String groupCode, String projectId, String classCode) throws Exception;

    /**
     * @description: 获取数据字典信息点属性信息
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: funcIdQueryDto 查询信息点参数类
     * @return: java.util.List<com.persagy.common.dto.InfoPointDictDTO>
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/29 10:56
     * @version: V1.0
     */
    List<InfoPointDictDTO> queryInfoPointDictList(String groupCode, String projectId, FuncIdQueryDTO funcIdQueryDto) throws Exception;

    /**
     * @description: 查询设备类型列表
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @return: java.util.List<com.persagy.dmp.dto.CategoryDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/24 11:17
     * @version: V1.0
     */
    List<CategoryDTO> queryCategoryList(String groupCode, String projectId) throws Exception;

    /**
     * @description: 查询专业
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @return: java.util.Map<java.lang.String, java.lang.String>
     * @exception:
     * @author: shiliqiang
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/11 17:42
     * @version: V1.0
     */
    Map<String, String> queryMajorMap(String groupCode, String projectId) throws Exception;
}
