package com.persagy.dmp.service;

import com.persagy.dmp.dto.DataQueryDTO;
import com.persagy.dmp.dto.HistoryDataDTO;

import java.util.List;

/**
 * @description: 历史数据接口
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/9/22 15:15
 * @version: V1.0
 **/
public interface HistoryDataService {

    /**
     * @description: 查询历史数据/分精度数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCode 信息点编码
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 时间间隔
     * @return: com.persagy.common.dto.HistoryDataDTO
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:06
     * @version: V1.0
     */
    HistoryDataDTO queryHistoryDataList(String groupCode, String projectId, String objectId, String infoCode, String startTime, String endTime, String period) throws Exception;

    /**
     * @description: 查询历史数据/分精度数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectId 对象id
     * @param: infoCodeList 信息点编码列表
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 时间间隔
     * @return: java.util.List<com.persagy.common.dto.HistoryDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:06
     * @version: V1.0
     */
    List<HistoryDataDTO> queryHistoryDataList(String groupCode, String projectId, String objectId, List<String> infoCodeList, String startTime, String endTime, String period) throws Exception;

    /**
     * @description: 查询历史数据/分精度数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIdList 对象id列表
     * @param: infoCode 信息点编码
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 时间间隔
     * @return: java.util.List<com.persagy.common.dto.HistoryDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:05
     * @version: V1.0
     */
    List<HistoryDataDTO> queryHistoryDataList(String groupCode, String projectId, List<String> objectIdList, String infoCode, String startTime, String endTime, String period) throws Exception;

    /**
     * @description: 查询历史数据/分精度数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: dataQueryDtoList 查询历史数据的对象id、信息点信息点编码类列表
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 时间间隔
     * @return: java.util.List<com.persagy.common.dto.HistoryDataDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/8/30 20:04
     * @version: V1.0
     */
    List<HistoryDataDTO> queryHistoryDataList(String groupCode, String projectId, List<DataQueryDTO> dataQueryDTOList, String startTime, String endTime, String period) throws Exception;
}
