package com.persagy.dmp.service;

import com.persagy.dmp.dto.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description: 位置接口
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/9/22 15:43
 * @version: V1.0
 **/
public interface LocationService {

    /**
     * @description: 查询设备位置
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: equipId 设备id
     * @return: com.persagy.common.dto.EquipLocationDTO
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:47
     * @version: V1.0
     */
    EquipLocationDTO queryEquipLocation(String groupCode, String projectId, String equipId) throws Exception;

    /**
     * @description: 查询设备位置
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: equipIds 设备id列表
     * @return: java.util.List<com.persagy.common.dto.EquipLocationDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:47
     * @version: V1.0
     */
    List<EquipLocationDTO> queryEquipLocation(String groupCode, String projectId, List<String> equipIds) throws Exception;

    /**
     * @description: 查询对象所属模块
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: equipId 设备id
     * @return: java.util.List<com.persagy.common.dto.EquipLocationDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:47
     * @version: V1.0
     */
    EquipBelongModularDTO queryEquipModularBox(String groupCode, String projectId, String equipId) throws Exception;

    /**
     * @description: 查询对象所属模块
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: equipIds 设备id列表
     * @return: java.util.List<com.persagy.common.dto.EquipLocationDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:47
     * @version: V1.0
     */
    HashMap<String,EquipBelongModularDTO> queryEquipModularBox(String groupCode, String projectId, List<String> equipIds) throws Exception;

    /**
     * @description: 查询设备位置
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: equipIds 设备id列表
     * @return: java.util.Map<java.lang.String, com.persagy.dmp.dto.EquipLocationDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:47
     * @version: V1.0
     */
    Map<String, EquipLocationDTO> queryEquipLocationMap(String groupCode, String projectId, List<String> equipIds) throws Exception;

    /**
     * @description: 查询系统位置
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: systemId 系统id
     * @return: com.persagy.common.dto.SystemLocationDTO
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:47
     * @version: V1.0
     */
    SystemLocationDTO querySystemLocation(String groupCode, String projectId, String systemId) throws Exception;

    /**
     * @description: 查询系统位置
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: systemIds 系统id列表
     * @return: java.util.List<com.persagy.common.dto.SystemLocationDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:47
     * @version: V1.0
     */
    List<SystemLocationDTO> querySystemLocation(String groupCode, String projectId, List<String> systemIds) throws Exception;

    /**
     * @description: 查询系统位置
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: systemIds 系统id列表
     * @return: java.util.Map<java.lang.String, com.persagy.dmp.dto.EquipLocationDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:47
     * @version: V1.0
     */
    Map<String, SystemLocationDTO> querySystemLocationMap(String groupCode, String projectId, List<String> systemIds) throws Exception;

    /**
     * @description: 查询空间位置
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: spaceId 空间id
     * @return: com.persagy.dmp.dto.SpaceLocation
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/23 17:04
     * @version: V1.0
     */
    SpaceLocationDTO querySpaceLocation(String groupCode, String projectId, String spaceId) throws Exception;

    /**
     * @description: 查询空间位置
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: spaceIds 空间id列表
     * @return: java.util.List<com.persagy.dmp.dto.SpaceLocation>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/23 17:04
     * @version: V1.0
     */
    List<SpaceLocationDTO> querySpaceLocation(String groupCode, String projectId, List<String> spaceIds) throws Exception;

    /**
     * @description: 查询空间位置
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: spaceIds 空间id列表
     * @return: java.util.Map<java.lang.String, com.persagy.dmp.dto.SpaceLocationDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/23 17:04
     * @version: V1.0
     */
    Map<String, SpaceLocationDTO> querySpaceLocationMap(String groupCode, String projectId, List<String> spaceIds) throws Exception;

    Map<String, LocationDTO> queryObjLocation(String groupCode, String projectId, List<String> objIds) throws Exception;
}