package com.persagy.dmp.service;

import com.persagy.dmp.entity.LogicalObjectEquipCached;

import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author : ZhangWenTao
 * @version : 1.0
 * @since : 2022/7/14 17:14
 */
public interface LogicalObjectEquipCachedService {

    /**
     * 查询唯一的缓存记录
     *
     * @param groupCode     分组Code
     * @param projectId     项目ID
     * @param ibmsSceneCode 场景码code
     * @param ibmsClassCode 中控对象类code
     * @return {@link LogicalObjectEquipCached}
     */
    LogicalObjectEquipCached getOne(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode);

    /**
     * 查询最后更新时间到当前时间大于期望时间间隔的所有数据
     *
     * @param interval 时间数值
     * @param unit     时间单位
     * @return {@link LogicalObjectEquipCached}
     */
    List<LogicalObjectEquipCached> listByLastUpdateTimeInterval(long interval, TimeUnit unit);

    /**
     * 保存或更新
     *
     * @param groupCode     分组Code
     * @param projectId     项目ID
     * @param ibmsSceneCode 场景码code
     * @param ibmsClassCode 中控对象类code
     */
    void saveOrUpdate(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode);

    /**
     * 查询未加载的缓存对象
     *
     * @param groupCode     分组Code
     * @param projectId     项目ID
     * @param ibmsSceneCode 场景码code
     * @param ibmsClassCode 中控对象类code
     * @return {@link LogicalObjectEquipCached}
     */
    List<LogicalObjectEquipCached> listUnloadEquipCached(
            String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode);

}
