package com.persagy.dmp.service;

/**
 * @author : ZhangWenTao
 * @version : 1.0
 * @since : 2022/7/13 11:06
 */
public interface LogicalObjectEquipService {

    /**
     * 加载设备数据到本地数据库表中, 如果数据已经加载过, 则直接返回, 本方法是线程安全的
     *
     * @param groupCode     分组Code
     * @param projectId     项目ID
     * @param ibmsSceneCode 场景码code
     * @param ibmsClassCode 中控对象类code
     */
    void loadIfAbsent(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode);

    /**
     * 加载设备数据到本地数据库表中, 本方法是线程安全的
     *
     * @param groupCode     分组Code
     * @param projectId     项目ID
     * @param ibmsSceneCode 场景码code
     * @param ibmsClassCode 中控对象类code
     * @param waitFor       如果不是当前线程进行加载, 是否等候其他线程加载完成再返回
     */
    void load(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode, boolean waitFor);

    /**
     * 根据参数下的所有逻辑编组加载数据, 加载完成返回true, 否则返回false
     *
     * @param groupCode     分组Code
     * @param projectId     项目ID
     * @param ibmsSceneCode 场景码code
     * @param ibmsClassCode 中控对象类code
     * @return 加载完成返回true, 否则返回false
     */
    boolean tryLoadIfAbsentWithGroupings(String groupCode, String projectId, String ibmsSceneCode,  String ibmsClassCode);

}
