package com.persagy.dmp.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.persagy.dmp.dto.LogicalObjectLightingSelectsDTO;
import com.persagy.dmp.dto.logicalGrouping.LogicalObjectDTO;
import com.persagy.dmp.dto.logicalGrouping.LogicalObjectTabularDTO;
import com.persagy.dmp.request.*;

import java.util.List;

/**
 * @author : ZhangWenTao
 * @version : 1.0
 * @since : 2022/7/15 16:00
 */
public interface LogicalObjectServiceV2 {

    /**
     * 列表查询逻辑编组对象数据
     *
     * @param request {@link QueryLogicalObjectTabularRequest}
     * @return {@link LogicalObjectTabularDTO}
     */
    IPage<LogicalObjectTabularDTO> queryLogicalObjectTabular(QueryLogicalObjectTabularRequest request);

    /**
     * 列表查询逻辑编组对象数据
     *
     * @param request {@link QueryLogicalObjectTabularRequest}
     * @return {@link LogicalObjectTabularDTO}
     */
    List<LogicalObjectTabularDTO> queryAllLogicalObjectTabular(QueryLogicalObjectTabularRequest request);

    /**
     * 按需查询照明逻辑编组所需的下拉数据
     *
     * @param request {@link LogicalObjectLightingSelectsRequest}
     * @return {@link LogicalObjectLightingSelectsDTO}
     */
    LogicalObjectLightingSelectsDTO queryAllLogicalObjectLightingSelects(LogicalObjectLightingSelectsRequest request);

    /**
     * {@link LogicalObjectService#listLogicalObjectDTO(LogicalObjectDTO)}
     *
     * @param request {@link ListLogicalObjectRequest}
     * @return {@link LogicalObjectDTO}
     */
    List<LogicalObjectDTO> listLogicalObjectDTO(ListLogicalObjectRequest request);

    /**
     * 批量覆盖逻辑对象绑定关系
     *
     * @param request {@link CoverBindLogicalObjectsRequest}
     */
    void coverBindLogicalObjects(CoverBindLogicalObjectsRequest request);

    /**
     * 保存逻辑编组下的对象关系
     *
     * @param groupCode         分组Code
     * @param projectId         项目ID
     * @param ibmsSceneCode     场景码code
     * @param ibmsClassCode     中控对象类code
     * @param logicalGroupingId 逻辑编组ID
     * @param objIds            对象ID
     */
    void saveBindLogicalObjects(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode,
                                String logicalGroupingId, List<String> objIds);


    /**
     * 删除逻辑编组下所有绑定的对象
     *
     * @param groupCode         分组Code
     * @param projectId         项目ID
     * @param ibmsSceneCode     场景码code
     * @param ibmsClassCode     中控对象类code
     * @param logicalGroupingId 逻辑编组ID
     */
    void removeBindLogicalObjects(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode,
                                  String logicalGroupingId);

    /**
     * 逻辑对象设备数量
     *
     * @param groupCode     分组Code
     * @param projectId     项目ID
     * @param ibmsSceneCode 场景码code
     * @param ibmsClassCode 中控对象类code
     * @return int
     */
    int countOfAllLogicalObjectEquip(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode);


    /**
     * 解绑逻辑编组下指定对象
     *
     * @param request {@link UnBindLogicalObjectsRequest}
     */
    void unBindLogicalObjects(UnBindLogicalObjectsRequest request);

}
