package com.persagy.dmp.service;

import com.persagy.dmp.dto.CurrentDataDTO;
import com.persagy.dmp.dto.DataTimeDTO;

import java.math.BigDecimal;
import java.util.List;

/**
 * @description: 统计相关接口
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/10/9 20:43
 * @version: V1.0
 **/
public interface ObjectStatisticsService {

    /**
     * @description: 计算对象列表下同一信息点的采集平均值
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIds 对象列表id
     * @param: infoCode  信息点(仅支持已绑点的动态信息点)
     * @return: BigDecimal
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/24 10:57
     * @version: V1.0
     */
    BigDecimal queryInfoCodeAverage(String groupCode, String projectId, List<String> objectIds, String infoCode) throws Exception;


    /**
     * @description: 计算采集平均值
     * @param: currentDataDTOList 信息点采集值
     * @return: BigDecimal
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/24 10:57
     * @version: V1.0
     */
    BigDecimal queryInfoCodeAverage(List<CurrentDataDTO> currentDataDTOList);


    /**
     * @description: 查询历史平均数据
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIds 对象id
     * @param: infoCode 信息点编码
     * @param: startTime 开始时间
     * @param: endTime 结束时间
     * @param: period 精度
     * @return: java.util.List<com.persagy.dmp.dto.DataTimeDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/19 11:58
     * @version: V1.0
     */
    List<DataTimeDTO> queryHistoryInfoCodeAverage(String groupCode, String projectId, List<String> objectIds, String infoCode, String startTime, String endTime, String period) throws Exception;

    /**
     * @description: 统计对象列表下同一信息点的采集比较值
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: objectIds  对象id列表
     * @param: infoCode 信息点(仅支持已绑点的动态信息点)
     * @param: data 比较值
     * @return: long
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/24 10:56
     * @version: V1.0
     */
    Long queryInfoCodeStatistics(String groupCode, String projectId, List<String> objectIds, String infoCode, BigDecimal data) throws Exception;
}
