package com.persagy.dmp.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import java.util.List;

/**
 * @description: 关系对象接口
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/9/22 15:17
 * @version: V1.0
 **/
public interface RelationObjectService {

    /**
     * @description: 查询对象关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型
     * @param: relCode 边类型
     * @param: fromId from对象id
     * @param: toClassCode to对象类型
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:53
     * @version: V1.0
     */
    JSONArray queryRelationObjectByFromId(String groupCode, String projectId, String graphCode, String relCode, String fromId, List<String> toClassCode, List<String> withColumns) throws Exception;

    /**
     * @description: 查询对象关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型
     * @param: relCode 边类型
     * @param: fromIds from对象id列表
     * @param: toClassCode to对象类型
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:52
     * @version: V1.0
     */
    JSONArray queryRelationObjectByFromIds(String groupCode, String projectId, String graphCode, String relCode, List<String> fromIds, List<String> toClassCode, List<String> withColumns) throws Exception;

    /**
     * @description: 查询对象关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型
     * @param: relCode 边类型
     * @param: toId to对象id
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:52
     * @version: V1.0
     */
    JSONArray queryRelationObjectByToId(String groupCode, String projectId, String graphCode, String relCode, String toId) throws Exception;

    /**
     * @description: 查询对象关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型
     * @param: relCode 边类型
     * @param: fromClassCode from对象类型
     * @param: toId to对象id
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:52
     * @version: V1.0
     */
    JSONArray queryRelationObjectByToId(String groupCode, String projectId, String graphCode, String relCode, List<String> fromClassCode, String toId, List<String> withColumns) throws Exception;

    /**
     * @description: 查询对象关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型
     * @param: relCode 边类型
     * @param: fromClassCode from对象类型
     * @param: toId to对象id
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:52
     * @version: V1.0
     */
    JSONArray queryRelationObjectByToId(String groupCode, String projectId, String graphCode, String relCode, String toId, String ibmsSceneCode, String ibmsClassCode) throws Exception;
    
    /**
     * @description: 查询对象关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型
     * @param: rel 边类型
     * @param: fromClassCode from对象类型
     * @param: toIds to对象id列表
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:52
     * @version: V1.0
     */
    JSONArray queryRelationObjectByToIds(String groupCode, String projectId, String graphCode, String relCode, List<String> fromClassCode, List<String> toIds, List<String> withColumns) throws Exception;

    /**
     * @description: 查询对象关系（fromIds和 toIds只能传一个）
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型
     * @param: relCode 边类型
     * @param: fromIds from对象id列表
     * @param: toIds to对象id列表
     * @param: objectCriteria 对象查询条件
     * @param: withColumns 附带对象信息点（默认只返回id, name, localId, localName, classCode）
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:52
     * @version: V1.0
     */
    JSONArray queryRelationObject(String groupCode, String projectId, String graphCode, String relCode, List<String> fromIds, List<String> toIds, JSONObject objectCriteria, List<String> withColumns) throws Exception;
}
