package com.persagy.dmp.service;

import com.persagy.dmp.dto.ObjRelationDTO;
import com.persagy.dmp.dto.relation.BatchRelationCriteria;
import com.persagy.dmp.dto.relation.CreateRelation;
import com.persagy.dmp.dto.relation.RelationCriteria;
import lombok.NonNull;

import java.util.List;

/**
 * @description: 关系接口
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/9/22 15:12
 * @version: V1.0
 **/
public interface RelationService {

    /**
     * @description: 查询关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: fromId from对象id
     * @return: java.util.List<java.lang.String>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:53
     * @version: V1.0
     */
    List<String> queryRelationByFromId(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull String fromId) throws Exception;

    /**
     * @description: 查询关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: fromId from对象id
     * @param: toClassCode to对象筛选对象类
     * @return: java.util.List<java.lang.String>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:53
     * @version: V1.0
     */
    List<String> queryRelationByFromId(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull String fromId, @NonNull String toClassCode) throws Exception;

    /**
     * @description: 查询关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: fromIds from对象id列表
     * @return: java.util.List<com.persagy.common.dto.ObjRelationDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:53
     * @version: V1.0
     */
    List<ObjRelationDTO> queryRelationByFromIds(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull List<String> fromIds) throws Exception;

    /**
     * @description: 查询关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: fromIds from对象id列表
     * @param: toClassCode to对象筛选对象类
     * @return: java.util.List<com.persagy.common.dto.ObjRelationDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:53
     * @version: V1.0
     */
    List<ObjRelationDTO> queryRelationByFromIds(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull List<String> fromIds, @NonNull String toClassCode) throws Exception;

    /**
     * @description: 查询关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: toId to对象id
     * @return: java.util.List<java.lang.String>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:53
     * @version: V1.0
     */
    List<String> queryRelationByToId(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull String toId) throws Exception;

    /**
     * @description: 查询关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: toId to对象id
     * @param: fromClassCode to对象筛选对象类
     * @return: java.util.List<java.lang.String>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:53
     * @version: V1.0
     */
    List<String> queryRelationByToId(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull String toId, @NonNull String fromClassCode) throws Exception;

    /**
     * @description: 查询关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: toIds to对象id列表
     * @return: java.util.List<com.persagy.common.dto.ObjRelationDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:53
     * @version: V1.0
     */
    List<ObjRelationDTO> queryRelationByToIds(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull List<String> toIds) throws Exception;

    /**
     * @description: 查询关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: toIds to对象id列表
     * @param: fromClassCode to对象筛选对象类
     * @return: java.util.List<com.persagy.common.dto.ObjRelationDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/9/17 14:53
     * @version: V1.0
     */
    List<ObjRelationDTO> queryRelationByToIds(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull List<String> toIds, @NonNull String fromClassCode) throws Exception;

    /**
     * @description: 查询关系(fromIds和toIds只能传一个)
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: graphCode 图类型编码
     * @param: relCode 边类型编码
     * @param: fromIds from对象id
     * @param: toIds to对象id列表
     * @return: java.util.List<com.persagy.dmp.dto.ObjRelationDTO>
     * @exception:
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/30 10:44
     * @version: V1.0
     */
    List<ObjRelationDTO> queryRelation(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, List<String> fromIds, List<String> toIds) throws Exception;

    /**
     * @description: 创建关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: createRelationList 创建关系
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:20
     * @version: V1.0
     */
    Boolean createRelation(@NonNull String groupCode, @NonNull String projectId, @NonNull CreateRelation createRelation);

    /**
     * @description: 批量创建关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: createRelationList 创建关系
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:20
     * @version: V1.0
     */
    Boolean createRelation(@NonNull String groupCode, @NonNull String projectId, @NonNull List<CreateRelation> createRelationList);

    /**
     * @description: 批量删除关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: criteria 删除关系条件
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:20
     * @version: V1.0
     */
    Boolean batchDeleteRelation(@NonNull String groupCode, @NonNull String projectId, @NonNull BatchRelationCriteria criteria);

    /**
     * @description: 删除关系
     * @param: groupCode 集团编码
     * @param: projectId 项目id
     * @param: criteria 删除关系条件
     * @return: java.lang.Integer
     * @author: xingmaojun
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/31 11:20
     * @version: V1.0
     */
    Boolean deleteRelation(@NonNull String groupCode, @NonNull String projectId, @NonNull RelationCriteria criteria);

}
