package com.persagy.dmp.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONArray;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.dmp.dao.DefDao;
import com.persagy.dmp.dto.ClassInfo;
import com.persagy.dmp.service.ClassInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.persagy.dmp.constant.CommonConstant.PARENT_CODE;
import static com.persagy.dmp.constant.CommonConstant.SYSTEM;

/**
 * @description:
 * @author: shiliqiang
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/10/11 15:11
 * @version: V1.0
 */
@Service
public class ClassInfoServiceImpl implements ClassInfoService {

    @Autowired
    private DefDao defDao;

    @Override
    public List<ClassInfo> queryClass(String groupCode, String projectId, Boolean subFlag, JacksonCriteria criteria) throws Exception {
        if (criteria == null) {
            criteria = JacksonCriteria.newInstance();
        }
        JSONArray classInfoArr = defDao.queryClassInfoList(groupCode, projectId, subFlag, criteria);
        if (classInfoArr != null && classInfoArr.size() > 0) {
            return classInfoArr.toJavaList(ClassInfo.class);
        }
        return new ArrayList<>();
    }

    /**
     * @param groupCode:
     * @param projectId:
     * @description: 查询项目下系统编码-系统名称 的map
     * @author: LuoGuangyi
     * @createTime: 2021/01/13 19:37
     * @return: java.util.Map<java.lang.String, java.lang.String>
     * @expression
     */
    @Override
    @Cacheable(cacheNames = "systemCodeNameMap")
    public Map<String, String> querySystemCodeNameMap(String groupCode, String projectId) throws Exception {
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add(PARENT_CODE, SYSTEM);
        JSONArray classInfoArr = defDao.queryClassInfoList(groupCode, projectId, false, criteria);
        if (CollectionUtil.isNotEmpty(classInfoArr)) {
            List<ClassInfo> classInfos = classInfoArr.toJavaList(ClassInfo.class);
            Map<String, String> systemCodeNameMap = classInfos.stream().collect(Collectors.toMap(ClassInfo::getCode, ClassInfo::getName));
            return systemCodeNameMap;
        }
        return new HashMap<>();
    }
}
