package com.persagy.dmp.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.dmp.dao.IOTDao;
import com.persagy.dmp.dto.CurrentDataDTO;
import com.persagy.dmp.dto.DataQueryDTO;
import com.persagy.dmp.dto.control.BatchPointSetParam;
import com.persagy.dmp.dto.control.BatchPointSetResult;
import com.persagy.dmp.dto.control.PointSetParam;
import com.persagy.dmp.dto.control.PointSetResult;
import com.persagy.dmp.service.ControlService;
import com.persagy.dmp.service.CurrentDataService;
import com.persagy.dmp.service.ObjectService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @description: 实时数据接口实现类
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/9/22 15:16
 * @version: V1.0
 **/
@Service
public class CurrentDataServiceImpl implements CurrentDataService {

    @Autowired
    private IOTDao iotDao;

    @Autowired
    private ControlService controlService;

    @Autowired
    private ObjectService objectService;

    @Override
    public CurrentDataDTO queryCurrentData(String groupCode, String projectId, String objectId, String infoCode) throws Exception {
        return iotDao.queryCurrentData(groupCode, projectId, objectId, infoCode);
    }

    @Override
    public Map<String, BigDecimal> queryCurrentDataMap(String groupCode, String projectId, List<String> objectIdList, String infoCode) throws Exception {
        List<CurrentDataDTO> currentDataDTOList = this.queryCurrentDataList(groupCode, projectId, objectIdList, infoCode);
        Map<String, BigDecimal> id2ValueMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(currentDataDTOList)) {
            currentDataDTOList = currentDataDTOList.stream().filter(o -> o.getData() != null && !StrUtil.isBlankOrUndefined(o.getTime())).collect(Collectors.toList());
            currentDataDTOList.forEach(o -> id2ValueMap.put(o.getObjectId(), o.getData()));
        }
        return id2ValueMap;
    }

    @Override
    public Map<String, BigDecimal> queryCurrentDataMap(String groupCode, String projectId, String objectId, List<String> infoCodes) throws Exception {
        List<CurrentDataDTO> currentDataDTOList = this.queryCurrentDataList(groupCode, projectId, objectId, infoCodes);
        Map<String, BigDecimal> id2ValueMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(currentDataDTOList)) {
            currentDataDTOList = currentDataDTOList.stream().filter(o -> o.getData() != null && !StrUtil.isBlankOrUndefined(o.getTime())).collect(Collectors.toList());
            currentDataDTOList.forEach(o -> id2ValueMap.put(o.getInfoCode(), o.getData()));
        }
        return id2ValueMap;
    }

    @Override
    public List<CurrentDataDTO> queryCurrentDataList(String groupCode, String projectId, List<String> objectIdList, String infoCode) throws Exception {
        if (CollectionUtils.isEmpty(objectIdList) || StringUtils.isBlank(infoCode)) {
            return new ArrayList<>();
        }
        return iotDao.queryCurrentDataList(groupCode, projectId, objectIdList, infoCode);
    }

    @Override
    public List<CurrentDataDTO> queryCurrentDataList(String groupCode, String projectId, String objectId, List<String> infoCodeList) throws Exception {
        if (CollectionUtils.isEmpty(infoCodeList) || StringUtils.isBlank(objectId)) {
            return new ArrayList<>();
        }
        return iotDao.queryCurrentDataList(groupCode, projectId, objectId, infoCodeList);
    }

    @Override
    public List<CurrentDataDTO> queryCurrentDataList(String groupCode, String projectId, List<DataQueryDTO> dataQueryDTOList) throws Exception {
        if (CollectionUtils.isEmpty(dataQueryDTOList)) {
            return new ArrayList<>();
        }
        return iotDao.queryCurrentDataList(groupCode, projectId, dataQueryDTOList);
    }

    @Override
    public CurrentDataDTO queryControlData(String groupCode, String projectId, String objectId, String infoCode) throws Exception {
        if (StringUtils.isBlank(infoCode) || StringUtils.isBlank(objectId)) {
            return null;
        }
        List<DataQueryDTO> dataQueryDTOList = new ArrayList<>();
        dataQueryDTOList.add(new DataQueryDTO(objectId, infoCode));
        List<CurrentDataDTO> currentDataDTOList = this.queryControlDataList(groupCode, projectId, dataQueryDTOList);
        if (!CollectionUtils.isEmpty(currentDataDTOList)) {
            return currentDataDTOList.get(0);
        }
        return null;
    }

    @Override
    public Map<String, BigDecimal> queryControlDataMap(String groupCode, String projectId, List<String> objectIdList, String infoCode) throws Exception {
        List<CurrentDataDTO> currentDataDTOList = this.queryControlDataList(groupCode, projectId, objectIdList, infoCode);
        Map<String, BigDecimal> id2ValueMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(currentDataDTOList)) {
            currentDataDTOList = currentDataDTOList.stream().filter(o -> o.getData() != null && !StrUtil.isBlankOrUndefined(o.getTime())).collect(Collectors.toList());
            currentDataDTOList.forEach(o -> id2ValueMap.put(o.getObjectId(), o.getData()));
        }
        return id2ValueMap;
    }

    @Override
    public List<CurrentDataDTO> queryControlDataList(String groupCode, String projectId, List<String> objectIdList, String infoCode) throws Exception {
        if (CollectionUtils.isEmpty(objectIdList) || StringUtils.isBlank(infoCode)) {
            return new ArrayList<>();
        }
        List<DataQueryDTO> dataQueryDTOList = objectIdList.stream().flatMap(objectId -> Stream.of(new DataQueryDTO(objectId, infoCode))).collect(Collectors.toList());
        return this.queryControlDataList(groupCode, projectId, dataQueryDTOList);
    }

    @Override
    public List<CurrentDataDTO> queryControlDataList(String groupCode, String projectId, String objectId, List<String> infoCodeList) throws Exception {
        if (CollectionUtils.isEmpty(infoCodeList) || StringUtils.isBlank(objectId)) {
            return new ArrayList<>();
        }
        List<DataQueryDTO> dataQueryDTOList = infoCodeList.stream().flatMap(infoCode -> Stream.of(new DataQueryDTO(objectId, infoCode))).collect(Collectors.toList());
        return this.queryControlDataList(groupCode, projectId, dataQueryDTOList);
    }

    @Override
    public List<CurrentDataDTO> queryControlDataList(String groupCode, String projectId, List<DataQueryDTO> dataQueryDTOList) throws Exception {

        if (CollectionUtils.isEmpty(dataQueryDTOList)) {
            return new ArrayList<>();
        }

        int threadCounts = dataQueryDTOList.size() / 100;
        if (threadCounts == 0) {
            threadCounts = 1;
        } else if (threadCounts > 32) {
            threadCounts = 32;
        }

        ExecutorService exec = Executors.newFixedThreadPool(threadCounts);
        List<Callable<List<CurrentDataDTO>>> callList = new ArrayList<Callable<List<CurrentDataDTO>>>();

        //平均分割List
        int len = dataQueryDTOList.size() / threadCounts;
        //List中的数量没有线程数多（很少存在）
        if (len == 0) {
            //采用一个线程处理List中的一个元素
            threadCounts = dataQueryDTOList.size();
            //重新平均分割List
            len = dataQueryDTOList.size() / threadCounts;
        }

        for (int i = 0; i < threadCounts; i++) {
            final List<DataQueryDTO> subList;
            if (i == threadCounts - 1) {
                subList = dataQueryDTOList.subList(i * len, dataQueryDTOList.size());
            } else {
                subList = dataQueryDTOList.subList(i * len, len * (i + 1) > dataQueryDTOList.size() ? dataQueryDTOList.size() : len * (i + 1));
            }
            //采用匿名内部类实现
            callList.add(() -> CurrentDataServiceImpl.this.querySubControlDataList(groupCode, projectId, subList));
        }

        List<Future<List<CurrentDataDTO>>> futureList = exec.invokeAll(callList);
        List<CurrentDataDTO> currentDataDTOList = new ArrayList<>();
        for (Future<List<CurrentDataDTO>> future : futureList) {
            List<CurrentDataDTO> subCurrentDataDTOList = future.get();
            if (!CollectionUtils.isEmpty(subCurrentDataDTOList)) {
                currentDataDTOList.addAll(subCurrentDataDTOList);
            }
        }
        exec.shutdown();

        return currentDataDTOList;
    }

    private List<CurrentDataDTO> querySubControlDataList(String groupCode, String projectId, List<DataQueryDTO> dataQueryDTOList) throws Exception {

        List<CurrentDataDTO> currentDataDTOList = new ArrayList<>();
        if (CollectionUtils.isEmpty(dataQueryDTOList)) {
            return currentDataDTOList;
        }
        //获取对象id，去重
        Set<String> objectIdSet = dataQueryDTOList.stream().map(DataQueryDTO::getObjectId).collect(Collectors.toSet());
        //根据id查询对象详情
        JSONArray objArr = objectService.queryObjectList(groupCode, projectId, new ArrayList<>(objectIdSet));

        if (CollectionUtils.isEmpty(objArr)) {
            return currentDataDTOList;
        }

        //对象id，信息点，表号-功能号Map
        Map<String, Map<String, String>> objId2InfoCode2MeterMap = new HashMap<>();
        for (Object object : objArr) {
            JSONObject objJson = (JSONObject) object;
            String objId = objJson.getString("id");
            for (Map.Entry<String, Object> entry : objJson.entrySet()) {
                if (entry.getValue() instanceof String && ((String) entry.getValue()).contains("-")) {
                    if (objId2InfoCode2MeterMap.containsKey(objId)) {
                        objId2InfoCode2MeterMap.get(objId).put(entry.getKey(), entry.getValue().toString());
                    } else {
                        Map<String, String> infoCode2MeterMap = new HashMap<>();
                        infoCode2MeterMap.put(entry.getKey(), entry.getValue().toString());
                        objId2InfoCode2MeterMap.put(objId, infoCode2MeterMap);
                    }
                }
            }
        }

        //给信息点匹配表号、功能号
        for (DataQueryDTO dataQueryDTO : dataQueryDTOList) {
            String objId = dataQueryDTO.getObjectId();
            String infoCode = dataQueryDTO.getInfoCode();
            CurrentDataDTO currentDataDTO = new CurrentDataDTO();
            currentDataDTO.setObjectId(objId);
            currentDataDTO.setInfoCode(infoCode);

            if (!objId2InfoCode2MeterMap.containsKey(objId)) {
                currentDataDTO.setError(String.format("对象(%s)不存在", objId));
                continue;
            }

            //表号-功能号
            String meterFuncId = objId2InfoCode2MeterMap.get(objId).get(infoCode);
            if (StringUtils.isNotBlank(meterFuncId)) {
                //拆分表号、功能号
                int index = meterFuncId.lastIndexOf("-");
                currentDataDTO.setMeter(meterFuncId.substring(0, index));
                currentDataDTO.setFunction(meterFuncId.substring(index + 1));
            } else {
                //信息点不存在或没绑点
                currentDataDTO.setError(String.format("信息点(%s)没绑定表号、功能号或不存在", infoCode));
            }
            currentDataDTOList.add(currentDataDTO);
        }

        BatchPointSetParam batchPointSetParam = new BatchPointSetParam();
        batchPointSetParam.setBuilding(projectId.substring(2, 12));
        List<PointSetParam> pointSetParamList = new ArrayList<>();
        batchPointSetParam.setPoints(pointSetParamList);
        //生成查询控制值参数
        for (CurrentDataDTO currentDataDTO : currentDataDTOList) {
            if (StrUtil.isNotBlank(currentDataDTO.getMeter()) && StrUtil.isNotBlank(currentDataDTO.getFunction())) {
                PointSetParam pointSetParam = new PointSetParam();
                pointSetParam.setMeter(currentDataDTO.getMeter());
                pointSetParam.setFuncId(Integer.valueOf(currentDataDTO.getFunction()));
                pointSetParamList.add(pointSetParam);
            }
        }
        //查询控制值
        BatchPointSetResult batchPointSetResult = null;
        if (batchPointSetParam != null && !CollectionUtils.isEmpty(batchPointSetParam.getPoints())) {
            batchPointSetResult = controlService.presentSetBatch(batchPointSetParam);
        }

        //转换成Map格式
        Map<String, PointSetResult> meterFuncId2DataMap = new HashMap<>();
        if (batchPointSetResult != null) {
            List<PointSetResult> points = batchPointSetResult.getPoints();
            if (!CollectionUtils.isEmpty(points)) {
                points.forEach(o -> {
                    meterFuncId2DataMap.put(o.getMeter() + "-" + o.getFuncId(), o);
                });
            }
        }

        //控制值绑点到对象、信息点上
        for (CurrentDataDTO currentDataDTO : currentDataDTOList) {
            if (StringUtils.isBlank(currentDataDTO.getMeter()) || StringUtils.isBlank(currentDataDTO.getFunction())) {
                continue;
            }
            String meterFuncId = String.join("-", currentDataDTO.getMeter(), currentDataDTO.getFunction());
            if (meterFuncId2DataMap.containsKey(meterFuncId) && meterFuncId2DataMap.get(meterFuncId).getReceiveTime() != null) {
                currentDataDTO.setData(meterFuncId2DataMap.get(meterFuncId).getData());
                currentDataDTO.setTime(meterFuncId2DataMap.get(meterFuncId).getReceiveTime());
            } else {
                currentDataDTO.setError(String.format("信息点(%s)未查询到控制值", currentDataDTO.getInfoCode()));
            }
        }

        return currentDataDTOList;
    }
}
