package com.persagy.dmp.service.impl;

import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.dmp.cache.DataDictCache;
import com.persagy.dmp.dao.DefDao;
import com.persagy.dmp.dao.DicDao;
import com.persagy.dmp.dto.CategoryDTO;
import com.persagy.dmp.dto.ClassInfo;
import com.persagy.dmp.dto.FuncIdQueryDTO;
import com.persagy.dmp.dto.InfoPointDictDTO;
import com.persagy.dmp.dto.dict.DictDataDTO;
import com.persagy.dmp.service.ClassInfoService;
import com.persagy.dmp.service.DictService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.CompletableFuture;

/**
 * @description: 数据字典接口实现类
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/9/22 15:10
 * @version: V1.0
 **/
@Service
public class DictServiceImpl implements DictService {

    @Autowired
    private DefDao defDao;

    @Autowired
    private DicDao dicDao;

    @Autowired
    private DataDictCache dataDictCache;

    @Autowired
    private ClassInfoService classInfoService;

    @Override
    public Map<String, InfoPointDictDTO> queryInfoPointDictMap(String groupCode, String projectId, String classCode) throws Exception {
        Map<String, InfoPointDictDTO> infoCode2DtoMap = new HashMap<>();
        List<InfoPointDictDTO> infoPointDictDTOList = this.queryInfoPointDictList(groupCode, projectId, classCode);
        if (!CollectionUtils.isEmpty(infoPointDictDTOList)) {
            infoPointDictDTOList.forEach(o -> infoCode2DtoMap.put(o.getCode(), o));
        }
        return infoCode2DtoMap;
    }

    @Override
    public List<InfoPointDictDTO> queryInfoPointDictList(String groupCode, String projectId, String classCode) throws Exception {
//        List<InfoPointDictDTO> infoPointDictDTOList = dataDictCache.getDataDict(groupCode, projectId, classCode);
//        if (!CollectionUtils.isEmpty(infoPointDictDTOList)) {
//            return infoPointDictDTOList;
//        }
        return defDao.queryInfoPointDictList(groupCode, projectId, classCode);

//        //异步存入缓存
//        List<InfoPointDictDTO> finalInfoPointDictDTOList = infoPointDictDTOList;
//        CompletableFuture.runAsync(() -> {
//            dataDictCache.putDataDict(groupCode, projectId, classCode, finalInfoPointDictDTOList);
//        });
//
//        return infoPointDictDTOList;
    }

    @Override
    public List<InfoPointDictDTO> queryInfoPointDictList(String groupCode, String projectId, FuncIdQueryDTO funcIdQueryDto) throws Exception {
        return defDao.queryInfoPointDictList(groupCode, projectId, funcIdQueryDto);
    }

    @Override
    @Cacheable(cacheNames = "category")
    public List<CategoryDTO> queryCategoryList(String groupCode, String projectId) throws Exception {
        // 查询系统、设备类
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.add("objType").in(Arrays.asList("system", "equipment"));
        List<ClassInfo> classInfoList = classInfoService.queryClass(groupCode, projectId, false, criteria);

        // 系统数据
        Map<String, ClassInfo> systemMap = new HashMap<>();
        // 设备数据
        Map<String, ClassInfo> equipMap = new HashMap<>();

        for (ClassInfo c : classInfoList) {
            if ("system".equals(c.getObjType())) {
                // 把父类型给去掉
                if (!"system".equals(c.getCode())) {
                    systemMap.put(c.getCode(), c);
                }
            } else {
                // 把父类型给去掉
                if (!"equipment".equals(c.getCode())) {
                    equipMap.put(c.getCode(), c);
                }
            }
        }
        //查询专业
        Map<String, String> majorMap = this.queryMajorMap(groupCode, projectId);

        //组装成平铺格式
        List<CategoryDTO> categoryList = new ArrayList<>();
        for (Map.Entry<String, ClassInfo> c : equipMap.entrySet()) {
            String code = c.getKey();
            ClassInfo classInfo = c.getValue();
            String majorCode = classInfo.getMajorCode();
            String systemCode = classInfo.getSystemCode();
            CategoryDTO categoryDTO = new CategoryDTO();

            categoryDTO.setCategoryCode(code);
            if (classInfo != null) {
                categoryDTO.setCategoryName(classInfo.getName());
                categoryDTO.setCategoryAliasCode(classInfo.getAliasCode());
                categoryDTO.setCategoryAliasName(classInfo.getAliasName());
            }

            ClassInfo classInfoSys = systemMap.get(systemCode);
            categoryDTO.setSystemCode(systemCode);
            if (classInfoSys != null) {
                categoryDTO.setSystemName(classInfoSys.getName());
                categoryDTO.setSystemAliasCode(classInfoSys.getAliasCode());
                categoryDTO.setSystemAliasName(classInfoSys.getAliasName());
            }

            String majorName = majorMap.get(majorCode);
            categoryDTO.setMajorCode(majorCode);
            if (majorName != null) {
                categoryDTO.setMajorName(majorName);
            }
            categoryList.add(categoryDTO);
        }
        return categoryList;
    }

    @Override
    @Cacheable(value = "major")
    public Map<String, String> queryMajorMap(String groupCode, String projectId) throws Exception {
        List<DictDataDTO> dictDataDTOList = dicDao.queryDictDataList(groupCode, projectId, "major");
        if (!CollectionUtils.isEmpty(dictDataDTOList)) {
            Map<String, String> majorMap = new HashMap<>();
            for (DictDataDTO m : dictDataDTOList) {
                majorMap.put(m.getCode(), m.getLabel());
            }
            return majorMap;
        }
        return new HashMap<>();
    }
}
