package com.persagy.dmp.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.dmp.constant.CommonConstant;
import com.persagy.dmp.dao.GeneralDictionaryDao;
import com.persagy.dmp.dto.GeneralDictionaryDTO;
import com.persagy.dmp.entity.GeneralDictionary;
import com.persagy.dmp.entity.Sort;
import com.persagy.dmp.service.GeneralDictionaryService;
import com.persagy.dmp.utils.CovertUtil;
import com.persagy.dmp.utils.ItvJsonUtil;
import com.persagy.dmp.utils.StringUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.io.Serializable;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;

/**
 * 通用数据字典表服务接口实现
 *
 * @author luoguangyi
 * @description 由 Mybatisplus Code Generator 创建
 * @since 2021-01-26 11:42:11
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class GeneralDictionaryServiceImpl implements GeneralDictionaryService {
    private final GeneralDictionaryDao generalDictionaryDao;

    List<String> generalDictionaryColumm = Arrays.asList("dictId", "groupCode", "projectId", "dictType", "code", "parentCode", "name", "sequence", "itemData", "arrayData", "remark", "createTime", "updateTime", "valid", "children");

    public static <T> Predicate<T> distinctByKey(Function<? super T, Object> keyExtractor) {
        Map<Object, Boolean> seen = new ConcurrentHashMap<>();
        return t -> seen.putIfAbsent(keyExtractor.apply(t), Boolean.TRUE) == null;
    }

    /**
     * @param sourceList:
     * @description: 字典去重，重复覆盖：项目级覆盖集团级覆盖平台级
     * 重复依据是dictType+code
     * @author: LuoGuangyi
     * @createTime: 2021/03/05 20:26
     * @return: java.util.List<com.persagy.dmp.entity.GeneralDictionary>
     * @expression
     */
    public static List<GeneralDictionary> distinctByDictTypeAndCode2CoverDuplicate(List<GeneralDictionary> sourceList) {
        if (ObjectUtil.isEmpty(sourceList)) {
            return new ArrayList<>();
        }
        List<GeneralDictionary> distinctList = sourceList.stream().collect(
                Collectors.toMap(item -> item.getDictType() + item.getCode(), Function.identity(), (item1, item2) -> {
                    //groupCode相等情况下
                    if (item1.getGroupCode().compareTo(item2.getGroupCode()) == 0) {
                        //item1不是0，返回item1
                        if (!"0".equals(item1.getProjectId())) {
                            return item1;
                        }
                        return item2;
                    }
                    //groupCode不相等情况下
                    int groupCodeNums = getCommonNumber(item1.getGroupCode(), item2.getGroupCode());
                    if (groupCodeNums == 1) {
                        return "0".equals(item1.getGroupCode()) ? item2 : item1;
                    } else if (!"0".equals(item1.getProjectId())) {
                        return item1;
                    }
                    //默认返回后者
                    return item2;
                })
        ).values().stream().collect(Collectors.toList());
        return distinctList;
    }

    private static int getCommonNumber(String... args) {
        int count = 0;
        for (String element : args) {
            if ("0".equals(element)) {
                count++;
            }
        }
        return count;
    }

    private static int getNotCommonNumber(String... args) {
        int count = 0;
        for (String element : args) {
            if (StrUtil.isNotBlank(element) && !"0".equals(element)) {
                count++;
            }
        }
        return count;
    }

    /**
     * @param record :
     * @description: 查询或者更新记录
     * 更新的时候必须更新传更新的额所有值
     * @author: LuoGuangyi
     * @createTime: 2021/01/26 16:19
     * @return: boolean
     * @expression
     */
    @Override
    public List<GeneralDictionaryDTO> saveOrUpdate(GeneralDictionaryDTO record) throws Exception {
        return saveOrUpdate(record, false);
    }

    /**
     * @param record                 : 保存的记录
     * @param automaticGeneratedCode : 是否自动生成code
     * @description: 查询或者更新记录
     * 更新的时候必须更新传更新的额所有值
     * @author: LuoGuangyi
     * @createTime: 2021/01/26 16:19
     * @return: boolean
     * @expression
     */
    @Override
    public List<GeneralDictionaryDTO> saveOrUpdate(GeneralDictionaryDTO record, boolean automaticGeneratedCode) throws Exception {
        String dictType = record.getDictType();
        if (automaticGeneratedCode) {
            if(StrUtil.isNotBlank(record.getDictId())) {
                GeneralDictionary generalDictionary = getGeneralDictionary(record.getDictId());
                if(generalDictionary == null) {
                    throw new RuntimeException("要更新的记录不存在！");
                }
                String code = generalDictionary.getCode();
                record.setCode(code);
            }else if (StringUtil.isNull(record.getCode())) {
                record.setCode(getMaxCode(dictType));
            }
        }
        String parentCode = record.getCode();
        List<GeneralDictionaryDTO> childrens = record.getChildren();
        GeneralDictionary generalDictionary = CovertUtil.INSTANCE.convertGeneralDictionary(record);
        String code = record.getCode();
        String groupCode = record.getGroupCode();
        String projectId = record.getProjectId();
        Assert.notBlank(dictType, "参数 [dict_type] 不能为空");
        Assert.notBlank(code, "参数 [code] 不能为空");
        //GENERAL_DICTIONARY_PARENT_CODE="/"是字典树顶级的编码，任何code不能使用该编码，否则会导致死循环
        Assert.isFalse(CommonConstant.GENERAL_DICTIONARY_PARENT_CODE.equals(code), "参数 [code] 不能为空");
        GeneralDictionaryDTO listParam = new GeneralDictionaryDTO();
        listParam.setDictType(dictType);
        listParam.setCode(code);
        listParam.setGroupCode(StringUtil.getString(groupCode, "0"));
        listParam.setProjectId(StringUtil.getString(projectId, "0"));
        List<GeneralDictionary> generalDictionaries = listGeneralDictionaryNotCoverDuplicate(listParam);
        boolean exist = generalDictionaries.stream().anyMatch(item -> code.equals(item.getCode()));
        if (exist) {
            //修改的时候把查询DictId并赋值
            String oldDictId = generalDictionaries.stream().filter(item -> code.equals(item.getCode())).findFirst().get().getDictId();
            Integer oldSequence = generalDictionaries.stream().filter(item -> code.equals(item.getCode())).findFirst().get().getSequence();
            generalDictionary.setDictId(oldDictId);
            //更新的时候默认使用原始排序顺序
            if (ObjectUtil.isEmpty(generalDictionary.getSequence())) {
                generalDictionary.setSequence(oldSequence);
            }
        }
        //展示顺序字段赋值
        if (ObjectUtil.isEmpty(generalDictionary.getSequence())) {
            generalDictionary.setSequence(getMaxSequence(dictType));
        }
        boolean result = generalDictionaryDao.saveOrUpdate(generalDictionary);
        Assert.isTrue(result, "保存数据库失败");
        if (ObjectUtil.isNotEmpty(childrens)) {
            for (GeneralDictionaryDTO children : childrens) {
                children.setParentCode(parentCode);
                saveOrUpdate(children);
            }
        }
        List<GeneralDictionaryDTO> list = new ArrayList<>();
        list.add(CovertUtil.INSTANCE.convertGeneralDictionaryDTO(generalDictionaryDao.getById(generalDictionary.getDictId())));
        return list;
    }


    /**
     * @param record:                 字段记录
     * @param automaticGeneratedCode: 是否自动生成code
     * @param uniqueByDictId:         唯一校验是否根据dictId，否的话根据groupCode+project+dictType+code唯一校验
     * @description: 保存更新字典，itemData属性合并,自动生成code
     * @author: LuoGuangyi
     * @createTime: 2021/03/09 12:44
     * @return: java.util.List<com.alibaba.fastjson.JSONObject>
     * @expression
     */
    @Override
    public List<JSONObject> saveOrUpdate(JSONObject record, boolean automaticGeneratedCode, boolean uniqueByDictId) throws Exception {
//        GeneralDictionaryDTO generalDictionaryDTO = JSONObject.parseObject(transferItemData(record,uniqueByDictId).toString(), GeneralDictionaryDTO.class);
        GeneralDictionaryDTO generalDictionaryDTO = JSONObject.parseObject(transferItemData(record,uniqueByDictId).toString(SerializerFeature.DisableCircularReferenceDetect), GeneralDictionaryDTO.class);
        List<GeneralDictionaryDTO> gds = saveOrUpdate(generalDictionaryDTO, automaticGeneratedCode);
        return listDictionary(gds);
    }

    /**
     * @param dictType:
     * @description: 获取最大序号+1
     * @author: LuoGuangyi
     * @createTime: 2021/01/26 16:29
     * @return: int
     * @expression
     */
    private int getMaxSequence(String dictType) throws Exception {
        GeneralDictionaryDTO listParam = new GeneralDictionaryDTO();
        listParam.setDictType(dictType);
        List<GeneralDictionary> generalDictionaries = listGeneralDictionaryNotCoverDuplicate(listParam);
        //新增的时候需要给sequence赋值当前最大值+1
        int sequence = 1;
        OptionalInt max = generalDictionaries.stream().mapToInt(GeneralDictionary::getSequence).max();
        if (max.isPresent()) {
            sequence = max.getAsInt() + 1;
        }
        return sequence;
    }

    /**
     * @param dictType:
     * @description: 获取最大code+1
     * 适用于需要自动生成code场景
     * @author: LuoGuangyi
     * @createTime: 2021/03/09 11:59
     * @return: int
     * @expression
     */
    private synchronized String getMaxCode(String dictType) throws Exception {
        GeneralDictionaryDTO listParam = new GeneralDictionaryDTO();
        listParam.setDictType(dictType);
        List<GeneralDictionary> generalDictionaries = listGeneralDictionaryNotCoverDuplicate(listParam);
        //新增的时候需要给sequence赋值当前最大值+1
        int code = 1;
        Optional<Integer> max = generalDictionaries.stream().map(item -> StringUtil.getInt(item.getCode())).max(Integer::compare);
        if (max.isPresent()) {
            code = max.get() + 1;
        }
        return String.valueOf(code);
    }

    /**
     * @param record :
     * @description: 查询字段数据
     * 重复覆盖：项目级覆盖集团级覆盖平台级
     * @author: LuoGuangyi
     * @createTime: 2021/01/26 15:22
     * @return: java.util.List<com.persagy.dmp.entity.GeneralDictionary>
     * @expression
     */
    @Override
    public List<GeneralDictionaryDTO> listGeneralDictionary(GeneralDictionaryDTO record) throws Exception {
        Assert.notBlank(record.getDictType(), "参数 [dict_type] 不能为空");
        if (ObjectUtil.isEmpty(record.getParentCode())) {
            //防止前端传空字符串""
            record.setParentCode(null);
        }
        List<String> projectIds = Arrays.asList(record.getProjectId(), "0").stream().filter(item->ObjectUtil.isNotEmpty(item)).distinct().collect(Collectors.toList());
        List<String> groupCodes = Arrays.asList(record.getGroupCode(), "0").stream().filter(item->ObjectUtil.isNotEmpty(item)).distinct().collect(Collectors.toList());
        QueryWrapper<GeneralDictionary> queryWrapper = new QueryWrapper<>();
        Sort sort = new Sort(GeneralDictionary.COL_SEQUENCE, true);
        queryWrapper.eq(StringUtils.isNotBlank(record.getDictType()), GeneralDictionary.COL_DICT_TYPE, record.getDictType())
                .eq(StringUtils.isNotBlank(record.getCode()), GeneralDictionary.COL_CODE, record.getCode())
                .in(true , GeneralDictionary.COL_PROJECT_ID, projectIds)
                .in(true, GeneralDictionary.COL_GROUP_CODE, groupCodes)
                .in(ObjectUtil.isNotEmpty(record.getDictIds()), GeneralDictionary.COL_DICT_ID,record.getDictIds())
                .orderBy(true, sort.getAsc(), sort.getColumn());
        List<GeneralDictionary> dictionaries = generalDictionaryDao.list(queryWrapper);
        if (getNotCommonNumber(record.getProjectId(), record.getGroupCode()) > 0) {
            dictionaries = distinctByDictTypeAndCode2CoverDuplicate(dictionaries);
        }
        if (ObjectUtil.isNotEmpty(record.getCode())) {
            return JSON.parseArray(JSON.toJSONString(dictionaries), GeneralDictionaryDTO.class);
        }
        //parentCode数据库默认值/，code中不能有/，否则会导致死循环
        String parentCode = ObjectUtil.isEmpty(record.getParentCode()) ? CommonConstant.GENERAL_DICTIONARY_PARENT_CODE : record.getParentCode();
        //得到parentCode的下级
        List<Tree<String>> treeNodes = TreeUtil.build(dictionaries, parentCode,
                (treeNode, tree) -> {
                    tree.setId(treeNode.getCode());
                    tree.setParentId(treeNode.getParentCode());
                    tree.setName(treeNode.getName());
                    tree.setWeight(treeNode.getSequence());
                    // 扩展属性 ...
                    tree.putAll(JSONObject.parseObject(treeNode.toString()));
                });
        List<GeneralDictionaryDTO> dictionaryDTOS = ItvJsonUtil.jsonToList(JSONObject.toJSONString(treeNodes), GeneralDictionaryDTO.class);
        return dictionaryDTOS;
    }

    /**
     * @param record:
     * @description: 查询字段数据-重复覆盖（itemData中的数据自动展现到外侧）
     * @author: LuoGuangyi
     * @createTime: 2021/03/05 20:29
     * @return: java.util.List<com.alibaba.fastjson.JSONObject>
     * @expression
     */
    @Override
    public List<JSONObject> listDictionary(GeneralDictionaryDTO record) throws Exception {
        List<GeneralDictionaryDTO> gdList = listGeneralDictionary(record);
        return transferJSONObject(gdList);
    }
    /**
     * @description: 根据id批量获取
     * @author: LuoGuangyi
     * @createTime: 2021/03/30 18:36
     * @param idList:
     * @return: java.util.List<com.alibaba.fastjson.JSONObject>
     * @expression
     */
    @Override
    public List<JSONObject> selectBatchIds(Collection<? extends Serializable> idList) throws Exception {
        List<GeneralDictionary> gdList = generalDictionaryDao.getBaseMapper().selectBatchIds(idList);
        return transferJSONObject(CovertUtil.INSTANCE.convertGeneralDictionarys(gdList));
    }
    @Override
    public JSONObject selectById(String dictId) throws Exception {
        GeneralDictionary item = generalDictionaryDao.getBaseMapper().selectById(dictId);
        JSONObject itemData = item.getItemData();
        if (ObjectUtil.isEmpty(itemData)) {
            itemData = new JSONObject();
        }
        item.setItemData(null);
        itemData.putAll(JSONObject.parseObject(item.toString()));
        return itemData;
    }

    private List<JSONObject> transferJSONObject(List<GeneralDictionaryDTO> gdList) {
        List<JSONObject> gds = gdList.stream().map(item -> {
            JSONObject itemData = item.getItemData();
            if (ObjectUtil.isEmpty(itemData)) {
                itemData = new JSONObject();
            }
            item.setItemData(null);
            if(ObjectUtil.isNotEmpty(item.getChildren())){
                itemData.put("children",transferJSONObject(item.getChildren()));
                item.setChildren(null);
            }
            itemData.putAll(JSONObject.parseObject(item.toString()));
            //移除不需要显示的字段
            //itemData.fluentRemove("dictType").fluentRemove("createTime").fluentRemove("updateTime").fluentRemove("valid");
            return itemData;
        }).collect(Collectors.toList());
        return gds;
    }

    @Override
    public List<JSONObject> listDictionary(List<GeneralDictionaryDTO> gdList) throws Exception {
        return transferJSONObject(gdList);
    }

    private JSONObject transferItemData(JSONObject record, boolean uniqueByDictId) throws Exception {
        JSONArray fields = record.getJSONArray("fields");
        if (ObjectUtil.isEmpty(fields)) {
            fields = record.keySet().stream().collect(Collectors.toCollection(JSONArray::new));
        }
        JSONArray childrens = record.getJSONArray("children");
        JSONObject itemData = (JSONObject) record.getOrDefault(record.getJSONObject("itemData"), new JSONObject());
        //如果是根据dictId进行唯一性校验的话
        if(uniqueByDictId && ObjectUtil.isEmpty(record.getString("dictId"))) {
            putItemData(record, uniqueByDictId, fields, childrens, itemData);
            return record;
        }
        //合并新旧的itemData的扩展数据，重复的字段新数据覆盖旧数据
        GeneralDictionaryDTO oldGD = new GeneralDictionaryDTO();
        oldGD.setGroupCode(StringUtil.getString(record.getString("groupCode"), "0"));
        oldGD.setProjectId(StringUtil.getString(record.getString("projectId"), "0"));
        oldGD.setDictType(record.getString("dictType"));
        oldGD.setCode(record.getString("code"));
        oldGD.setDictId(record.getString("dictId"));
        List<GeneralDictionary> oldGDs = listGeneralDictionaryNotCoverDuplicate(oldGD);
        if (oldGDs.size() > 1) {
            throw new RuntimeException("查询到多条字典，属性覆盖失败");
        }
        //需要先查询旧的属性，不传的属性不能给覆盖掉
        if (oldGDs.size() == 1) {
            JSONObject oldItemData = StringUtil.isEmpty(oldGDs.get(0).getItemData())?new JSONObject():oldGDs.get(0).getItemData();
            oldItemData.putAll(itemData);
            itemData = oldItemData;
        }
        putItemData(record, uniqueByDictId, fields, childrens, itemData);
        return record;
    }

    private void putItemData(JSONObject record, boolean uniqueByDictId, JSONArray fields, JSONArray childrens, JSONObject itemData) throws Exception {
        for (Object param : fields) {
            if (!generalDictionaryColumm.contains(param)) {
                itemData.put((String) param, record.get(param));
            }
        }
        if (!itemData.isEmpty()) {
            record.put("itemData", itemData);
        }
        if (ObjectUtil.isNotEmpty(childrens)) {
            for (int i = 0; i < childrens.size(); i++) {
                JSONObject children = childrens.getJSONObject(i);
                transferItemData(children, uniqueByDictId);
            }
        }
    }

    /**
     * @param record:
     * @description: 获取code-name的map键值对
     * @author: LuoGuangyi
     * @createTime: 2021/02/01 14:53
     * @return: java.util.HashMap<java.lang.String, java.lang.String>
     * @expression
     */
    @Override
    public HashMap<String, String> queryGDCodeNameMap(GeneralDictionaryDTO record) throws Exception {
        List<GeneralDictionaryDTO> generalDictionaryDTOS = listGeneralDictionary(record);
        HashMap<String, String> codeName = new HashMap<>();
        for (GeneralDictionaryDTO generalDictionaryDTO : generalDictionaryDTOS) {
            codeName.put(generalDictionaryDTO.getCode(), generalDictionaryDTO.getName());
        }
        return codeName;
    }
    /**
     * @description: 获取dictId-name的map键值对
     * @author: LuoGuangyi
     * @createTime: 2021/03/19 16:05
     * @param record:
     * @return: java.util.HashMap<java.lang.String,java.lang.String>
     * @expression
     */
    @Override
    public HashMap<String, String> queryDictIdNameMap(GeneralDictionaryDTO record) throws Exception {
        List<GeneralDictionaryDTO> generalDictionaryDTOS = listGeneralDictionary(record);
        HashMap<String, String> dictIdName = new HashMap<>();
        for (GeneralDictionaryDTO generalDictionaryDTO : generalDictionaryDTOS) {
            dictIdName.put(generalDictionaryDTO.getDictId(), generalDictionaryDTO.getName());
        }
        return dictIdName;
    }

    /**
     * @param record:
     * @description: 查询字段数据（重复不覆盖）
     * @author: LuoGuangyi
     * @createTime: 2021/01/26 15:22
     * @return: java.util.List<com.persagy.dmp.entity.GeneralDictionary>
     * @expression
     */
    public List<GeneralDictionary> listGeneralDictionaryNotCoverDuplicate(GeneralDictionaryDTO record) throws Exception {
        Assert.notBlank(record.getDictType(), "参数 [dict_type] 不能为空");
        QueryWrapper<GeneralDictionary> queryWrapper = new QueryWrapper<>();
        Sort sort = new Sort(GeneralDictionary.COL_SEQUENCE, true);
        queryWrapper.eq(StringUtils.isNotBlank(record.getDictType()), GeneralDictionary.COL_DICT_TYPE, record.getDictType())
                .eq(StringUtils.isNotBlank(record.getCode()), GeneralDictionary.COL_CODE, record.getCode())
                .eq(StringUtils.isNotBlank(record.getDictId()), GeneralDictionary.COL_DICT_ID, record.getDictId())
                .eq(ObjectUtil.isNotEmpty(record.getProjectId()), GeneralDictionary.COL_PROJECT_ID, record.getProjectId())
                .eq(ObjectUtil.isNotEmpty(record.getGroupCode()), GeneralDictionary.COL_GROUP_CODE, record.getGroupCode())
                .orderBy(true, sort.getAsc(), sort.getColumn());
        return generalDictionaryDao.list(queryWrapper);
    }

    public GeneralDictionary getGeneralDictionary(String dictId) throws Exception {
        Assert.notEmpty(dictId,"dictId不能为空！");
        return generalDictionaryDao.getById(dictId);
    }

    /**
     * @param record:删除内容
     * @description: 删除字典
     * @author: LuoGuangyi
     * @createTime: 2021/01/29 15:44
     * @return: void
     * @expression
     */
    @Override
    public void remove(GeneralDictionaryDTO record) throws Exception {
        Assert.notBlank(record.getDictType(), "参数 [dict_type] 不能为空");
        record.setGroupCode(StringUtil.getString(record.getGroupCode(), "0"));
        record.setProjectId(StringUtil.getString(record.getProjectId(), "0"));
        QueryWrapper<GeneralDictionary> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(record.getDictType()), GeneralDictionary.COL_DICT_TYPE, record.getDictType())
                .eq(StringUtils.isNotBlank(record.getDictId()), GeneralDictionary.COL_DICT_ID, record.getDictId())
                .eq(StringUtils.isNotBlank(record.getCode()), GeneralDictionary.COL_CODE, record.getCode())
                .in(ObjectUtil.isNotEmpty(record.getCodes()), GeneralDictionary.COL_CODE, record.getCodes())
                .eq(ObjectUtil.isNotEmpty(record.getProjectId()), GeneralDictionary.COL_PROJECT_ID, record.getProjectId())
                .eq(ObjectUtil.isNotEmpty(record.getGroupCode()), GeneralDictionary.COL_GROUP_CODE, record.getGroupCode());
        boolean result = generalDictionaryDao.remove(queryWrapper);
        Assert.isTrue(result, "未查询到要删除的数据或删除失败");
    }
    @Override
    public void removeById(String dictId) throws Exception {
        Assert.notBlank(dictId, "参数 [dictId] 不能为空");
        boolean result = generalDictionaryDao.removeById(dictId);
        Assert.isTrue(result, "未查询到要删除的数据或删除失败");
    }
}