package com.persagy.dmp.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.dmp.dao.GroupProjectRelDao;
import com.persagy.dmp.entity.GroupProjectRel;
import com.persagy.dmp.service.GroupProjectRelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @description: 集团-项目关联表
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/12/14 17:40
 * @version: V1.0
 **/
@Service
public class GroupProjectRelServiceImpl implements GroupProjectRelService {

    @Autowired
    private GroupProjectRelDao groupProjectRelDao;

    @Override
    @Cacheable(value = "groupCode", key = "#projectId")
    public String queryGroupCode(String projectId) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("project_id", projectId);
        GroupProjectRel groupProjectRel = groupProjectRelDao.getOne(queryWrapper);
        return groupProjectRel.getGroupCode().toUpperCase();
    }

    @Override
    @Cacheable(value = "groupCode", key = "#groupCode")
    public List<String> queryProjectIds(String groupCode) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("group_code", groupCode);
        List<GroupProjectRel> groupProjectRelList = groupProjectRelDao.list(queryWrapper);
        List<String> projectIds = new ArrayList<>();
        if (!CollectionUtils.isEmpty(groupProjectRelList)) {
            projectIds = groupProjectRelList.stream().map(GroupProjectRel::getProjectId).collect(Collectors.toList());
        }
        return projectIds;
    }

    @Override
    public List<GroupProjectRel> queryGroupProjectRel() {
        return groupProjectRelDao.list();
    }

}
