package com.persagy.dmp.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.google.common.collect.Lists;
import com.persagy.dmp.cache.LocationCache;
import com.persagy.dmp.dao.InstanceDao;
import com.persagy.dmp.dto.*;
import com.persagy.dmp.enumeration.EnumGraphCode;
import com.persagy.dmp.service.LocationService;
import com.persagy.dmp.service.ObjectService;
import com.persagy.dmp.service.RelationService;
import com.persagy.dmp.utils.ItvJsonUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

/**
 * @description: 位置接口实现类
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/9/22 15:43
 * @version: V1.0
 **/
@Service
public class LocationServiceImpl implements LocationService {

    @Autowired
    private InstanceDao instanceDao;

    @Autowired
    private RelationService relationService;

    @Autowired
    private ObjectService objectService;

    @Autowired
    private LocationCache locationCache;


    @Override
    public EquipLocationDTO queryEquipLocation(String groupCode, String projectId, String equipId) throws Exception {
        List<EquipLocationDTO> equipLocationDTOList = this.queryEquipLocation(groupCode, projectId, Arrays.asList(equipId));
        if (CollectionUtils.isNotEmpty(equipLocationDTOList)) {
            return equipLocationDTOList.get(0);
        }
        return null;
    }

    @Override
    public List<EquipLocationDTO> queryEquipLocation(String groupCode, String projectId, List<String> equipIds) throws Exception {
        List<EquipLocationDTO> equipLocationDTOList = new ArrayList<>();
        if (CollectionUtils.isEmpty(equipIds)) {
            return equipLocationDTOList;
        }

        //查询缓存
        List<EquipLocationDTO> cacheEquipLocationDTOList = locationCache.getEquipLocation(groupCode, projectId, equipIds);
        if (CollectionUtils.isNotEmpty(cacheEquipLocationDTOList)) {
            if (equipIds.size() == cacheEquipLocationDTOList.size()) {
                return cacheEquipLocationDTOList;
            } else {
                List<String> cacheEquipIds = cacheEquipLocationDTOList.stream().map(EquipLocationDTO::getEquipId).collect(Collectors.toList());
                //取差集
                equipIds = Lists.newLinkedList(equipIds);
                equipIds.removeAll(cacheEquipIds);
            }
        }

        //对象id列表
        List<String> objectIdList = new ArrayList<>();

        //查询设备所在空间:默认业务空间
        List<ObjRelationDTO> eq2SpRelList = relationService.queryRelationByFromIds(groupCode, projectId, EnumGraphCode.MechInArch.getCode(), "Eq2Sp", equipIds);
        //空间id列表
        Set<String> spaceIdSet;
        //设备空间对应关系
        Map<String, String> eq2SpMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(eq2SpRelList)) {
            eq2SpRelList.forEach(o -> {
                eq2SpMap.put(o.getObjFrom(), o.getObjTo());
            });
            spaceIdSet = eq2SpRelList.stream().map(ObjRelationDTO::getObjTo).collect(Collectors.toSet());
            if (CollectionUtils.isNotEmpty(spaceIdSet)) {
                objectIdList.addAll(spaceIdSet);
            }
        }

        //楼层id列表
        Set<String> floorIdSet;
        //设备楼层对应关系
        Map<String, String> eq2FlMap = new HashMap<>();
        //查询设备所在楼层
        List<ObjRelationDTO> eq2FlRelList = instanceDao.queryRelationByFromIds(groupCode, projectId, EnumGraphCode.MechInArch.getCode(), "Eq2Fl", equipIds);
        if (CollectionUtils.isNotEmpty(eq2FlRelList)) {
            eq2FlRelList.forEach(o -> {
                eq2FlMap.put(o.getObjFrom(), o.getObjTo());
            });
            floorIdSet = eq2FlRelList.stream().map(ObjRelationDTO::getObjTo).collect(Collectors.toSet());
            if (CollectionUtils.isNotEmpty(floorIdSet)) {
                objectIdList.addAll(floorIdSet);
            }
        }

        //建筑id列表
        Set<String> buildIdSet;
        //设备建筑对应关系
        Map<String, String> eq2BdMap = new HashMap<>();
        //查询设备所在建筑
        List<ObjRelationDTO> eq2BdRelList = relationService.queryRelationByFromIds(groupCode, projectId, EnumGraphCode.MechInArch.getCode(), "Eq2Bd", equipIds);
        if (CollectionUtils.isNotEmpty(eq2BdRelList)) {
            eq2BdRelList.forEach(o -> {
                eq2BdMap.put(o.getObjFrom(), o.getObjTo());
            });
            buildIdSet = eq2BdRelList.stream().map(ObjRelationDTO::getObjTo).collect(Collectors.toSet());
            if (CollectionUtils.isNotEmpty(buildIdSet)) {
                objectIdList.addAll(buildIdSet);
            }
        }

        //查询对象详情
        Map<String, ObjectInfoDTO> id2ObjectInfoMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(objectIdList)) {
            id2ObjectInfoMap = objectService.queryObjectMap(groupCode, projectId, objectIdList);
        }

        String spaceId, floorId, buildId;
        ObjectInfoDTO objectInfoDTO;
        for (String equipId : equipIds) {
            EquipLocationDTO equipPositionDTO = new EquipLocationDTO();

            equipPositionDTO.setProjectId(projectId);
            equipPositionDTO.setEquipId(equipId);

            //空间id、名称、编码
            if (eq2SpMap.containsKey(equipId)) {
                spaceId = eq2SpMap.get(equipId);
                equipPositionDTO.setSpaceId(spaceId);
                if (id2ObjectInfoMap.containsKey(spaceId)) {
                    objectInfoDTO = id2ObjectInfoMap.get(spaceId);
                    equipPositionDTO.setSpaceCode(StrUtil.isBlank(objectInfoDTO.getIbmsxianshibianma()) ? objectInfoDTO.getLocalId() : objectInfoDTO.getIbmsxianshibianma());
                    equipPositionDTO.setSpaceName(StrUtil.isBlank(objectInfoDTO.getIbmsxianshimingcheng()) ? objectInfoDTO.getLocalName() : objectInfoDTO.getIbmsxianshimingcheng());
                }
            }

            //楼层id、名称、编码
            if (eq2FlMap.containsKey(equipId)) {
                floorId = eq2FlMap.get(equipId);
                equipPositionDTO.setFloorId(floorId);
                if (id2ObjectInfoMap.containsKey(floorId)) {
                    objectInfoDTO = id2ObjectInfoMap.get(floorId);
                    equipPositionDTO.setFloorCode(StrUtil.isBlank(objectInfoDTO.getIbmsxianshibianma()) ? objectInfoDTO.getLocalId() : objectInfoDTO.getIbmsxianshibianma());
                    equipPositionDTO.setFloorName(StrUtil.isBlank(objectInfoDTO.getIbmsxianshimingcheng()) ? objectInfoDTO.getLocalName() : objectInfoDTO.getIbmsxianshimingcheng());
                }
            }

            //建筑id、名称、编码
            if (eq2BdMap.containsKey(equipId)) {
                buildId = eq2BdMap.get(equipId);
                equipPositionDTO.setBuildId(buildId);
                if (id2ObjectInfoMap.containsKey(buildId)) {
                    objectInfoDTO = id2ObjectInfoMap.get(buildId);
                    equipPositionDTO.setBuildCode(StrUtil.isBlank(objectInfoDTO.getIbmsxianshibianma()) ? objectInfoDTO.getLocalId() : objectInfoDTO.getIbmsxianshibianma());
                    equipPositionDTO.setBuildName(StrUtil.isBlank(objectInfoDTO.getIbmsxianshimingcheng()) ? objectInfoDTO.getLocalName() : objectInfoDTO.getIbmsxianshimingcheng());
                }
            }
            equipLocationDTOList.add(equipPositionDTO);
        }

        //异步存入缓存
        CompletableFuture.runAsync(() -> {
            locationCache.putEquipLocation(groupCode, projectId, equipLocationDTOList);
        });

        if (CollectionUtils.isNotEmpty(cacheEquipLocationDTOList)) {
            //缓存取到位置并入结果集
            equipLocationDTOList.addAll(cacheEquipLocationDTOList);
        }
        return equipLocationDTOList;
    }

    @Override
    public EquipBelongModularDTO queryEquipModularBox(String groupCode, String projectId, String equipId) throws Exception {
        return queryEquipModularBox(groupCode, projectId, Collections.singletonList(equipId)).get(equipId);
    }

    @Override
    public HashMap<String, EquipBelongModularDTO> queryEquipModularBox(String groupCode, String projectId, List<String> equipIds) throws Exception {
        List<EquipBelongModularDTO> equipLocationDTOList = new ArrayList<>();
        //设备id to 所属模块&配电箱
        HashMap<String, EquipBelongModularDTO> equipModularBoxMap = new HashMap<>();
        if (CollectionUtils.isEmpty(equipIds)) {
            return equipModularBoxMap;
        }

        //查询缓存
        List<EquipBelongModularDTO> cacheEquipBelongModularDTOList = locationCache.getEquipBelongModular(groupCode, projectId, equipIds);
        if (CollectionUtils.isNotEmpty(cacheEquipBelongModularDTOList)) {
            if (equipIds.size() == cacheEquipBelongModularDTOList.size()) {
                for (EquipBelongModularDTO belongModularBoxDTO : cacheEquipBelongModularDTOList
                ) {
                    equipModularBoxMap.put(belongModularBoxDTO.getEquipId(), belongModularBoxDTO);
                }
                return equipModularBoxMap;
            } else {
                List<String> cacheEquipIds = cacheEquipBelongModularDTOList.stream().map(EquipBelongModularDTO::getEquipId).collect(Collectors.toList());
                //取差集
                equipIds = Lists.newLinkedList(equipIds);
                equipIds.removeAll(cacheEquipIds);
            }
        }

        //对象id列表
        List<String> objectIdList = new ArrayList<>();

        //查询设备所在空间:所在模块
//        List<ObjRelationDTO> eq2MdRelList = relationService.queryRelationByFromIds(groupCode, projectId, EnumGraphCode.MechCtrl.getCode(), "EqCtrl", equipIds);
        List<ObjRelationDTO> eq2MdRelList = relationService.queryRelationByToIds(groupCode, projectId, EnumGraphCode.MechCtrl.getCode(), "EqCtrl", equipIds);
//        List<ObjRelationDTO> eq2MdRelList = rwdServerClientNewService.queryRelation(groupCode, projectId, EnumGraphCode.MechCtrl.getCode(),"EqCtrl",null,equipIds);
        //模块id列表
        Set<String> modularIdSet = new HashSet<>();
        //设备所属模块对应关系
        Map<String, String> eq2ModularIdMap = new HashMap<>();
        // 模块 设备 的对应关系
        Map<String, List<String>> modularId2EqsMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(eq2MdRelList)) {
            eq2MdRelList.forEach(o -> {
                eq2ModularIdMap.put(o.getObjTo(), o.getObjFrom());
                modularId2EqsMap.computeIfAbsent(o.getObjFrom(), k -> Lists.newLinkedList()).add(o.getObjTo());
            });
            eq2MdRelList.stream().map(ObjRelationDTO::getObjFrom).forEach(modularIdSet::add);
            if (CollectionUtils.isNotEmpty(modularIdSet)) {
                objectIdList.addAll(modularIdSet);
            }
        }

        //配电箱id列表
        Set<String> belongPowerBoxIdSet = new HashSet<>();
        //设备配电箱对应关系
        Map<String, String> eq2PowerBoxMap = new HashMap<>();
        // 配电箱 设备 的对应关系
        Map<String, List<String>> powerBox2EqsMap = new HashMap<>();
        //查询设备所在配电箱
        List<ObjRelationDTO> eq2PowerBoxRelList = instanceDao.queryRelationByToIds(groupCode, projectId, EnumGraphCode.MechPower.getCode(), "EqNormal", Lists.newLinkedList(modularIdSet));

        if (CollectionUtils.isNotEmpty(eq2PowerBoxRelList)) {
            eq2PowerBoxRelList.forEach(o -> {
                List<String> modularEquipIds = modularId2EqsMap.getOrDefault(o.getObjTo(), Collections.emptyList());
                for (String equipId : modularEquipIds) {
                    eq2PowerBoxMap.put(equipId, o.getObjFrom());
                    powerBox2EqsMap.computeIfAbsent(o.getObjFrom(), k -> Lists.newLinkedList()).add(equipId);
                }
            });
            eq2PowerBoxRelList.stream().map(ObjRelationDTO::getObjFrom).forEach(belongPowerBoxIdSet::add);
            if (CollectionUtils.isNotEmpty(belongPowerBoxIdSet)) {
                objectIdList.addAll(belongPowerBoxIdSet);
            }
        }

        //电井id列表
        Set<String> powerWellsIdSet = new HashSet<>();
        // 设备与电井关系
        Map<String, String> eq2PowerWellsMap = new HashMap<>();
        // 查询设备与电井关系
        List<ObjRelationDTO> eq2PowerWellsRelList = instanceDao.queryRelationByFromIds(groupCode, projectId, EnumGraphCode.MechInArch.getCode(), "Eq2Sp", Lists.newLinkedList(belongPowerBoxIdSet));

        if (CollectionUtils.isNotEmpty(eq2PowerWellsRelList)) {
            eq2PowerWellsRelList.forEach(o -> {
                List<String> powerBoxEquipIds = powerBox2EqsMap.getOrDefault(o.getObjFrom(), Collections.emptyList());
                for (String equipId : powerBoxEquipIds) {
                    eq2PowerWellsMap.put(equipId, o.getObjTo());
                }
            });
            eq2PowerWellsRelList.stream().map(ObjRelationDTO::getObjTo).forEach(powerWellsIdSet::add);
            if (CollectionUtils.isNotEmpty(powerWellsIdSet)) {
                objectIdList.addAll(powerWellsIdSet);
            }
        }

        //查询对象详情
        Map<String, ObjectInfoDTO> id2ObjectInfoMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(objectIdList)) {
            id2ObjectInfoMap = objectService.queryObjectMap(groupCode, projectId, objectIdList);
        }

        String modularId, powerBoxId, powerWellsId;
        ObjectInfoDTO objectInfoDTO;
        for (String equipId : equipIds) {
            EquipBelongModularDTO equipBelongModularDTO = new EquipBelongModularDTO();

            equipBelongModularDTO.setProjectId(projectId);
            equipBelongModularDTO.setEquipId(equipId);
            //自己的回路编码，回路名称
            if (id2ObjectInfoMap.containsKey(equipId)) {
                objectInfoDTO = id2ObjectInfoMap.get(equipId);
                equipBelongModularDTO.setLocalId(getObjectInfoCode(objectInfoDTO, groupCode));
                equipBelongModularDTO.setLocalName(getObjectInfoName(objectInfoDTO, groupCode));
            }

            //模块id、名称、编码
            if (eq2ModularIdMap.containsKey(equipId)) {
                modularId = eq2ModularIdMap.get(equipId);
                equipBelongModularDTO.setBelongModularId(modularId);
                if (id2ObjectInfoMap.containsKey(modularId)) {
                    objectInfoDTO = id2ObjectInfoMap.get(modularId);

                    equipBelongModularDTO.setBelongModularCode(getObjectInfoCode(objectInfoDTO, groupCode));
                    equipBelongModularDTO.setBelongModularName(getObjectInfoName(objectInfoDTO, groupCode));
                }
            }
            //楼层id、名称、编码
            if (eq2PowerBoxMap.containsKey(equipId)) {
                powerBoxId = eq2PowerBoxMap.get(equipId);
                equipBelongModularDTO.setBelongPowerBoxId(powerBoxId);
                if (id2ObjectInfoMap.containsKey(powerBoxId)) {
                    objectInfoDTO = id2ObjectInfoMap.get(powerBoxId);

                    equipBelongModularDTO.setBelongPowerBoxCode(getObjectInfoCode(objectInfoDTO, groupCode));
                    equipBelongModularDTO.setBelongPowerBoxName(getObjectInfoName(objectInfoDTO, groupCode));
                }
            }
            //楼层id、名称、编码
            if (eq2PowerWellsMap.containsKey(equipId)) {
                powerWellsId = eq2PowerWellsMap.get(equipId);
                equipBelongModularDTO.setPowerWellsId(powerWellsId);
                if (id2ObjectInfoMap.containsKey(powerWellsId)) {
                    objectInfoDTO = id2ObjectInfoMap.get(powerWellsId);

                    equipBelongModularDTO.setPowerWellsCode(getObjectInfoCode(objectInfoDTO, groupCode));
                    equipBelongModularDTO.setPowerWellsName(getObjectInfoName(objectInfoDTO, groupCode));
                }
            }
            equipLocationDTOList.add(equipBelongModularDTO);
        }

        //异步存入缓存
        CompletableFuture.runAsync(() -> {
            locationCache.putModularBox(groupCode, projectId, equipLocationDTOList);
        });


        if (CollectionUtils.isNotEmpty(cacheEquipBelongModularDTOList)) {
            //缓存取到位置并入结果集
            equipLocationDTOList.addAll(cacheEquipBelongModularDTOList);
        }

        for (EquipBelongModularDTO belongModularBoxDTO : equipLocationDTOList) {
            equipModularBoxMap.put(belongModularBoxDTO.getEquipId(), belongModularBoxDTO);
        }

        return equipModularBoxMap;
    }

    private String getObjectInfoCode(ObjectInfoDTO objectInfoDTO, String groupCode) {
        if ("wd".equalsIgnoreCase(groupCode)) {
            if (Objects.nonNull(objectInfoDTO.getWdHuiyunbendibianma())) {
                return objectInfoDTO.getWdHuiyunbendibianma();
            }
        }

        return ObjectUtils.defaultIfNull(objectInfoDTO.getIbmsxianshibianma(), objectInfoDTO.getLocalId());
    }

    private String getObjectInfoName(ObjectInfoDTO objectInfoDTO, String groupCode) {
        if ("wd".equalsIgnoreCase(groupCode)) {
            if (Objects.nonNull(objectInfoDTO.getWdHuiyunbendimingcheng())) {
                return objectInfoDTO.getWdHuiyunbendimingcheng();
            }
        }

        if (Objects.nonNull(objectInfoDTO.getIbmsxianshimingcheng())) {
            return objectInfoDTO.getIbmsxianshimingcheng();
        }

        return ObjectUtils.defaultIfNull(objectInfoDTO.getLocalName(), objectInfoDTO.getName());
    }

    @Override
    public Map<String, EquipLocationDTO> queryEquipLocationMap(String groupCode, String projectId, List<String> equipIds) throws Exception {
        List<EquipLocationDTO> equipLocationDTOList = this.queryEquipLocation(groupCode, projectId, equipIds);
        Map<String, EquipLocationDTO> locationMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(equipLocationDTOList)) {
            equipLocationDTOList.forEach(o -> {
                locationMap.put(o.getEquipId(), o);
            });
        }
        return locationMap;
    }

    @Override
    public SystemLocationDTO querySystemLocation(String groupCode, String projectId, String systemId) throws Exception {
        List<SystemLocationDTO> systemLocationDTOList = this.querySystemLocation(groupCode, projectId, Arrays.asList(systemId));
        if (CollectionUtils.isEmpty(systemLocationDTOList)) {
            return systemLocationDTOList.get(0);
        }
        return null;
    }

    @Override
    public List<SystemLocationDTO> querySystemLocation(String groupCode, String projectId, List<String> systemIds) throws Exception {
        List<SystemLocationDTO> systemLocationDTOList = new ArrayList<>();
        if (CollectionUtils.isEmpty(systemIds)) {
            return systemLocationDTOList;
        }

        //查询缓存
        List<SystemLocationDTO> cacheSystemLocationList = locationCache.getSystemLocation(groupCode, projectId, systemIds);
        if (CollectionUtils.isNotEmpty(cacheSystemLocationList)) {
            if (systemIds.size() == cacheSystemLocationList.size()) {
                return cacheSystemLocationList;
            } else {
                List<String> cacheEquipIds = cacheSystemLocationList.stream().map(SystemLocationDTO::getSystemId).collect(Collectors.toList());
                //取差集
                systemIds = Lists.newLinkedList(systemIds);
                systemIds.removeAll(cacheEquipIds);
            }
        }

        //对象id列表
        List<String> objectIdList = new ArrayList<>();

        //查询系统所在空间
        List<ObjRelationDTO> sy2SpRelList = relationService.queryRelationByFromIds(groupCode, projectId, EnumGraphCode.MechInArch.getCode(), "Sy2Sp", systemIds);
        //空间id列表
        Set<String> spaceIdSet;
        //系统空间对应关系
        Map<String, String> sy2SpMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(sy2SpRelList)) {
            sy2SpRelList.forEach(o -> {
                sy2SpMap.put(o.getObjFrom(), o.getObjTo());
            });
            spaceIdSet = sy2SpRelList.stream().map(ObjRelationDTO::getObjTo).collect(Collectors.toSet());
            if (CollectionUtils.isNotEmpty(spaceIdSet)) {
                objectIdList.addAll(spaceIdSet);
            }
        }

        //楼层id列表
        Set<String> floorIdSet;
        //系统楼层对应关系
        Map<String, String> sy2FlMap = new HashMap<>();
        //查询系统所在楼层
        List<ObjRelationDTO> sy2FlRelList = instanceDao.queryRelationByFromIds(groupCode, projectId, EnumGraphCode.MechInArch.getCode(), "Sy2Fl", systemIds);
        if (CollectionUtils.isNotEmpty(sy2FlRelList)) {
            sy2FlRelList.forEach(o -> {
                sy2FlMap.put(o.getObjFrom(), o.getObjTo());
            });
            floorIdSet = sy2FlRelList.stream().map(ObjRelationDTO::getObjTo).collect(Collectors.toSet());
            if (CollectionUtils.isNotEmpty(floorIdSet)) {
                objectIdList.addAll(floorIdSet);
            }
        }

        //建筑id列表
        Set<String> buildIdSet;
        //设备建筑对应关系
        Map<String, String> sy2BdMap = new HashMap<>();
        //查询系统所在建筑
        List<ObjRelationDTO> sy2BdRelList = relationService.queryRelationByFromIds(groupCode, projectId, EnumGraphCode.MechInArch.getCode(), "Sy2Bd", systemIds);
        if (CollectionUtils.isNotEmpty(sy2BdRelList)) {
            sy2BdRelList.forEach(o -> {
                sy2BdMap.put(o.getObjFrom(), o.getObjTo());
            });
            buildIdSet = sy2BdRelList.stream().map(ObjRelationDTO::getObjTo).collect(Collectors.toSet());
            if (CollectionUtils.isNotEmpty(buildIdSet)) {
                objectIdList.addAll(buildIdSet);
            }
        }

        //查询对象详情
        Map<String, ObjectInfoDTO> id2ObjectInfoMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(objectIdList)) {
            id2ObjectInfoMap = objectService.queryObjectMap(groupCode, projectId, objectIdList);
        }

        String spaceId, floorId, buildId;
        ObjectInfoDTO objectInfoDTO;
        for (String systemId : systemIds) {
            SystemLocationDTO systemLocationDTO = new SystemLocationDTO();

            systemLocationDTO.setProjectId(projectId);
            systemLocationDTO.setSystemId(systemId);

            //空间id、名称、编码
            if (sy2SpMap.containsKey(systemId)) {
                spaceId = sy2SpMap.get(systemId);
                systemLocationDTO.setSpaceId(spaceId);
                if (id2ObjectInfoMap.containsKey(spaceId)) {
                    objectInfoDTO = id2ObjectInfoMap.get(spaceId);
                    systemLocationDTO.setSpaceCode(StrUtil.isBlank(objectInfoDTO.getIbmsxianshibianma()) ? objectInfoDTO.getLocalId() : objectInfoDTO.getIbmsxianshibianma());
                    systemLocationDTO.setSpaceName(StrUtil.isBlank(objectInfoDTO.getIbmsxianshimingcheng()) ? objectInfoDTO.getLocalName() : objectInfoDTO.getIbmsxianshimingcheng());
                }
            }

            //楼层id、名称、编码
            if (sy2FlMap.containsKey(systemId)) {
                floorId = sy2FlMap.get(systemId);
                systemLocationDTO.setFloorId(floorId);
                if (id2ObjectInfoMap.containsKey(floorId)) {
                    objectInfoDTO = id2ObjectInfoMap.get(floorId);
                    systemLocationDTO.setFloorCode(StrUtil.isBlank(objectInfoDTO.getIbmsxianshibianma()) ? objectInfoDTO.getLocalId() : objectInfoDTO.getIbmsxianshibianma());
                    systemLocationDTO.setFloorName(StrUtil.isBlank(objectInfoDTO.getIbmsxianshimingcheng()) ? objectInfoDTO.getLocalName() : objectInfoDTO.getIbmsxianshimingcheng());
                }
            }

            //建筑id、名称、编码
            if (sy2BdMap.containsKey(systemId)) {
                buildId = sy2BdMap.get(systemId);
                systemLocationDTO.setBuildId(buildId);
                if (id2ObjectInfoMap.containsKey(buildId)) {
                    objectInfoDTO = id2ObjectInfoMap.get(buildId);
                    systemLocationDTO.setBuildCode(StrUtil.isBlank(objectInfoDTO.getIbmsxianshibianma()) ? objectInfoDTO.getLocalId() : objectInfoDTO.getIbmsxianshibianma());
                    systemLocationDTO.setBuildName(StrUtil.isBlank(objectInfoDTO.getIbmsxianshimingcheng()) ? objectInfoDTO.getLocalName() : objectInfoDTO.getIbmsxianshimingcheng());
                }
            }
            systemLocationDTOList.add(systemLocationDTO);
        }

        //异步存入缓存
        CompletableFuture.runAsync(() -> {
            locationCache.putSystemLocation(groupCode, projectId, systemLocationDTOList);
        });

        if (CollectionUtils.isNotEmpty(cacheSystemLocationList)) {
            //缓存取到位置并入结果集
            systemLocationDTOList.addAll(cacheSystemLocationList);
        }

        return systemLocationDTOList;
    }

    @Override
    public Map<String, SystemLocationDTO> querySystemLocationMap(String groupCode, String projectId, List<String> systemIds) throws Exception {
        List<SystemLocationDTO> equipLocationDTOList = this.querySystemLocation(groupCode, projectId, systemIds);
        Map<String, SystemLocationDTO> locationMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(equipLocationDTOList)) {
            equipLocationDTOList.forEach(o -> {
                locationMap.put(o.getSystemId(), o);
            });
        }
        return locationMap;
    }

    @Override
    public SpaceLocationDTO querySpaceLocation(String groupCode, String projectId, String spaceId) throws Exception {
        List<SpaceLocationDTO> spaceLocationDTOList = this.querySpaceLocation(groupCode, projectId, Arrays.asList(spaceId));
        if (CollectionUtils.isNotEmpty(spaceLocationDTOList)) {
            return spaceLocationDTOList.get(0);
        }
        return null;
    }

    @Override
    public List<SpaceLocationDTO> querySpaceLocation(String groupCode, String projectId, List<String> spaceIds) throws Exception {
        List<SpaceLocationDTO> spaceLocationList = new ArrayList<>();
        if (CollectionUtils.isEmpty(spaceIds)) {
            return spaceLocationList;
        }

        //查询缓存
        List<SpaceLocationDTO> cacheSpaceLocationList = locationCache.getSpaceLocation(groupCode, projectId, spaceIds);
        if (CollectionUtils.isNotEmpty(cacheSpaceLocationList)) {
            if (spaceIds.size() == cacheSpaceLocationList.size()) {
                return cacheSpaceLocationList;
            } else {
                List<String> cacheEquipIds = cacheSpaceLocationList.stream().map(SpaceLocationDTO::getSpaceId).collect(Collectors.toList());
                //取差集
                spaceIds = Lists.newLinkedList(spaceIds);
                spaceIds.removeAll(cacheEquipIds);
            }
        }

        //空间id列表
        Set<String> spaceIdSet = new HashSet<>(spaceIds);
        //对象id列表
        List<String> objectIdList = new ArrayList<>(spaceIdSet);

        //楼层id列表
        Set<String> floorIdSet;
        //空间楼层对应关系
        Map<String, String> sp2FlMap = new HashMap<>();

        //查询空间所在楼层
        List<ObjRelationDTO> fl2SpRelList = relationService.queryRelationByToIds(groupCode, projectId, EnumGraphCode.ArchSubset.getCode(), "Fl2Sp", new ArrayList<>(spaceIdSet));
        if (CollectionUtils.isNotEmpty(fl2SpRelList)) {
            fl2SpRelList.forEach(o -> {
                sp2FlMap.put(o.getObjTo(), o.getObjFrom());
            });
            floorIdSet = fl2SpRelList.stream().map(ObjRelationDTO::getObjFrom).collect(Collectors.toSet());
            if (CollectionUtils.isNotEmpty(floorIdSet)) {
                objectIdList.addAll(floorIdSet);
            }
        }

        //建筑id列表
        Set<String> buildIdSet;
        //空间建筑对应关系
        Map<String, String> sp2BdMap = new HashMap<>();

        //查询楼层所在建筑
        List<ObjRelationDTO> bd2SpRelList = relationService.queryRelationByToIds(groupCode, projectId, EnumGraphCode.ArchSubset.getCode(), "Bd2Sp", new ArrayList<>(spaceIdSet));
        if (CollectionUtils.isNotEmpty(bd2SpRelList)) {
            bd2SpRelList.forEach(o ->
                    sp2BdMap.put(o.getObjTo(), o.getObjFrom())
            );
            buildIdSet = bd2SpRelList.stream().map(ObjRelationDTO::getObjFrom).collect(Collectors.toSet());
            if (CollectionUtils.isNotEmpty(buildIdSet)) {
                objectIdList.addAll(buildIdSet);
            }
        }

        //查询对象详情
        Map<String, ObjectInfoDTO> id2ObjectInfoMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(objectIdList)) {
            id2ObjectInfoMap = objectService.queryObjectMap(groupCode, projectId, objectIdList);
        }

        String floorId, buildId;
        for (String spaceId : spaceIds) {
            SpaceLocationDTO spaceLocation = new SpaceLocationDTO();

            spaceLocation.setProjectId(projectId);

            //空间id、名称
            spaceLocation.setSpaceId(spaceId);
            if (id2ObjectInfoMap.containsKey(spaceId)) {
                ObjectInfoDTO objectInfo = id2ObjectInfoMap.get(spaceId);
                spaceLocation.setSpaceCode(StrUtil.isBlank(objectInfo.getIbmsxianshibianma()) ? objectInfo.getLocalId() : objectInfo.getIbmsxianshibianma());
                spaceLocation.setSpaceName(StrUtil.isBlank(objectInfo.getIbmsxianshimingcheng()) ? objectInfo.getLocalName() : objectInfo.getIbmsxianshimingcheng());
            }

            //楼层id、名称
            if (sp2FlMap.containsKey(spaceId)) {
                floorId = sp2FlMap.get(spaceId);
                spaceLocation.setFloorId(floorId);
                if (id2ObjectInfoMap.containsKey(floorId)) {
                    ObjectInfoDTO objectInfo = id2ObjectInfoMap.get(floorId);
                    spaceLocation.setFloorCode(StrUtil.isBlank(objectInfo.getIbmsxianshibianma()) ? objectInfo.getLocalId() : objectInfo.getIbmsxianshibianma());
                    spaceLocation.setFloorName(StrUtil.isBlank(objectInfo.getIbmsxianshimingcheng()) ? objectInfo.getLocalName() : objectInfo.getIbmsxianshimingcheng());
                }
            }

            //建筑id、名称
            if (sp2BdMap.containsKey(spaceId)) {
                buildId = sp2BdMap.get(spaceId);
                spaceLocation.setBuildId(buildId);
                if (id2ObjectInfoMap.containsKey(buildId)) {
                    ObjectInfoDTO objectInfo = id2ObjectInfoMap.get(buildId);
                    spaceLocation.setBuildCode(StrUtil.isBlank(objectInfo.getIbmsxianshibianma()) ? objectInfo.getLocalId() : objectInfo.getIbmsxianshibianma());
                    spaceLocation.setBuildName(StrUtil.isBlank(objectInfo.getIbmsxianshimingcheng()) ? objectInfo.getLocalName() : objectInfo.getIbmsxianshimingcheng());
                }
            }
            spaceLocationList.add(spaceLocation);
        }

        //异步存入缓存
        CompletableFuture.runAsync(() ->
                locationCache.putSpaceLocation(groupCode, projectId, spaceLocationList)
        );

        if (CollectionUtils.isNotEmpty(cacheSpaceLocationList)) {
            //缓存取到位置并入结果集
            spaceLocationList.addAll(cacheSpaceLocationList);
        }

        return spaceLocationList;
    }

    @Override
    public Map<String, SpaceLocationDTO> querySpaceLocationMap(String groupCode, String projectId, List<String> spaceIds) throws Exception {
        List<SpaceLocationDTO> spaceLocationDTOList = this.querySpaceLocation(groupCode, projectId, spaceIds);
        Map<String, SpaceLocationDTO> locationMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(spaceLocationDTOList)) {
            spaceLocationDTOList.forEach(o ->
                    locationMap.put(o.getSpaceId(), o)
            );
        }
        return locationMap;
    }

    @Override
    public Map<String, LocationDTO> queryObjLocation(String groupCode, String projectId, List<String> objIds) throws Exception {
        JSONArray equipArray = objectService.queryObjectList(groupCode, projectId, objIds);
        List<LocationDTO> locationDTOS = ItvJsonUtil.jsonToList(equipArray.toJSONString(), LocationDTO.class);
        Map<String, List<LocationDTO>> objTypeMap = locationDTOS.stream().collect(Collectors.groupingBy(LocationDTO::getObjType));
        Map<String, LocationDTO> locationMap = locationDTOS.stream().collect(Collectors.toMap(LocationDTO::getId, item -> item));
        if (ObjectUtil.isNotEmpty(objTypeMap.get("equipment"))) {
            List<String> equipmentObjIds = objTypeMap.get("equipment").stream().map(LocationDTO::getId).distinct().collect(Collectors.toList());
            //查询设备对象位置信息
            List<EquipLocationDTO> equipLocationDTOS = queryEquipLocation(groupCode, projectId, equipmentObjIds);
            for (EquipLocationDTO equipLocationDTO : equipLocationDTOS) {
                locationMap.merge(equipLocationDTO.getEquipId(), equipLocationDTO, (oldValue, newValue) -> {
                    BeanUtils.copyProperties(oldValue, newValue);
                    return newValue;
                });
            }
        }
        if (ObjectUtil.isNotEmpty(objTypeMap.get("space"))) {
            List<String> spaceObjIds = objTypeMap.get("space").stream().map(LocationDTO::getId).distinct().collect(Collectors.toList());
            //查询空间对象位置信息
            List<SpaceLocationDTO> spaceLocationDTOS = querySpaceLocation(groupCode, projectId, spaceObjIds);
            for (SpaceLocationDTO spaceLocationDTO : spaceLocationDTOS) {
                locationMap.merge(spaceLocationDTO.getSpaceId(), spaceLocationDTO, (oldValue, newValue) -> {
                    BeanUtils.copyProperties(oldValue, newValue);
                    return newValue;
                });
            }
        }
        if (ObjectUtil.isNotEmpty(objTypeMap.get("system"))) {
            List<String> systemObjIds = objTypeMap.get("system").stream().map(LocationDTO::getId).distinct().collect(Collectors.toList());
            //查询系统对象位置信息
            List<SystemLocationDTO> systemLocationDTOS = querySystemLocation(groupCode, projectId, systemObjIds);
            for (SystemLocationDTO systemLocationDTO : systemLocationDTOS) {
                locationMap.merge(systemLocationDTO.getSystemId(), systemLocationDTO, (oldValue, newValue) -> {
                    BeanUtils.copyProperties(oldValue, newValue);
                    return newValue;
                });
            }
        }
        return locationMap;
    }
}
