package com.persagy.dmp.service.impl;

import static com.persagy.dmp.constant.GeneralDictionaryConstant.LOGICALG_ROUPING;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.OptionalInt;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.persagy.dmp.constant.DmpConstant;
import com.persagy.dmp.dto.EquipBelongModularDTO;
import com.persagy.dmp.dto.EquipLocationDTO;
import com.persagy.dmp.dto.FuncIdQueryDTO;
import com.persagy.dmp.dto.GeneralDictionaryDTO;
import com.persagy.dmp.dto.InfoPointDictDTO;
import com.persagy.dmp.dto.SpaceLocationDTO;
import com.persagy.dmp.dto.SystemLocationDTO;
import com.persagy.dmp.dto.logicalGrouping.LogicalGroupingDTO;
import com.persagy.dmp.dto.logicalGrouping.LogicalObjectDTO;
import com.persagy.dmp.entity.LogicalGrouping;
import com.persagy.dmp.mapper.LogicalGroupingMapper;
import com.persagy.dmp.service.DictService;
import com.persagy.dmp.service.GeneralDictionaryService;
import com.persagy.dmp.service.LocationService;
import com.persagy.dmp.service.LogicalGroupingService;
import com.persagy.dmp.service.LogicalObjectService;
import com.persagy.dmp.service.ObjectService;
import com.persagy.dmp.service.RelationService;
import com.persagy.dmp.utils.BeanDtoVoUtils;
import com.persagy.dmp.utils.CovertUtil;
import com.persagy.dmp.utils.DataUtils;
import com.persagy.dmp.utils.ItvJsonUtil;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;

@Service
public class LogicalGroupingServiceImpl extends ServiceImpl<LogicalGroupingMapper, LogicalGrouping> implements LogicalGroupingService {

    @Autowired
    LogicalObjectService logicalObjectService;

    @Autowired
    ObjectService objectService;

    @Autowired
    LocationService locationService;


    @Autowired
    RelationService relationService;

    @Autowired
    GeneralDictionaryService generalDictionaryService;
    @Autowired
    DictService dictService;
    /**
     * 功能描述 保存逻辑编组
     *
     * @param logicalGroupingDTO :
     * @return java.lang.Boolean
     * @author luogy
     * @since 2021/6/24 12:18
     */
    @Override
    public LogicalGrouping save(LogicalGroupingDTO logicalGroupingDTO) throws Exception {
        if (ObjectUtil.isEmpty(logicalGroupingDTO.getParentId())) {
            logicalGroupingDTO.setParentId("0");
        }
        //展示顺序字段赋值
        if (ObjectUtil.isEmpty(logicalGroupingDTO.getSequence())) {
            logicalGroupingDTO.setSequence(getMaxSequence(logicalGroupingDTO.getGroupCode(), logicalGroupingDTO.getProjectId(), logicalGroupingDTO.getIbmsSceneCode(), logicalGroupingDTO.getIbmsClassCode(), logicalGroupingDTO.getParentId()));
        }
        if (!ObjectUtil.isAllEmpty(logicalGroupingDTO.getFloorId(), logicalGroupingDTO.getFloorName(), logicalGroupingDTO.getFloorSequenceId())) {
            JSONObject groupExtra = ObjectUtil.isEmpty(logicalGroupingDTO.getGroupExtra()) ? new JSONObject() : logicalGroupingDTO.getGroupExtra();
            groupExtra.put("floorId", logicalGroupingDTO.getFloorId());
            groupExtra.put("floorName", logicalGroupingDTO.getFloorName());
            groupExtra.put("floorSequenceId", logicalGroupingDTO.getFloorSequenceId());
            logicalGroupingDTO.setGroupExtra(groupExtra);
        }
        LogicalGrouping logicalGrouping = CovertUtil.INSTANCE.convertLogicalGrouping(logicalGroupingDTO);
        boolean result = save(logicalGrouping);
        Assert.isTrue(result, "保存数据库失败");
        if (ObjectUtil.isNotEmpty(logicalGroupingDTO.getAdd())) {
            LogicalObjectDTO logicalObjectDTO = new LogicalObjectDTO();
            logicalObjectDTO.setGroupCode(logicalGroupingDTO.getGroupCode());
            logicalObjectDTO.setProjectId(logicalGroupingDTO.getProjectId());
            logicalObjectDTO.setLogicalGroupingId(logicalGrouping.getLogicalGroupingId());
            logicalObjectDTO.setAdd(logicalGroupingDTO.getAdd());
            logicalObjectService.updateBatch(logicalObjectDTO);
        }
        return logicalGrouping;
    }

    /**
     * 批量更新编组
     *
     * @param logicalGroupingDTOs logicalGroupingDTOs
     * @return {@link Boolean}
     * @author luogy
     * @since 2021/6/24 15:50
     */
    @Override
    @Transactional
    public List<LogicalGrouping> updateBatchById(List<LogicalGroupingDTO> logicalGroupingDTOs) throws Exception {

        List<LogicalGrouping> logicalGroupings = CovertUtil.INSTANCE.convertLogicalGroupings(logicalGroupingDTOs);
        for (LogicalGrouping logicalGrouping : logicalGroupings) {
            if (!ObjectUtil.isAllEmpty(logicalGrouping.getFloorId(), logicalGrouping.getFloorName(), logicalGrouping.getFloorSequenceId())) {
                JSONObject groupExtra = ObjectUtil.isEmpty(logicalGrouping.getGroupExtra()) ? new JSONObject() : logicalGrouping.getGroupExtra();
                groupExtra.put("floorId", logicalGrouping.getFloorId());
                groupExtra.put("floorName", logicalGrouping.getFloorName());
                groupExtra.put("floorSequenceId", logicalGrouping.getFloorSequenceId());
                logicalGrouping.setGroupExtra(groupExtra);
            }
        }


        //先删除编辑时候的对象，处理额外的照明编辑对象的情况
        if(logicalGroupingDTOs.size()==1){
            //表示只有一个编辑
            LogicalGroupingDTO logicalGroupingDTO = logicalGroupingDTOs.get(0);
            List<String> deleteObjIds =  logicalGroupingDTO.getDeleteObjIds();
            String logicalGroupingId =   logicalGroupingDTO.getLogicalGroupingId();
            if(!ObjectUtil.isAllEmpty(deleteObjIds, logicalGroupingId)){
                if(!CollectionUtils.isEmpty(deleteObjIds)){
                    boolean delFlag =  logicalObjectService.removeByLogicalGroupingObjIds(deleteObjIds,logicalGroupingId);
                    Assert.isTrue(delFlag, "保存数据库失败");
                }
                LogicalGrouping logicalGrouping = CovertUtil.INSTANCE.convertLogicalGrouping(logicalGroupingDTO);
                //保存編輯對象
                if (ObjectUtil.isNotEmpty(logicalGroupingDTO.getAdd())) {
                    LogicalObjectDTO logicalObjectDTO = new LogicalObjectDTO();
                    logicalObjectDTO.setGroupCode(logicalGroupingDTO.getGroupCode());
                    logicalObjectDTO.setProjectId(logicalGroupingDTO.getProjectId());
                    logicalObjectDTO.setLogicalGroupingId(logicalGrouping.getLogicalGroupingId());
                    logicalObjectDTO.setAdd(logicalGroupingDTO.getAdd());
                    logicalObjectService.updateBatch(logicalObjectDTO);
                }
            }
        }


        boolean result = updateBatchById(logicalGroupings);
        Assert.isTrue(result, "保存数据库失败");

        return logicalGroupings;
    }

    /**
     * 更新编组
     *
     * @param logicalGroupingDTO logicalGroupingDTO
     * @return {@link Boolean}
     * @author luogy
     * @since 2021/6/24 15:52
     */
    @Override
    public Boolean update(LogicalGroupingDTO logicalGroupingDTO) throws Exception {
        LogicalGrouping logicalGrouping = CovertUtil.INSTANCE.convertLogicalGrouping(logicalGroupingDTO);
        boolean result = updateById(logicalGrouping);
        Assert.isTrue(result, "保存数据库失败");
        return result;
    }

    /**
     * 功能描述 查询逻辑编组
     *
     * @param record :
     * @return java.util.List<com.persagy.dmp.dto.logicalGrouping.LogicalGroupingDTO>
     * @author luogy
     * @since 2021/6/24 12:17
     */
    @Override
    public IPage<LogicalGroupingDTO> listLogicalGroupingDTO(LogicalGroupingDTO record) throws Exception {
        IPage<LogicalGrouping> memberPage = new Page<>(record.getPage(), record.getSize(), true);

        //和下边的写法等效
        QueryWrapper<LogicalGrouping> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("logical_grouping_id", "group_code", "project_id", "parent_id", "ibms_scene_code", "obj_num", "ibms_class_code", "logical_grouping_name", "remark",
                "first_code", "second_code", "`sequence`", "create_time", "update_time", "`valid`", "group_extra", "floor_id", "floor_name", "floor_sequence_id")
                .eq(ObjectUtil.isNotEmpty(record.getGroupCode()), LogicalGrouping.COL_GROUP_CODE, record.getGroupCode())
                .eq(ObjectUtil.isNotEmpty(record.getProjectId()), LogicalGrouping.COL_PROJECT_ID, record.getProjectId())
                .eq(ObjectUtil.isNotEmpty(record.getIbmsSceneCode()), LogicalGrouping.COL_IBMS_SCENE_CODE, record.getIbmsSceneCode())
                .eq(ObjectUtil.isNotEmpty(record.getIbmsClassCode()), LogicalGrouping.COL_IBMS_CLASS_CODE, record.getIbmsClassCode())
                .eq(ObjectUtil.isNotEmpty(record.getFirstCode()), LogicalGrouping.COL_FIRST_CODE, record.getFirstCode())
                .eq(ObjectUtil.isNotEmpty(record.getSecondCode()), LogicalGrouping.COL_SECOND_CODE, record.getSecondCode())
                .eq(ObjectUtil.isNotEmpty(record.getParentId()), LogicalGrouping.COL_PARENT_ID, record.getParentId())
                .eq(ObjectUtil.isNotEmpty(record.getFloorId()), LogicalGrouping.COL_FLOOR_ID, record.getFloorId())
                .like(ObjectUtil.isNotEmpty(record.getLogicalGroupingName()), LogicalGrouping.COL_LOGICAL_GROUPING_NAME, record.getLogicalGroupingName())
                .eq(ObjectUtil.isNotEmpty(record.getLogicalGroupingId()) && !record.getRecursive(), LogicalGrouping.COL_LOGICAL_GROUPING_ID, record.getLogicalGroupingId())
                .and(ObjectUtil.isNotEmpty(record.getLogicalGroupingId()) && record.getRecursive(), item -> {
                    //逻辑编组只有两级的时候使用这个方式递归
                    item.eq(LogicalGrouping.COL_LOGICAL_GROUPING_ID, record.getLogicalGroupingId()).or().eq(LogicalGrouping.COL_PARENT_ID, record.getLogicalGroupingId());
                })
                .orderBy(true, true, LogicalGrouping.COL_SEQUENCE);
        IPage<LogicalGrouping> pageList = getBaseMapper().selectPage(memberPage, queryWrapper);
        IPage<LogicalGroupingDTO> pageListDTO = BeanDtoVoUtils.pageVo(pageList, LogicalGroupingDTO.class);

        if(!CollectionUtils.isEmpty(pageList.getRecords())){
            List<LogicalGroupingDTO> list = pageListDTO.getRecords();
            for (LogicalGroupingDTO logicalGroupingDTO : list) {
                QueryWrapper<LogicalGrouping> querSecondLogicCount = new QueryWrapper<LogicalGrouping>();
                querSecondLogicCount.eq(LogicalGrouping.COL_PARENT_ID,logicalGroupingDTO.getLogicalGroupingId());
                Integer count = getBaseMapper().selectCount(querSecondLogicCount);
                logicalGroupingDTO.setSecondLogicNum(count);
            }
        }


        if (record.getRecursive()) {
            List<LogicalGroupingDTO> list = pageListDTO.getRecords();
            String defaultParentId = "0";
            if (ObjectUtil.isNotEmpty(record.getLogicalGroupingId())) {
                defaultParentId = list.stream().filter(item -> record.getLogicalGroupingId().equals(item.getLogicalGroupingId())).findFirst().orElse(LogicalGroupingDTO.builder().parentId("0").build()).getParentId();
            }
            //parentiId数据库默认值0，code中不能有0，否则会导致死循环
            String parentId = ObjectUtil.isEmpty(record.getParentId()) ? defaultParentId : record.getParentId();
            //得到parentId的下级
            List<Tree<String>> treeNodes = TreeUtil.build(list, parentId,
                    (treeNode, tree) -> {
                        tree.setId(treeNode.getLogicalGroupingId());
                        tree.setParentId(treeNode.getParentId());
                        tree.setName(treeNode.getLogicalGroupingName());
                        tree.setWeight(treeNode.getSequence());
                        // 扩展属性 ...
                        tree.putAll(JSONObject.parseObject(JSON.toJSONString(treeNode)));
                    });
            list = ItvJsonUtil.jsonToList(JSONObject.toJSONString(treeNodes), LogicalGroupingDTO.class);
            pageListDTO.setRecords(list);
        }
        return pageListDTO;
    }

    /**
     * 获取Id-name的map键值对
     *
     * @param record record
     * @return {@link HashMap< String, String>}
     * @author luogy
     * @since 2021/6/24 16:48
     */
    @Override
    public HashMap<String, String> queryLogicalGroupingIdNameMap(LogicalGroupingDTO record) throws Exception {
        IPage<LogicalGroupingDTO> logicalGroupingDTOIPage = listLogicalGroupingDTO(record);
        HashMap<String, String> idNameMap = new HashMap<>();
        for (LogicalGroupingDTO item : logicalGroupingDTOIPage.getRecords()) {
            idNameMap.put(item.getLogicalGroupingId(), item.getLogicalGroupingName());
            if (ObjectUtil.isNotEmpty(item.getChildren())) {
                queryLogicalGroupingIdNameMap(item.getChildren(), idNameMap);
            }
        }
        return idNameMap;
    }

    private void queryLogicalGroupingIdNameMap(List<LogicalGroupingDTO> logicalGroupingDTOS, HashMap<String, String> idNameMap) throws Exception {
        for (LogicalGroupingDTO item : logicalGroupingDTOS) {
            idNameMap.put(item.getLogicalGroupingId(), item.getLogicalGroupingName());
            if (ObjectUtil.isNotEmpty(item.getChildren())) {
                queryLogicalGroupingIdNameMap(item.getChildren(), idNameMap);
            }
        }
    }

    /**
     * 根据当前id，查询里面父级id，封装到list集合（包含本身id）
     * 递归查询由于mysql没有原生提供递归函数，只能通过函数递归
     * 此处直接循环递归查询,方便程序使用
     *
     * @param id
     * @param idList
     */
    private void selectParentById(String id, Collection<String> idList) {
        //查询菜单里面子菜单id，封装idList里面
        idList.add(id);
        LogicalGrouping logicalGrouping = baseMapper.selectById(id);
        //把childIdList里面菜单id值获取出来，封装idList里面，做递归查询
        if (ObjectUtil.isNotEmpty(logicalGrouping) && ObjectUtil.isNotEmpty(logicalGrouping.getParentId()) && !"0".equals(logicalGrouping.getParentId())) {
            this.selectParentById(logicalGrouping.getParentId(), idList);
        }
    }

    /**
     * 获取当前编组及其子编组(不会递归)的编组id
     *
     * @param id id
     * @return {@link java.util.List<java.lang.String>}
     * @author luogy
     * @since 2021/6/29 19:42
     */
    @Override
    public List<String> selectChildByIdNotRecursive(String id) {
        List<LogicalGrouping> list = lambdaQuery().nested(ObjectUtil.isNotEmpty(id), item -> item.eq(LogicalGrouping::getLogicalGroupingId, id).or().eq(LogicalGrouping::getParentId, id))
                .list();
        return list.stream().map(LogicalGrouping::getLogicalGroupingId).collect(Collectors.toList());
    }

    /**
     * 功能描述获取子节点
     *
     * @return
     * @author luogy
     * @since 2021/6/24 15:24
     */
    @Override
    public Collection<String> selectParentById(String id) {
        HashSet<String> ids = new HashSet<>();
        selectParentById(id, ids);
        return ids;
    }

    /**
     * getMaxSequence 自动获取生成最大序号
     *
     * @param groupCode     groupCode
     * @param projectId     projectId
     * @param ibmsSceneCode ibmsSceneCode
     * @param ibmsClassCode ibmsClassCode
     * @param parentId      parentId
     * @return: int
     * @author luogy
     * @since 2021/6/24 15:37
     */
    private int getMaxSequence(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode, String parentId) throws Exception {

        LogicalGroupingDTO listParam = new LogicalGroupingDTO();
        listParam.setGroupCode(groupCode);
        listParam.setProjectId(projectId);
        listParam.setIbmsSceneCode(ibmsSceneCode);
        listParam.setIbmsClassCode(ibmsClassCode);
        listParam.setParentId(parentId);
        List<LogicalGroupingDTO> logicalGroupingDTOS = listLogicalGroupingDTO(listParam).getRecords();
        //新增的时候需要给sequence赋值当前最大值+1
        int sequence = 1;
        OptionalInt max = logicalGroupingDTOS.stream().mapToInt(LogicalGroupingDTO::getSequence).max();
        if (max.isPresent()) {
            sequence = max.getAsInt() + 1;
        }
        return sequence;
    }

    @Override
    public LogicalGrouping getDetailById(String id) {
        QueryWrapper<LogicalGrouping> wrapper = new QueryWrapper<>();
        wrapper.eq(LogicalGrouping.COL_LOGICAL_GROUPING_ID, id);
        LogicalGrouping logicalGrouping = this.getBaseMapper().selectOne(wrapper);
        return logicalGrouping;
    }

    public boolean removeBatchByIds(List<String> ids) {
        Assert.isTrue(ObjectUtil.isNotEmpty(ids), "参数错误" );
        boolean result = removeByIds(ids);
        //查询该编组下面是否还有二级编组
        if(ids.size()==1){
            Map<String, Object> columnMap = new HashMap<>();
            columnMap.put("parent_id",ids.get(0));
           List<LogicalGrouping> lists =  listByMap(columnMap);
           if(CollectionUtils.isEmpty(lists)){
               //二级编组
               logicalObjectService.removeByLogicalGroupingIds(ids);
               Assert.isTrue(result, "删除失败");
           }else {
               for (LogicalGrouping logicalGrouping:lists
               ) {
                   result = removeById(logicalGrouping.getLogicalGroupingId());
                   Assert.isTrue(result, "删除失败");

                   result = logicalObjectService.removeByLogicalGroupingId(logicalGrouping.getLogicalGroupingId());
                   Assert.isTrue(result, "删除失败");
               }
           }
        }else {
            logicalObjectService.removeByLogicalGroupingIds(ids);
            Assert.isTrue(result, "删除失败");
        }
        return result;
    }

    @Override
    public List<LogicalGroupingDTO> listGroupingObject(LogicalGroupingDTO logicalGroupingDTO) throws Exception {

        String groupCode = logicalGroupingDTO.getGroupCode();
        String projectId = logicalGroupingDTO.getProjectId();
        String ibmsSceneCode = logicalGroupingDTO.getIbmsSceneCode();
        String ibmsClassCode = logicalGroupingDTO.getIbmsClassCode();
        JSONArray objectArray = objectService.queryObjectList(groupCode, projectId, ibmsSceneCode, ibmsClassCode, null).getData(JSONArray.class);

        LogicalGroupingDTO param = new LogicalGroupingDTO();
        param.setGroupCode(groupCode);
        param.setProjectId(projectId);
        param.setIbmsSceneCode(ibmsSceneCode);
        param.setIbmsClassCode(ibmsClassCode);
        List<LogicalGroupingDTO> groupings = listLogicalGroupingDTO(param).getRecords();
        HashMap<String, String> idNameMap = new HashMap<>();
        for (LogicalGroupingDTO grouping : groupings) {
            idNameMap.put(grouping.getLogicalGroupingId(), grouping.getLogicalGroupingName());
        }
        List<String> groupingIds = groupings.stream().map(LogicalGroupingDTO::getLogicalGroupingId).collect(Collectors.toList());
        LogicalObjectDTO LogicalObjectParam = new LogicalObjectDTO();
        List<LogicalObjectDTO> groupingLogicalObjects = new ArrayList<>();
        if(!CollectionUtils.isEmpty(groupingIds)) {
            LogicalObjectParam.setLogicalGroupingIds(groupingIds);
            groupingLogicalObjects = logicalObjectService.listLogicalObjectDTO(LogicalObjectParam);
        }
        Map<String, LogicalObjectDTO> groupingLogicalObjectsMap = groupingLogicalObjects.stream()
                .collect(Collectors.toMap(LogicalObjectDTO::getObjId, item -> item, (o, n) -> n));

        Map<String, List<JSONObject>> allObjectMaps = objectArray.stream().map(item -> (JSONObject) item).map(item -> {
            LogicalObjectDTO tmp = groupingLogicalObjectsMap.get(item.getString("id"));
            if (tmp != null) {
                item.put("floorId", tmp.getFloorId());
                item.put("floorName", tmp.getFloorName());
                item.put("spaceId", tmp.getSpaceId());
                item.put("spaceName", tmp.getSpaceName());
                item.put("buildId", tmp.getBuildId());
                item.put("buildName", tmp.getBuildName());
                item.put("logicalGroupingId", tmp.getLogicalGroupingId());
                item.put("logicalGroupingName", idNameMap.get(tmp.getLogicalGroupingId()));

            } else {
                item.put("logicalGroupingId", "0");
                item.put("logicalGroupingName", "未分组");
            }
            return item;
        }).collect(Collectors.groupingBy(item -> item.getString("logicalGroupingId")));
        for (LogicalGroupingDTO grouping : groupings) {
            grouping.setLogicalObjects(allObjectMaps.get(grouping.getLogicalGroupingId()));
        }
        if (!ObjectUtil.isAllEmpty(logicalGroupingDTO.getLogicalGroupingId(), logicalGroupingDTO.getParentId())) {
            groupings = groupings.stream().filter(item -> {
                if (ObjectUtil.isNotEmpty(logicalGroupingDTO.getLogicalGroupingId())) {
                    return logicalGroupingDTO.getLogicalGroupingId().equals(item.getLogicalGroupingId());
                }
                return true;
            }).filter(item -> {
                if (ObjectUtil.isNotEmpty(logicalGroupingDTO.getParentId())) {
                    return logicalGroupingDTO.getParentId().equals(item.getParentId());
                }
                return true;
            }).collect(Collectors.toList());
        }
        //获取未变组设备
        if(logicalGroupingDTO.isContainsNotGrouping()){
            LogicalGroupingDTO notGrouping = new LogicalGroupingDTO();
            notGrouping.setLogicalGroupingName("未分组设备");
            notGrouping.setIbmsSceneCode(ibmsSceneCode);
            notGrouping.setIbmsClassCode(ibmsClassCode);
            List<JSONObject> notGroupingbjects = allObjectMaps.get("0");
            notGrouping.setLogicalObjects(notGroupingbjects);
            notGrouping.setObjNum(ObjectUtil.isEmpty(notGroupingbjects) ? 0 : notGroupingbjects.size());
            groupings.add(0, notGrouping);
        }
        return groupings;
    }

    @Override
    public Page<JSONObject> listPageNotGroupingObject(LogicalGroupingDTO logicalGroupingDTO) throws Exception {
        String groupCode = logicalGroupingDTO.getGroupCode();
        String projectId = logicalGroupingDTO.getProjectId();
        String ibmsSceneCode = logicalGroupingDTO.getIbmsSceneCode();
        String ibmsClassCode = logicalGroupingDTO.getIbmsClassCode();
        JSONArray allObjectArray = objectService.queryObjectList(groupCode, projectId, ibmsSceneCode, ibmsClassCode, null).getData(JSONArray.class);

        LogicalGroupingDTO param = new LogicalGroupingDTO();
        param.setGroupCode(groupCode);
        param.setProjectId(projectId);
        param.setIbmsSceneCode(ibmsSceneCode);
        param.setIbmsClassCode(ibmsClassCode);
        List<LogicalGroupingDTO> groupings = listLogicalGroupingDTO(param).getRecords();
        HashMap<String, String> idNameMap = new HashMap<>();
        for (LogicalGroupingDTO grouping : groupings) {
            idNameMap.put(grouping.getLogicalGroupingId(), grouping.getLogicalGroupingName());
        }
        List<String> groupingIds = groupings.stream().map(LogicalGroupingDTO::getLogicalGroupingId).collect(Collectors.toList());
        LogicalObjectDTO LogicalObjectParam = new LogicalObjectDTO();
        LogicalObjectParam.setLogicalGroupingIds(groupingIds);
        List<LogicalObjectDTO> groupingLogicalObjects = logicalObjectService.listLogicalObjectDTO(LogicalObjectParam);
        List<String> groupingObjIds = groupingLogicalObjects.stream().map(LogicalObjectDTO::getObjId).collect(Collectors.toList());
        List<JSONObject> notGroupingObjs = allObjectArray.stream().map(item -> (JSONObject) item).filter(item -> !groupingObjIds.contains(item.getString("id"))).collect(Collectors.toList());
        List<String> notGroupingObjIds = notGroupingObjs.stream().map(item -> item.getString("id")).collect(Collectors.toList());
        GeneralDictionaryDTO gd = new GeneralDictionaryDTO();
        gd.setGroupCode(groupCode);
        gd.setProjectId(projectId);
        gd.setDictType(LOGICALG_ROUPING);
        List<JSONObject> gdList = generalDictionaryService.listDictionary(gd);
        Optional<JSONObject> scene = gdList.stream().filter(item -> ibmsSceneCode.equals(item.getString("ibmsSceneCode"))).findFirst();
        if(scene.isPresent()){
            JSONObject jsonObject = scene.get();
            if(ObjectUtil.isNotEmpty(jsonObject.getJSONArray("children"))){
                Optional<JSONObject> first = jsonObject.getJSONArray("children").stream().map(item->(JSONObject)item).filter(item -> ibmsSceneCode.equals(item.getString("ibmsSceneCode")) && ibmsClassCode.equals(item.getString("ibmsClassCode"))).findFirst();
                if (first.isPresent()) {
                    String objType = first.get().getString("objType");
                    if(ObjectUtil.isEmpty(objType)){

                    }else if ("equipment".equals(objType)) {
                        Map<String, EquipLocationDTO> equipLocationDTOMap = locationService.queryEquipLocationMap(groupCode, projectId, notGroupingObjIds);
                        for (JSONObject notGroupingObj : notGroupingObjs) {
                            String objId = notGroupingObj.getString("id");
                            notGroupingObj.put("buildId", equipLocationDTOMap.getOrDefault(objId, new EquipLocationDTO()).getBuildId());
                            notGroupingObj.put("buildName", equipLocationDTOMap.getOrDefault(objId, new EquipLocationDTO()).getBuildName());
                            notGroupingObj.put("floorId", equipLocationDTOMap.getOrDefault(objId, new EquipLocationDTO()).getFloorId());
                            notGroupingObj.put("floorName", equipLocationDTOMap.getOrDefault(objId, new EquipLocationDTO()).getFloorName());
                            notGroupingObj.put("spaceId", equipLocationDTOMap.getOrDefault(objId, new EquipLocationDTO()).getSpaceId());
                            notGroupingObj.put("spaceName", equipLocationDTOMap.getOrDefault(objId, new EquipLocationDTO()).getSpaceName());
                        }
                    } else if ("space".equals(objType)) {
                        Map<String, SpaceLocationDTO> spaceLocationDTOMap = locationService.querySpaceLocationMap(groupCode, projectId, notGroupingObjIds);
                        for (JSONObject notGroupingObj : notGroupingObjs) {
                            String objId = notGroupingObj.getString("id");
                            notGroupingObj.put("buildId", spaceLocationDTOMap.getOrDefault(objId, new SpaceLocationDTO()).getBuildId());
                            notGroupingObj.put("buildName", spaceLocationDTOMap.getOrDefault(objId, new SpaceLocationDTO()).getBuildName());
                            notGroupingObj.put("floorId", spaceLocationDTOMap.getOrDefault(objId, new SpaceLocationDTO()).getFloorId());
                            notGroupingObj.put("floorName", spaceLocationDTOMap.getOrDefault(objId, new SpaceLocationDTO()).getFloorName());
                            notGroupingObj.put("spaceId", spaceLocationDTOMap.getOrDefault(objId, new SpaceLocationDTO()).getSpaceId());
                            notGroupingObj.put("spaceName", spaceLocationDTOMap.getOrDefault(objId, new SpaceLocationDTO()).getSpaceName());
                        }

                    } else if ("system".equals(objType)) {
                        Map<String, SystemLocationDTO> systemLocationDTOMap = locationService.querySystemLocationMap(groupCode, projectId, notGroupingObjIds);
                        for (JSONObject notGroupingObj : notGroupingObjs) {
                            String objId = notGroupingObj.getString("id");
                            notGroupingObj.put("buildId", systemLocationDTOMap.getOrDefault(objId, new SystemLocationDTO()).getBuildId());
                            notGroupingObj.put("buildName", systemLocationDTOMap.getOrDefault(objId, new SystemLocationDTO()).getBuildName());
                            notGroupingObj.put("floorId", systemLocationDTOMap.getOrDefault(objId, new SystemLocationDTO()).getFloorId());
                            notGroupingObj.put("floorName", systemLocationDTOMap.getOrDefault(objId, new SystemLocationDTO()).getFloorName());
                            notGroupingObj.put("spaceId", systemLocationDTOMap.getOrDefault(objId, new SystemLocationDTO()).getSpaceId());
                            notGroupingObj.put("spaceName", systemLocationDTOMap.getOrDefault(objId, new SystemLocationDTO()).getSpaceName());
                        }
                    }
                }
            }
        }

        if(!ObjectUtil.isAllEmpty(logicalGroupingDTO.getFloorId(),logicalGroupingDTO.getKeyWord())){
            notGroupingObjs = notGroupingObjs.stream().filter(item->{
                boolean result =true;
                if(ObjectUtil.isNotEmpty(logicalGroupingDTO.getFloorId())){
                    result = result && logicalGroupingDTO.getFloorId().equals(item.getString("floorId"));
                }

                if(ObjectUtil.isNotEmpty(logicalGroupingDTO.getKeyWord())){
                    Pattern pattern = Pattern.compile(logicalGroupingDTO.getKeyWord());
                    result =  result && (pattern.matcher(StringUtils.trimToEmpty(DmpConstant.getIbmsxianshibianma(item))).find()
                        || pattern.matcher(StringUtils.trimToEmpty(DmpConstant.getIbmsxianshimingcheng(item))).find());
//                    result = result && (logicalGroupingDTO.getKeyWord().equals(item.getString("localId")) || logicalGroupingDTO.getKeyWord().equals(item.getString("localName")));
                }
                return result;
            }).collect(Collectors.toList());
        }

//        if(ObjectUtil.isNotEmpty(logicalGroupingDTO.getFloorId())){
//            notGroupingObjs = notGroupingObjs.stream().filter(item->logicalGroupingDTO.getFloorId().equals(item.getString("floorId"))).collect(Collectors.toList());
//        }
//
//        if(ObjectUtil.isNotEmpty(logicalGroupingDTO.getKeyWord())){
//            Pattern pattern = Pattern.compile(logicalGroupingDTO.getKeyWord());
//            notGroupingObjs = notGroupingObjs.stream().filter(o -> {
//                return pattern.matcher(StringUtils.trimToEmpty(o.getString("localId"))).find()
//                        || pattern.matcher(StringUtils.trimToEmpty(o.getString("localName"))).find();
//            }).collect(Collectors.toList());
//        }

        //分页
        Integer page = logicalGroupingDTO.getPage();
        Integer size = logicalGroupingDTO.getSize();
        int totalSize = notGroupingObjs.size();
        Page<JSONObject> pageResult = null;
        if (page != null && size != null) {
            Integer skip = (page - 1) * size;
            notGroupingObjs = notGroupingObjs.stream().skip(skip).limit(size).collect(Collectors.toList());
            pageResult = new Page<>(page, size, totalSize);
        }else {
            pageResult = new Page<>(1,totalSize);
        }
        pageResult.setRecords(notGroupingObjs);
        return pageResult;
    }

    @Override
    public ArrayList<JSONObject> listlightingCircuit(LogicalGroupingDTO logicalGroupingDTO) throws Exception {
        String groupCode = logicalGroupingDTO.getGroupCode();
        String projectId = logicalGroupingDTO.getProjectId();
        //String ibmsSceneCode = logicalGroupingDTO.getIbmsSceneCode();
        //String ibmsClassCode = logicalGroupingDTO.getIbmsClassCode();
        String logicalGroupingId = logicalGroupingDTO.getLogicalGroupingId();
        logicalGroupingDTO.setSize(null);
        logicalGroupingDTO.setPage(null);
        List<JSONObject> notGroupIngobjectList = listPageNotGroupingObject(logicalGroupingDTO).getRecords();
        for (JSONObject object : notGroupIngobjectList) {
            object.put("checked", "uncheck");
        }
        if (ObjectUtil.isNotEmpty(logicalGroupingId)) {
            LogicalObjectDTO logicalObjectDTO = new LogicalObjectDTO();
            logicalObjectDTO.setLogicalGroupingId(logicalGroupingId);
            logicalObjectDTO.setProjectId(projectId);
            logicalObjectDTO.setGroupCode(groupCode);
            List<LogicalObjectDTO> logicalObjectDTOS = logicalObjectService.listLogicalObjectDTO(logicalObjectDTO);
            //accounts.stream().collect(Collectors.toMap(Account::getId, Account::getUsername));
            Map<String,LogicalObjectDTO> idToLobjMap = logicalObjectDTOS.stream().collect(Collectors.toMap(LogicalObjectDTO::getObjId, lobj -> lobj));
            List<String> groupingObjIds = logicalObjectDTOS.stream().map(item -> item.getObjId()).collect(Collectors.toList());
            List<JSONObject> groupIngobjectList = objectService.queryObjectList(groupCode, projectId, groupingObjIds).stream().map(item -> (JSONObject) item).map(item -> {
                if(idToLobjMap.containsKey(item.getString("id")) && !ObjectUtils.isEmpty(idToLobjMap.get(item.getString("id")) )){
                    LogicalObjectDTO lObject = idToLobjMap.get(item.getString("id"));
                    item.put("floorId",lObject.getFloorId());
                    item.put("floorName",lObject.getFloorName());
                    item.put("buildId",lObject.getBuildId());
                    item.put("buildName",lObject.getBuildName());
                    item.put("spaceId",lObject.getSpaceId());
                    item.put("spaceName",lObject.getSpaceName());
                }
                item.put("logicalGroupingId", logicalGroupingId);
                item.put("checked", "checked");
                return item;
            }).collect(Collectors.toList());
            notGroupIngobjectList.addAll(groupIngobjectList);
        }

        if(StringUtils.isNotBlank(logicalGroupingDTO.getFloorId())){
            //按照楼层筛选回路
            notGroupIngobjectList =  notGroupIngobjectList.stream().filter(item -> logicalGroupingDTO.getFloorId().equals(((JSONObject)item).getString("floorId"))).collect(Collectors.toList());
        }

        Map<String, List<JSONObject>> lightLocate = notGroupIngobjectList.stream().collect(Collectors.groupingBy(item -> item.getString("lightLocate")));
        FuncIdQueryDTO funcIdQueryDto = new FuncIdQueryDTO();
        funcIdQueryDto.setClassCode("SELTCC");
        funcIdQueryDto.setCode("lightLocate");
        List<InfoPointDictDTO> infoPointDictDTOList = dictService.queryInfoPointDictList(groupCode, projectId, funcIdQueryDto);
        ArrayList<JSONObject> objResult = new ArrayList<>();
        if (infoPointDictDTOList.size() == 1) {
            Object[] dataSource = infoPointDictDTOList.get(0).getDataSource();
            Map<String, String> lightLocateCodeNameMap = Arrays.stream(dataSource).map(item -> JSONObject.parseObject(JSON.toJSONString(item))).collect(Collectors.toMap(item -> item.getString("code"), item -> item.getString("name")));
            for (Map.Entry<String, List<JSONObject>> entry : lightLocate.entrySet()) {
                JSONObject tmp = new JSONObject();
                tmp.put("code", entry.getKey());
                tmp.put("name", lightLocateCodeNameMap.get(entry.getKey()));
                tmp.put("children", entry.getValue());
                objResult.add(tmp);
            }
        }
        return objResult;
    }
    @Override
    public Page<LogicalObjectDTO> listGroupLoopDetailsObject(LogicalGroupingDTO logicalGroupingDTO) throws Exception {
        //集团code
        String groupCode = logicalGroupingDTO.getGroupCode();
        //项目id
        String projectId = logicalGroupingDTO.getProjectId();
        //编组
        String logicalGroupingId = logicalGroupingDTO.getLogicalGroupingId();
        LogicalObjectDTO logicalObjectParam = new LogicalObjectDTO();
        logicalObjectParam.setLogicalGroupingId(logicalGroupingId);
        logicalObjectParam.setProjectId(projectId);
        logicalObjectParam.setGroupCode(groupCode);
        //查询对象设备
        List<LogicalObjectDTO> groupingLogicalObjects = logicalObjectService.listLogicalObjectDTOLocalId(logicalObjectParam);

        //查询回路，所属模块,所属配电箱
        List<String> objIds = groupingLogicalObjects.stream().map(LogicalObjectDTO::getObjId).collect(Collectors.toList());
        HashMap<String,EquipBelongModularDTO> equipBelongModularBoxMap  = locationService.queryEquipModularBox(groupCode,projectId,objIds);

        //分页
        Integer page = logicalGroupingDTO.getPage();
        Integer size = logicalGroupingDTO.getSize();
        int totalSize = groupingLogicalObjects.size();
        if (page != null && size != null) {
            Integer skip = (page - 1) * size;
            groupingLogicalObjects = groupingLogicalObjects.stream().skip(skip).limit(size).collect(Collectors.toList());
        }
        //组装所属所属模块和配电箱
        for (LogicalObjectDTO logicalObjectDTO:groupingLogicalObjects) {

            if(equipBelongModularBoxMap.containsKey(logicalObjectDTO.getObjId())){
                EquipBelongModularDTO belongModularDTO =  equipBelongModularBoxMap.get(logicalObjectDTO.getObjId());
                logicalObjectDTO.setBelongModularName(belongModularDTO.getBelongModularName()==null?"":belongModularDTO.getBelongModularName());
                logicalObjectDTO.setBelongPowerBoxName(belongModularDTO.getBelongPowerBoxName()==null?"":belongModularDTO.getBelongPowerBoxName());
            }
            //组装位置
            logicalObjectDTO.setLocation(DataUtils.join("-", logicalObjectDTO.getBuildName(), logicalObjectDTO.getFloorName(), logicalObjectDTO.getSpaceName()));
            if(logicalObjectDTO.getLocalId()==null){
                //给个默认值
                logicalObjectDTO.setLocalId("");
            }
            if(logicalObjectDTO.getFloorName()==null){
                //给个默认值
                logicalObjectDTO.setFloorName("");
            }
        }

        Page<LogicalObjectDTO> pageResult = new Page<>(page, size, totalSize);
        pageResult.setRecords(groupingLogicalObjects);
        return pageResult;
    }


    @Override
    public Map<String, String> getFirstNameAndSecondName(String secondLogicalId) {
        //查询二级编组
        QueryWrapper<LogicalGrouping> querySecondNameWrapper = new QueryWrapper<LogicalGrouping>();
        querySecondNameWrapper.eq("logical_grouping_id",secondLogicalId);
        List<LogicalGrouping> logicalGroupingList = list(querySecondNameWrapper);
        //查询一级编组
        QueryWrapper<LogicalGrouping> queryFirstWrapper = new QueryWrapper<LogicalGrouping>();
        queryFirstWrapper.eq("logical_grouping_id",logicalGroupingList.get(0).getParentId());
        List<LogicalGrouping> logicalGroupingParentList =  list(queryFirstWrapper);
        //组装数据
        Map<String,String> groupName = new HashMap<>();
        groupName.put("first",logicalGroupingParentList.get(0).getLogicalGroupingName());
        groupName.put("second",logicalGroupingList.get(0).getLogicalGroupingName());
        return groupName;
    }

	@Override
	public List<LogicalGrouping> queryFirstLogicalGroupingList(String groupCode, String projectId, String ibmsSceneCode, String firstLogicalGroupingId, String floorId) throws Exception {
		List<LogicalGrouping> listLogicalGrouping = null;
		LogicalGroupingDTO logicalGroupingDTO = new LogicalGroupingDTO();
		logicalGroupingDTO.setGroupCode(groupCode);
		logicalGroupingDTO.setProjectId(projectId);
		logicalGroupingDTO.setIbmsSceneCode(ibmsSceneCode);
		if (StrUtil.isBlank(floorId)) {
			// 默认只查一级的逻辑编组
			logicalGroupingDTO.setParentId("0");
			
			QueryWrapper<LogicalGrouping> queryWrapper = new QueryWrapper<>();
	        queryWrapper.select("logical_grouping_id", "group_code", "project_id", "parent_id", "ibms_scene_code", "obj_num", "ibms_class_code", "logical_grouping_name", "remark",
	                "first_code", "second_code", "`sequence`", "create_time", "update_time", "`valid`", "group_extra", "floor_id", "floor_name", "floor_sequence_id")
	                .eq(LogicalGrouping.COL_GROUP_CODE, groupCode)
	                .eq(LogicalGrouping.COL_PROJECT_ID, projectId)
	                .eq(StrUtil.isNotBlank(firstLogicalGroupingId), LogicalGrouping.COL_LOGICAL_GROUPING_ID, firstLogicalGroupingId)
	                .eq(StrUtil.isNotBlank(ibmsSceneCode), LogicalGrouping.COL_IBMS_SCENE_CODE, ibmsSceneCode)
	                .eq(LogicalGrouping.COL_PARENT_ID, logicalGroupingDTO.getParentId())
	                .orderBy(true, true, LogicalGrouping.COL_SEQUENCE);
	        listLogicalGrouping = this.baseMapper.selectList(queryWrapper);
		} else {
			// 根据楼层ID查询出对应的二级编组，然后在反查出对应的一级编组，因valid字段之前未使用，这里不做区分
			listLogicalGrouping = this.baseMapper.queryLogicalGroupingByFloorId(groupCode, projectId, floorId);
		}
		if (CollectionUtil.isEmpty(listLogicalGrouping)) {
			return Lists.newArrayList();
		}
		
		return listLogicalGrouping;
	}
	
}
