package com.persagy.dmp.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.dmp.entity.LogicalObjectEquipCached;
import com.persagy.dmp.mapper.LogicalObjectEquipCachedMapper;
import com.persagy.dmp.service.LogicalObjectEquipCachedService;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author : ZhangWenTao
 * @version : 1.0
 * @since : 2022/7/14 17:14
 */
@Service
public class LogicalObjectEquipCachedServiceImpl extends ServiceImpl<LogicalObjectEquipCachedMapper, LogicalObjectEquipCached> implements LogicalObjectEquipCachedService {

    @Override
    public LogicalObjectEquipCached getOne(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode) {
        return getOne(
                Wrappers.<LogicalObjectEquipCached>lambdaQuery()
                        .select(LogicalObjectEquipCached::getId)
                        .eq(LogicalObjectEquipCached::getGroupCode, groupCode)
                        .eq(LogicalObjectEquipCached::getProjectId, projectId)
                        .eq(LogicalObjectEquipCached::getIbmsSceneCode, ibmsSceneCode)
                        .eq(LogicalObjectEquipCached::getIbmsClassCode, ibmsClassCode)
                        .last("limit 1"));
    }

    @Override
    public List<LogicalObjectEquipCached> listByLastUpdateTimeInterval(long interval, TimeUnit unit) {
        Date date = new Date();
        date.setTime(date.getTime() - unit.toMillis(interval));

        return list(Wrappers.<LogicalObjectEquipCached>lambdaQuery().lt(LogicalObjectEquipCached::getUpdateTime, date));
    }

    @Override
    public void saveOrUpdate(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode) {
        saveOrUpdate(
                LogicalObjectEquipCached.builder()
                        .groupCode(groupCode)
                        .projectId(projectId)
                        .ibmsSceneCode(ibmsSceneCode)
                        .ibmsClassCode(ibmsClassCode)
                        .build(),
                Wrappers.<LogicalObjectEquipCached>lambdaQuery()
                        .select(LogicalObjectEquipCached::getId)
                        .eq(LogicalObjectEquipCached::getGroupCode, groupCode)
                        .eq(LogicalObjectEquipCached::getProjectId, projectId)
                        .eq(LogicalObjectEquipCached::getIbmsSceneCode, ibmsSceneCode)
                        .eq(LogicalObjectEquipCached::getIbmsClassCode, ibmsClassCode)

        );
    }

    @Override
    public List<LogicalObjectEquipCached> listUnloadEquipCached(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode) {
        return getBaseMapper().listUnloadEquipCached(groupCode, projectId, ibmsSceneCode, ibmsClassCode);
    }
}
