package com.persagy.dmp.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.dmp.dto.LocationDTO;
import com.persagy.dmp.dto.logicalGrouping.LogicalGroupingDTO;
import com.persagy.dmp.dto.logicalGrouping.LogicalObjectDTO;
import com.persagy.dmp.dto.logicalGrouping.LogicalObjectRelationDTO;
import com.persagy.dmp.entity.LogicalGrouping;
import com.persagy.dmp.entity.LogicalObject;
import com.persagy.dmp.mapper.LogicalObjectMapper;
import com.persagy.dmp.service.LocationService;
import com.persagy.dmp.service.LogicalGroupingService;
import com.persagy.dmp.service.LogicalObjectService;
import com.persagy.dmp.utils.BeanDtoVoUtils;
import com.persagy.dmp.utils.CovertUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class LogicalObjectServiceImpl extends ServiceImpl<LogicalObjectMapper, LogicalObject> implements LogicalObjectService {

    @Autowired
    LogicalGroupingService logicalGroupingService;

    @Autowired
    LocationService locationService;

    /**
     * 更新逻辑编组对象（逻辑编组添加对象，删除对象）
     *
     * @param logicalObjectDTO logicalObjectDTO
     * @return {@link boolean}
     * @author luogy
     * @since 2021/6/28 19:48
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateBatch(LogicalObjectDTO logicalObjectDTO) throws Exception {

        List<String> deleteObjIds = ObjectUtil.isEmpty(logicalObjectDTO.getDelete()) ? new ArrayList<>() : logicalObjectDTO.getDelete();
        List<String> addObjIds = ObjectUtil.isEmpty(logicalObjectDTO.getAdd()) ? new ArrayList<>() : logicalObjectDTO.getAdd();
        List<String> updateObjIds = ObjectUtil.isEmpty(logicalObjectDTO.getUpdate()) ? new ArrayList<>() : logicalObjectDTO.getUpdate();
        //过滤掉前端控件问题误传的楼层对象
        addObjIds = addObjIds.stream().filter(item -> !item.startsWith("Fl") && !item.startsWith("Bd")).collect(Collectors.toList());
        updateObjIds = updateObjIds.stream().filter(item -> !item.startsWith("Fl") && !item.startsWith("Bd")).collect(Collectors.toList());
        boolean result = updateRelation(logicalObjectDTO.getGroupCode(), logicalObjectDTO.getProjectId(), logicalObjectDTO.getLogicalGroupingId(), deleteObjIds, addObjIds, updateObjIds);
        List<LogicalObjectRelationDTO> updateBatch = logicalObjectDTO.getUpdateBatch();
        if (ObjectUtil.isNotEmpty(updateBatch)) {
            for (LogicalObjectRelationDTO objectRelationDTO : updateBatch) {
                String objId = objectRelationDTO.getObjId();
                if (ObjectUtil.isEmpty(objId)) {
                    continue;
                }
                if (ObjectUtil.isEmpty(objectRelationDTO.getLogicalGroupingId())) {
                    removeByObjId(objId);
                } else {
                    updateRelation(logicalObjectDTO.getGroupCode(), logicalObjectDTO.getProjectId(), objectRelationDTO.getLogicalGroupingId(), null, null, Arrays.asList(objId));
                }
            }
        }
        return result;
    }

    private boolean updateRelation(String groupCode, String projectId, String logicalGroupingId, List<String> deleteObjIds, List<String> addObjIds, List<String> updateObjIds) throws Exception {
        boolean result = true;
        if (!ObjectUtil.isAllEmpty(deleteObjIds, addObjIds, updateObjIds)) {
            LogicalGrouping groupDetail = logicalGroupingService.getById(logicalGroupingId);
            Assert.notNull(groupDetail, "记录 {} 未找到", logicalGroupingId);
            HashSet<String> updateObjNumLogicalGroupingIdSet = new HashSet<>();
            updateObjNumLogicalGroupingIdSet.add(logicalGroupingId);

            //添加关系前删除旧的关系（级联删除和父编组的关系）
            ArrayList<String> logicalGroupingIdList = new ArrayList<>();
            logicalGroupingIdList.add(logicalGroupingId);
            if (ObjectUtil.isNotEmpty(groupDetail.getParentId()) && !"0".equals(groupDetail.getParentId())) {
                logicalGroupingIdList.add(groupDetail.getParentId());
            }
            if (!ObjectUtil.isAllEmpty(deleteObjIds, addObjIds)) {
                ArrayList<String> deleteIds = new ArrayList<>();
                deleteIds.addAll(deleteObjIds);
                deleteIds.addAll(addObjIds);
                List<LogicalObject> list = lambdaQuery()
                        .eq(ObjectUtil.isNotEmpty(groupCode), LogicalObject::getGroupCode, groupCode)
                        .eq(ObjectUtil.isNotEmpty(projectId), LogicalObject::getProjectId, projectId)
                        .in(ObjectUtil.isNotEmpty(logicalGroupingIdList), LogicalObject::getLogicalGroupingId, logicalGroupingIdList)
                        .in(ObjectUtil.isNotEmpty(deleteIds), LogicalObject::getObjId, deleteIds)
                        .list();
                for (LogicalObject logicalObject : list) {
                    updateObjNumLogicalGroupingIdSet.add(logicalObject.getLogicalGroupingId());
                }
                this.removeByIds(list.stream().map(item -> item.getId()).collect(Collectors.toList()));
            }
            if (ObjectUtil.isNotEmpty(updateObjIds)) {
                List<LogicalObject> list = lambdaQuery()
                        .eq(ObjectUtil.isNotEmpty(groupCode), LogicalObject::getGroupCode, groupCode)
                        .eq(ObjectUtil.isNotEmpty(projectId), LogicalObject::getProjectId, projectId)
                        .in(ObjectUtil.isNotEmpty(updateObjIds), LogicalObject::getObjId, updateObjIds)
                        .list();
                for (LogicalObject logicalObject : list) {
                    updateObjNumLogicalGroupingIdSet.add(logicalObject.getLogicalGroupingId());
                }
                this.removeByIds(list.stream().map(item -> item.getId()).collect(Collectors.toList()));
            }
            List<LogicalObject> LogicalObjects = new ArrayList<>();
            addObjIds.addAll(updateObjIds);
            if (ObjectUtil.isNotEmpty(addObjIds)) {
                Map<String, LocationDTO> objLocation = locationService.queryObjLocation(groupCode, projectId, addObjIds);
                //逻辑编组添加对象
                for (String addObjId : addObjIds) {
                    LogicalObject logicalObject = new LogicalObject();
                    logicalObject.setGroupCode(groupCode);
                    logicalObject.setProjectId(projectId);
                    logicalObject.setObjId(addObjId);
                    logicalObject.setLogicalGroupingId(logicalGroupingId);
                    logicalObject.setObjName(objLocation.getOrDefault(addObjId, new LocationDTO()).getIbmsxianshimingcheng());
                    logicalObject.setLocalId(objLocation.getOrDefault(addObjId, new LocationDTO()).getIbmsxianshibianma());
                    logicalObject.setObjType(objLocation.getOrDefault(addObjId, new LocationDTO()).getObjType());
                    logicalObject.setGroupExtra(JSONObject.parseObject(JSON.toJSONString(objLocation.getOrDefault(addObjId, new LocationDTO()))));
                    LogicalObjects.add(logicalObject);
                }
                result = saveBatch(LogicalObjects);
            }
            for (String groupingId : updateObjNumLogicalGroupingIdSet) {
                updateObjNum(groupingId);
            }
        }
        return result;
    }

    protected void updateObjNum(String lgicalGroupingId) {
        //更新编组对象数量
        LogicalGrouping groupDetail = logicalGroupingService.getById(lgicalGroupingId);
        Integer count = lambdaQuery().eq(LogicalObject::getLogicalGroupingId, lgicalGroupingId).count();
        LogicalGrouping logicalGrouping = new LogicalGrouping();
        logicalGrouping.setLogicalGroupingId(lgicalGroupingId);
        logicalGrouping.setObjNum(count);
        logicalGroupingService.updateById(logicalGrouping);
        if (ObjectUtil.isNotEmpty(groupDetail.getParentId()) && !"0".equals(groupDetail.getParentId())) {
            //更新父编组对象数量
            List<LogicalGrouping> list = logicalGroupingService.lambdaQuery().nested(q -> q.eq(LogicalGrouping::getLogicalGroupingId, groupDetail.getParentId()).or().eq(LogicalGrouping::getParentId, groupDetail.getParentId())).select(LogicalGrouping::getLogicalGroupingId).list();
            list.add(logicalGrouping);
            List<String> logicalGroupingIds = list.stream().map(LogicalGrouping::getLogicalGroupingId).collect(Collectors.toList());
            Integer parentCount = lambdaQuery().in(LogicalObject::getLogicalGroupingId, logicalGroupingIds).count();
            LogicalGrouping logicalGroupingParent = new LogicalGrouping();
            logicalGroupingParent.setLogicalGroupingId(groupDetail.getParentId());
            logicalGroupingParent.setObjNum(parentCount);
            logicalGroupingService.updateById(logicalGroupingParent);
        }
    }

    /**
     * 查询逻辑编组下的对象
     *
     * @param record record
     * @return {@link IPage< LogicalObjectDTO>}
     * @author luogy
     * @since 2021/6/28 19:55
     */
    @Override
    public IPage<LogicalObjectDTO> listPageLogicalObjectDTO(LogicalObjectDTO record) throws Exception {

        if (!StringUtils.isEmpty(record.getLogicalGroupingId())) {
//            JSONObject.parseArray(JSONArray.toJSONString(notLGroupingDTOS), LogicalGroupingDTO.class);
            LogicalGrouping logicalGrouping = logicalGroupingService.getById(record.getLogicalGroupingId());
            String ibmsSceneCode = logicalGrouping.getIbmsSceneCode();
            String ibmsClassCode = logicalGrouping.getIbmsClassCode();
            String uniquFlag = record.getGroupCode() + record.getProjectId() + ibmsSceneCode + ibmsClassCode;
            if (uniquFlag.equals(record.getLogicalGroupingId())) {
                //表示查询全部
                LogicalGroupingDTO logicalGroupingDTO = LogicalGroupingDTO.builder().groupCode(record.getGroupCode()).projectId(record.getProjectId()).ibmsSceneCode(ibmsSceneCode).ibmsClassCode(ibmsClassCode).build();

                List<JSONObject> logicalObjectDTOList = new ArrayList<>();
                List<LogicalGroupingDTO> lGroupingDTOS = logicalGroupingService.listGroupingObject(logicalGroupingDTO);
                for (LogicalGroupingDTO lobj : lGroupingDTOS
                ) {
                    if (CollectionUtils.isNotEmpty(lobj.getLogicalObjects())) {
                        logicalObjectDTOList.addAll(lobj.getLogicalObjects());
                    }
                }
                List<JSONObject> notLGroupingDTOS = logicalGroupingService.listPageNotGroupingObject(logicalGroupingDTO).getRecords();
                if (CollectionUtils.isNotEmpty(notLGroupingDTOS)) {
                    logicalObjectDTOList.addAll(notLGroupingDTOS);
                }
                //将id转成objId
                List<JSONObject> logicalObjectDTOListobjId = new ArrayList<>();
                for (JSONObject jsonObject : logicalObjectDTOList
                ) {
                    jsonObject.put("objId", jsonObject.get("id"));
                    jsonObject.put("id", null);
                    logicalObjectDTOListobjId.add(jsonObject);
                }

                List<LogicalObjectDTO> logicalObjectDTORespose = JSONObject.parseArray(JSONArray.toJSONString(logicalObjectDTOListobjId), LogicalObjectDTO.class);
                IPage<LogicalObjectDTO> iPage = new Page<LogicalObjectDTO>();
                iPage.setRecords(logicalObjectDTORespose);
                iPage.setTotal(logicalObjectDTORespose.size());
                return iPage;
            }

        }

        IPage<LogicalObject> memberPage = new Page<>(record.getPage(), record.getSize(), true);
        QueryWrapper<LogicalObject> queryWrapper = new QueryWrapper<>();
        List<String> childByIds = null;
        if (ObjectUtil.isNotEmpty(record.getLogicalGroupingId())) {
            childByIds = logicalGroupingService.selectChildByIdNotRecursive(record.getLogicalGroupingId());
//            Assert.isTrue(ObjectUtil.isNotEmpty(childByIds), "记录 {} 未找到",record.getLogicalGroupingId());
        }
        queryWrapper.select("id", "group_code", "project_id", "logical_grouping_id", "obj_id", "obj_type", "obj_name", "create_time", "update_time",
                        "local_id", "floor_id", "floor_name", "floor_sequence_id", "space_id", "space_name", "build_id", "build_name")
                .eq(ObjectUtil.isNotEmpty(record.getGroupCode()), LogicalObject.COL_GROUP_CODE, record.getGroupCode())
                .eq(ObjectUtil.isNotEmpty(record.getProjectId()), LogicalObject.COL_PROJECT_ID, record.getProjectId())
                .in(ObjectUtil.isNotEmpty(childByIds), LogicalObject.COL_LOGICAL_GROUPING_ID, childByIds)
                .in(ObjectUtil.isNotEmpty(record.getLogicalGroupingIds()), LogicalObject.COL_LOGICAL_GROUPING_ID, record.getLogicalGroupingIds())
                .eq(ObjectUtil.isNotEmpty(record.getFloorId()), LogicalObject.COL_FLOOR_ID, record.getFloorId())
                .and(ObjectUtil.isNotEmpty(record.getKeyWord()), q -> q.like(LogicalObject.COL_OBJ_NAME, record.getKeyWord()).or().like(LogicalObject.COL_LOCAL_ID, record.getKeyWord()));
        IPage<LogicalObject> pageList = this.getBaseMapper().selectPage(memberPage, queryWrapper);

        return BeanDtoVoUtils.pageVo(pageList, LogicalObjectDTO.class);
    }

    /**
     * 查询逻辑编组下的对象不带分页
     *
     * @param record record
     * @return {@link IPage< LogicalObjectDTO>}
     * @author luogy
     * @since 2021/6/28 19:55
     */
    @Override
    public List<LogicalObjectDTO> listLogicalObjectDTO(LogicalObjectDTO record) throws Exception {
        QueryWrapper<LogicalObject> queryWrapper = new QueryWrapper<>();
        List<String> childIds = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(record.getLogicalGroupingId())) {
            childIds = logicalGroupingService.selectChildByIdNotRecursive(record.getLogicalGroupingId());
            Assert.isTrue(ObjectUtil.isNotEmpty(childIds), "编组记录未找到");
        }
        queryWrapper.select("id", "group_code", "project_id", "logical_grouping_id", "obj_id", "obj_type", "obj_name", "create_time", "update_time",
                        "floor_id", "floor_name", "floor_sequence_id", "space_id", "space_name", "build_id", "build_name")
                .eq(ObjectUtil.isNotEmpty(record.getGroupCode()), LogicalObject.COL_GROUP_CODE, record.getGroupCode())
                .eq(ObjectUtil.isNotEmpty(record.getProjectId()), LogicalObject.COL_PROJECT_ID, record.getProjectId())
                .in(ObjectUtil.isNotEmpty(childIds), LogicalObject.COL_LOGICAL_GROUPING_ID, childIds)
                .in(ObjectUtil.isNotEmpty(record.getLogicalGroupingIds()), LogicalObject.COL_LOGICAL_GROUPING_ID, record.getLogicalGroupingIds())
                .eq(ObjectUtil.isNotEmpty(record.getFloorId()), LogicalObject.COL_FLOOR_ID, record.getFloorId())
                .and(ObjectUtil.isNotEmpty(record.getKeyWord()), q -> q.like(LogicalObject.COL_OBJ_NAME, record.getKeyWord()).or().eq(LogicalObject.COL_LOCAL_ID, record.getKeyWord()));
        List<LogicalObject> logicalObjects = this.getBaseMapper().selectList(queryWrapper);
        return CovertUtil.INSTANCE.convertLogicalObjectDTOs(logicalObjects);
    }

    /**
     * 查询逻辑编组下的对象不带分页,增加localId查询的
     *
     * @param record record
     * @return {@link IPage< LogicalObjectDTO>}
     * @author luogy
     * @since 2021/6/28 19:55
     */
    public List<LogicalObjectDTO> listLogicalObjectDTOLocalId(LogicalObjectDTO record) throws Exception {
        QueryWrapper<LogicalObject> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("id", "group_code", "project_id", "logical_grouping_id", "obj_id", "obj_type", "obj_name", "create_time", "update_time",
                        "floor_id", "floor_name", "floor_sequence_id", "space_id", "space_name", "build_id", "build_name", "local_id")
                .eq(ObjectUtil.isNotEmpty(record.getGroupCode()), LogicalObject.COL_GROUP_CODE, record.getGroupCode())
                .eq(ObjectUtil.isNotEmpty(record.getProjectId()), LogicalObject.COL_PROJECT_ID, record.getProjectId())
                .in(ObjectUtil.isNotEmpty(record.getLogicalGroupingId()), LogicalObject.COL_LOGICAL_GROUPING_ID, logicalGroupingService.selectChildByIdNotRecursive(record.getLogicalGroupingId()))
                .in(ObjectUtil.isNotEmpty(record.getLogicalGroupingIds()), LogicalObject.COL_LOGICAL_GROUPING_ID, record.getLogicalGroupingIds())
                .eq(ObjectUtil.isNotEmpty(record.getFloorId()), LogicalObject.COL_FLOOR_ID, record.getFloorId())
                .nested(ObjectUtil.isNotEmpty(record.getKeyWord()), q -> q.like(LogicalObject.COL_OBJ_NAME, record.getKeyWord())
                        .or().eq(LogicalObject.COL_LOCAL_ID, record.getKeyWord()));
        List<LogicalObject> logicalObjects = this.getBaseMapper().selectList(queryWrapper);
        return CovertUtil.INSTANCE.convertLogicalObjectDTOs(logicalObjects);
    }

    /**
     * 删除编组的时候顺便删除编组对象
     *
     * @param logicalGroupingIds logicalGroupingIds
     * @return {@link boolean}
     * @author luogy
     * @since 2021/7/5 14:54
     */
    @Override
    public boolean removeByLogicalGroupingIds(List<String> logicalGroupingIds) {
        return lambdaUpdate().in(ObjectUtil.isNotEmpty(logicalGroupingIds), LogicalObject::getLogicalGroupingId, logicalGroupingIds).remove();
    }

    @Override
    public boolean removeByLogicalGroupingId(String logicalGroupingId) {
        return remove(Wrappers.<LogicalObject>query().lambda().eq(LogicalObject::getLogicalGroupingId, logicalGroupingId));
    }

    /**
     * 批量删除对象
     *
     * @param objIds objIds
     * @return {@link boolean}
     * @author luogy
     * @since 2021/7/5 14:54
     */
    @Override
    public boolean removeByLogicalGroupingObjIds(List<String> objIds, String logicalGroupingId) {
        return lambdaUpdate().in(ObjectUtil.isNotEmpty(objIds), LogicalObject::getObjId, objIds)
                .eq(ObjectUtil.isNotEmpty(objIds), LogicalObject::getLogicalGroupingId, logicalGroupingId).remove();
    }

    /**
     * 删除对象的编组关系
     *
     * @param objId objId
     * @return {@link boolean}
     * @author luogy
     * @since 2021/7/5 14:54
     */
    public boolean removeByObjId(String objId) {
        return lambdaUpdate().eq(ObjectUtil.isNotEmpty(objId), LogicalObject::getObjId, objId).remove();
    }
}
