package com.persagy.dmp.service.impl;

import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Iterables;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.persagy.dmp.dto.LogicalObjectLightingSelectsDTO;
import com.persagy.dmp.dto.logicalGrouping.LogicalGroupingDTO;
import com.persagy.dmp.dto.logicalGrouping.LogicalObjectDTO;
import com.persagy.dmp.dto.logicalGrouping.LogicalObjectTabularDTO;
import com.persagy.dmp.entity.LogicalGrouping;
import com.persagy.dmp.entity.LogicalObject;
import com.persagy.dmp.entity.LogicalObjectEquip;
import com.persagy.dmp.enumeration.EnumValid;
import com.persagy.dmp.mapper.LogicalObjectEquipMapper;
import com.persagy.dmp.mapper.LogicalObjectMapper;
import com.persagy.dmp.request.*;
import com.persagy.dmp.service.LogicalGroupingService;
import com.persagy.dmp.service.LogicalObjectEquipService;
import com.persagy.dmp.service.LogicalObjectService;
import com.persagy.dmp.service.LogicalObjectServiceV2;
import lombok.SneakyThrows;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : ZhangWenTao
 * @version : 1.0
 * @since : 2022/7/16 16:50
 */
@Service
public class LogicalObjectServiceV2Impl implements LogicalObjectServiceV2 {

    // 逻辑编组根节点父ID
    private final String rootGroupParentId = "0";

    @Autowired
    private LogicalObjectMapper logicalObjectMapper;

    @Autowired
    private LogicalObjectService logicalObjectService;

    @Autowired
    private LogicalGroupingService logicalGroupingService;

    @Autowired
    private LogicalObjectEquipMapper logicalObjectEquipMapper;

    @Autowired
    private LogicalObjectEquipService logicalObjectEquipService;

    @Override
    public IPage<LogicalObjectTabularDTO> queryLogicalObjectTabular(QueryLogicalObjectTabularRequest request) {
        // 尝试加载数据
        logicalObjectEquipService.loadIfAbsent(
                request.getGroupCode(),
                request.getProjectId(),
                request.getIbmsSceneCode(),
                request.getIbmsClassCode());

        int size = request.getSize();
        int page = request.getPage();

        // 分页查询
        if (size > 0) {
            return logicalObjectMapper.queryAllLogicalObjectTabular(
                    new Page<>(page, size),
                    request);
        }

        List<LogicalObjectTabularDTO> data = logicalObjectMapper.queryAllLogicalObjectTabular(request);
        Page<LogicalObjectTabularDTO> result = new Page<>(1, data.size(), data.size());
        result.setRecords(data);

        return result;
    }

    @Override
    public List<LogicalObjectTabularDTO> queryAllLogicalObjectTabular(QueryLogicalObjectTabularRequest request) {
        request.setSize(-1);
        return queryLogicalObjectTabular(request).getRecords();
    }

    @Override
    public LogicalObjectLightingSelectsDTO queryAllLogicalObjectLightingSelects(LogicalObjectLightingSelectsRequest request) {
        // 尝试加载数据
        logicalObjectEquipService.loadIfAbsent(
                request.getGroupCode(),
                request.getProjectId(),
                request.getIbmsSceneCode(),
                request.getIbmsClassCode());

        LogicalObjectLightingSelectsDTO resp = new LogicalObjectLightingSelectsDTO();

        // 照明分组下拉
        if (request.isWithLightingGroups()) {
            List<LogicalGrouping> logicalGroupings = logicalGroupingService.getBaseMapper()
                    .selectList(Wrappers.<LogicalGrouping>lambdaQuery()
                            .select(LogicalGrouping::getLogicalGroupingId,
                                    LogicalGrouping::getLogicalGroupingName,
                                    LogicalGrouping::getParentId)
                            .eq(LogicalGrouping::getGroupCode, request.getGroupCode())
                            .eq(LogicalGrouping::getProjectId, request.getProjectId())
                            .eq(LogicalGrouping::getIbmsSceneCode, request.getIbmsSceneCode())
                            .eq(LogicalGrouping::getIbmsClassCode, request.getIbmsClassCode()));

            if (CollectionUtils.isNotEmpty(logicalGroupings)) {
                List<Tree<String>> treeList = TreeUtil.build(logicalGroupings, "0", (o, t) -> {
                            t.setId(o.getLogicalGroupingId());
                            t.setParentId(o.getParentId());
                            t.setWeight(o.getLogicalGroupingName());
                            t.setName(o.getLogicalGroupingName());
                        }
                );

                List<LogicalObjectLightingSelectsDTO.LightingGroup> lightingGroups = Lists.newLinkedList();
                for (Tree<String> tree : treeList) {
                    List<LogicalObjectLightingSelectsDTO.SelectItem> selectItems = ObjectUtils.defaultIfNull(
                            tree.getChildren(),
                            Collections.<Tree<String>>emptyList()).stream()
                            .map(o -> LogicalObjectLightingSelectsDTO.SelectItem.builder()
                                    .id(o.getId())
                                    .name(String.valueOf(o.getName()))
                                    .build())
                            .collect(Collectors.toList());


                    lightingGroups.add(
                            LogicalObjectLightingSelectsDTO.LightingGroup.builder()
                                    .id(tree.getId())
                                    .name(String.valueOf(tree.getName()))
                                    .items(selectItems)
                                    .build());
                }

                resp.setLightingGroups(lightingGroups);
            }
        }

        List<LogicalObjectEquip> equips = logicalObjectEquipMapper.selectList(Wrappers.<LogicalObjectEquip>lambdaQuery()
                .select(LogicalObjectEquip::getId,
                        LogicalObjectEquip::getPowerBoxId,
                        LogicalObjectEquip::getPowerBoxName,
                        LogicalObjectEquip::getPowerWellsId,
                        LogicalObjectEquip::getPowerWellsName,
                        LogicalObjectEquip::getModularId,
                        LogicalObjectEquip::getModularName)
                .eq(LogicalObjectEquip::getGroupCode, request.getGroupCode())
                .eq(LogicalObjectEquip::getProjectId, request.getProjectId())
                .eq(LogicalObjectEquip::getIbmsSceneCode, request.getIbmsSceneCode())
                .eq(LogicalObjectEquip::getIbmsClassCode, request.getIbmsClassCode())
        );

        resp.setPowerBox(
                convertSelectItem(
                        equips,
                        LogicalObjectEquip::getPowerBoxName));

        resp.setPowerWells(
                convertSelectItem(
                        equips,
                        LogicalObjectEquip::getPowerWellsName));

        resp.setModular(
                convertSelectItem(
                        equips,
                        LogicalObjectEquip::getModularName));


        return resp;
    }

    private List<LogicalObjectLightingSelectsDTO.SelectItem> convertSelectItem(
            List<LogicalObjectEquip> equips,
            Function<LogicalObjectEquip, String> nameFunc) {
        return equips.stream()
                .filter(o -> ObjectUtils.allNotNull(o, nameFunc.apply(o)))
                .map(nameFunc)
                .collect(Collectors.toSet())
                .stream()
                .map(e -> LogicalObjectLightingSelectsDTO.SelectItem.builder().name(e).build())
                .sorted(Comparator.comparing(LogicalObjectLightingSelectsDTO.SelectItem::getName))
                .collect(Collectors.toList());
    }

    @Override
    @SneakyThrows
    public List<LogicalObjectDTO> listLogicalObjectDTO(ListLogicalObjectRequest request) {
        // 缓存数据加载成功, 用2.0方式查询， 否则用1.0方式查询
        String groupCode = request.getGroupCode();
        String projectId = request.getProjectId();
        String ibmsSceneCode = request.getIbmsSceneCode();
        String ibmsClassCode = request.getIbmsClassCode();

        // 精确查询直接加载数据
        if (StringUtils.isNoneEmpty(ibmsSceneCode, ibmsClassCode)) {
            logicalObjectEquipService.loadIfAbsent(
                    groupCode,
                    projectId,
                    ibmsSceneCode,
                    ibmsClassCode
            );
        }

        if (logicalObjectEquipService.tryLoadIfAbsentWithGroupings(
                groupCode,
                projectId,
                ibmsSceneCode,
                ibmsClassCode)) {
            return logicalObjectMapper.listLogicalObjectDTOWithEquipCache(request, true);
        }

        LogicalObjectDTO record = new LogicalObjectDTO();
        BeanUtils.copyProperties(request, record);

        if (StringUtils.isNotEmpty(ibmsSceneCode) || StringUtils.isNotEmpty(ibmsClassCode)) {
            if (Objects.isNull(record.getLogicalGroupingIds())) {
                record.setLogicalGroupingIds(Lists.newLinkedList());
            }

            List<String> logicalGroupingIds = record.getLogicalGroupingIds();

            LogicalGroupingDTO logicalGroupingDTO = new LogicalGroupingDTO();
            logicalGroupingDTO.setGroupCode(groupCode);
            logicalGroupingDTO.setProjectId(projectId);
            logicalGroupingDTO.setIbmsSceneCode(ibmsSceneCode);
            logicalGroupingDTO.setIbmsClassCode(ibmsClassCode);
            logicalGroupingService.listLogicalGroupingDTO(logicalGroupingDTO)
                    .getRecords()
                    .stream()
                    .map(LogicalGroupingDTO::getLogicalGroupingId)
                    .forEach(logicalGroupingIds::add);
        }

        return logicalObjectService.listLogicalObjectDTO(record);
    }

    @Override
    @Transactional
    public void coverBindLogicalObjects(CoverBindLogicalObjectsRequest request) {
        String groupCode = request.getGroupCode();
        String projectId = request.getProjectId();
        String ibmsSceneCode = request.getIbmsSceneCode();
        String ibmsClassCode = request.getIbmsClassCode();
        Set<String> objIds = request.getObjIds();

        Set<String> toUpdateParentLogicalGroupingIds = Sets.newHashSet();

        Set<String> toBindLogicalGroupingIds = request.getLogicalGroupingIds();
        Set<String> removedLogicalGroupingIds = Sets.newHashSet();

        // 验证逻辑编组是否存在
        if (CollectionUtils.isNotEmpty(toBindLogicalGroupingIds)) {
            for (String logicalGroupingId : toBindLogicalGroupingIds) {
                LogicalGrouping grouping = logicalGroupingService.getById(logicalGroupingId);

                if (Objects.isNull(grouping)) {
                    throw new IllegalArgumentException("逻辑编组不存在: " + logicalGroupingId);
                }

                toUpdateParentLogicalGroupingIds.add(grouping.getParentId());
            }
        }


        List<LogicalObject> toDeletes = logicalObjectMapper.selectList(
                Wrappers.<LogicalObject>lambdaQuery()
                        .eq(LogicalObject::getGroupCode, groupCode)
                        .eq(LogicalObject::getProjectId, projectId)
                        .in(LogicalObject::getObjId, objIds)
        );

        // 处理需要清理的数据
        if (CollectionUtils.isNotEmpty(toDeletes)) {
            List<String> toDeleteIds = Lists.newLinkedList();

            for (LogicalObject toDelete : toDeletes) {
                toDeleteIds.add(toDelete.getId());

                String logicalGroupingId = toDelete.getLogicalGroupingId();
                if (!toBindLogicalGroupingIds.contains(logicalGroupingId)) {
                    removedLogicalGroupingIds.add(logicalGroupingId);
                }
            }

            if (CollectionUtils.isNotEmpty(removedLogicalGroupingIds)) {
                logicalGroupingService.listByIds(removedLogicalGroupingIds)
                        .stream()
                        .map(LogicalGrouping::getParentId)
                        .forEach(toUpdateParentLogicalGroupingIds::add);
            }

            // 清空老数据
            logicalObjectMapper.deleteBatchIds(toDeleteIds);
        }

        toUpdateParentLogicalGroupingIds.remove("0");

        // 插入新对象数据
        insertNewestLogicalObjects(
                groupCode, projectId, ibmsSceneCode, ibmsClassCode,
                objIds, toBindLogicalGroupingIds);

        // 更新对象数量
        for (String logicalGroupingId : Iterables.concat(toBindLogicalGroupingIds, removedLogicalGroupingIds)) {
            updateLogicalGroupingObjNum(logicalGroupingId);
        }

        for (String parentLogicalGroupingId : toUpdateParentLogicalGroupingIds) {
            updateLogicalGroupingObjNumWithChild(parentLogicalGroupingId);
        }
    }

    /**
     * 插入新对象数据
     *
     * @param groupCode          分组Code
     * @param projectId          项目ID
     * @param ibmsSceneCode      场景码code
     * @param ibmsClassCode      中控对象类code
     * @param objIds             对象ID
     * @param logicalGroupingIds 逻辑编组ID列表
     */
    private void insertNewestLogicalObjects(
            String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode,
            Collection<String> objIds, Set<String> logicalGroupingIds) {

        if (CollectionUtils.isEmpty(objIds)) {
            return;
        }

        // 查询缓存的设备数据
        List<LogicalObjectEquip> equips = logicalObjectEquipMapper.selectList(
                Wrappers.<LogicalObjectEquip>lambdaQuery()
                        .eq(LogicalObjectEquip::getGroupCode, groupCode)
                        .eq(LogicalObjectEquip::getProjectId, projectId)
                        .eq(LogicalObjectEquip::getIbmsSceneCode, ibmsSceneCode)
                        .eq(LogicalObjectEquip::getIbmsClassCode, ibmsClassCode)
                        .in(LogicalObjectEquip::getObjId, objIds)
        );


        if (CollectionUtils.isNotEmpty(equips)) {

            // 通过缓存数据构建Entity
            List<LogicalObject> entities = Lists.newLinkedList();
            for (LogicalObjectEquip equip : equips) {
                for (String toBindLogicalGroupingId : logicalGroupingIds) {
                    LogicalObject entity = new LogicalObject();
                    entity.setGroupCode(groupCode);
                    entity.setProjectId(projectId);
                    entity.setObjId(equip.getObjId());
                    entity.setLogicalGroupingId(toBindLogicalGroupingId);
                    entity.setObjName(equip.getObjName());
                    entity.setLocalId(equip.getLocalId());
                    entity.setObjType(equip.getObjType());
                    entity.setGroupExtra(equip.getGroupExtra());

                    entities.add(entity);
                }
            }


            // 插入数据
            logicalObjectService.saveBatch(entities, 200);
        }
    }

    /**
     * 更新逻辑分组对象数量字段
     *
     * @param logicalGroupingId 逻辑分组ID
     */
    private void updateLogicalGroupingObjNum(String logicalGroupingId) {
        //更新编组对象数量
        updateLogicalGroupingObjNumInternal(
                logicalGroupingId,
                logicalObjectService.count(
                        Wrappers.<LogicalObject>lambdaQuery()
                                .eq(LogicalObject::getLogicalGroupingId, logicalGroupingId))
        );
    }

    /**
     * 更新逻辑分组对象数量字段(包含子分组数量)
     *
     * @param parentLogicalGroupingId 逻辑分组ID
     */
    private void updateLogicalGroupingObjNumWithChild(String parentLogicalGroupingId) {
        Set<String> relationLogicalGroupingIds = logicalGroupingService.list(
                Wrappers.<LogicalGrouping>lambdaQuery()
                        .eq(LogicalGrouping::getLogicalGroupingId, parentLogicalGroupingId)
                        .or()
                        .eq(LogicalGrouping::getParentId, parentLogicalGroupingId))
                .stream()
                .map(LogicalGrouping::getLogicalGroupingId)
                .collect(Collectors.toSet());

        updateLogicalGroupingObjNumInternal(
                parentLogicalGroupingId,
                logicalObjectService.count(
                        Wrappers.<LogicalObject>lambdaQuery()
                                .in(LogicalObject::getLogicalGroupingId, relationLogicalGroupingIds))
        );
    }

    /**
     * 修改逻辑编组下对象数量
     *
     * @param logicalGroupingId 逻辑编组ID
     * @param objNum            对象数量
     */
    private void updateLogicalGroupingObjNumInternal(String logicalGroupingId, int objNum) {
        LogicalGrouping logicalGrouping = new LogicalGrouping();
        logicalGrouping.setLogicalGroupingId(logicalGroupingId);
        logicalGrouping.setObjNum(objNum);

        logicalGroupingService.updateById(logicalGrouping);
    }

    @Override
    @Transactional
    public void saveBindLogicalObjects(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode,
                                       String logicalGroupingId, List<String> objIds) {
        // 验证逻辑编组是否存在
        if (StringUtils.isEmpty(logicalGroupingId)) {
            throw new IllegalArgumentException("逻辑编组ID不能为空");
        }

        LogicalGrouping grouping = logicalGroupingService.getById(logicalGroupingId);
        if (Objects.isNull(grouping)) {
            throw new IllegalArgumentException("逻辑编组不存在");
        }

        // 删除老数据
        logicalObjectMapper.delete(
                Wrappers.<LogicalObject>lambdaQuery()
                        .eq(LogicalObject::getGroupCode, groupCode)
                        .eq(LogicalObject::getProjectId, projectId)
                        .eq(LogicalObject::getLogicalGroupingId, logicalGroupingId)
        );

        // 插入新对象数据
        insertNewestLogicalObjects(
                groupCode, projectId, ibmsSceneCode, ibmsClassCode,
                objIds, Collections.singleton(logicalGroupingId));

        //更新对象数量
        updateLogicalGroupingObjNum(logicalGroupingId);

        // 非根节点逻辑编组更新父项对象数量
        if (!"0".equals(grouping.getParentId())) {
            updateLogicalGroupingObjNumWithChild(grouping.getParentId());
        }

    }


    @Override
    @Transactional
    public void removeBindLogicalObjects(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode, String logicalGroupingId) {
        // 验证逻辑编组是否存在
        if (StringUtils.isEmpty(logicalGroupingId)) {
            throw new IllegalArgumentException("逻辑编组ID不能为空");
        }

        List<String> childLogicalGroupingIds = Lists.newLinkedList();

        LogicalGrouping grouping = logicalGroupingService.getById(logicalGroupingId);
        if (Objects.isNull(grouping)) {
            throw new IllegalArgumentException("逻辑编组不存在");
        }

        // 根节点得话同时删除子节点的对象数据
        String parentLogicalGroupingId = grouping.getParentId();
        if (rootGroupParentId.equals(parentLogicalGroupingId)) {
            logicalGroupingService.list(
                    Wrappers.<LogicalGrouping>lambdaQuery()
                            .eq(LogicalGrouping::getParentId, logicalGroupingId))
                    .stream()
                    .map(LogicalGrouping::getLogicalGroupingId)
                    .forEach(childLogicalGroupingIds::add);
        }

        // 先删除子节点的绑定关系
        if (CollectionUtils.isNotEmpty(childLogicalGroupingIds)) {
            logicalObjectMapper.delete(
                    Wrappers.<LogicalObject>lambdaQuery()
                            .in(LogicalObject::getLogicalGroupingId, childLogicalGroupingIds)
            );

            // 修改对象数量
            for (String childLogicalGroupingId : childLogicalGroupingIds) {
                updateLogicalGroupingObjNumInternal(childLogicalGroupingId, 0);
            }
        }

        // 删除当前节点
        logicalObjectMapper.delete(
                Wrappers.<LogicalObject>lambdaQuery()
                        .eq(LogicalObject::getLogicalGroupingId, logicalGroupingId)
        );

        updateLogicalGroupingObjNumInternal(logicalGroupingId, 0);

        // 父节点处理
        if (!rootGroupParentId.equals(parentLogicalGroupingId)) {
            updateLogicalGroupingObjNumWithChild(parentLogicalGroupingId);
        }


    }

    @Override
    public int countOfAllLogicalObjectEquip(String groupCode, String projectId, String ibmsSceneCode, String ibmsClassCode) {
        return logicalObjectEquipMapper.selectCount(
                Wrappers.<LogicalObjectEquip>lambdaQuery()
                        .eq(LogicalObjectEquip::getGroupCode, groupCode)
                        .eq(LogicalObjectEquip::getProjectId, projectId)
                        .eq(LogicalObjectEquip::getIbmsSceneCode, ibmsSceneCode)
                        .eq(LogicalObjectEquip::getIbmsClassCode, ibmsClassCode)
                        .eq(LogicalObjectEquip::getValid, EnumValid.TRUE.getValue())
        );
    }

    @Override
    @Transactional
    public void unBindLogicalObjects(UnBindLogicalObjectsRequest request) {
        String logicalGroupingId = request.getLogicalGroupingId();

        LogicalGrouping grouping = logicalGroupingService.getById(logicalGroupingId);
        if (Objects.isNull(grouping)) {
            throw new IllegalArgumentException("逻辑编组不存在");
        }

        String groupCode = request.getGroupCode();
        String projectId = request.getProjectId();

        Set<String> objIds = request.getObjIds();

        // 删除关系
        logicalObjectMapper.delete(
                Wrappers.<LogicalObject>lambdaQuery()
                        .eq(LogicalObject::getGroupCode, groupCode)
                        .eq(LogicalObject::getProjectId, projectId)
                        .in(LogicalObject::getObjId, objIds)
        );

        // 更新逻辑编组对象绑定数量
        updateLogicalGroupingObjNum(logicalGroupingId);

        // 非根节点修改父编组对象绑定数量
        String parentLogicalGroupingId = grouping.getParentId();
        if (!rootGroupParentId.equals(parentLogicalGroupingId)) {
            updateLogicalGroupingObjNumWithChild(parentLogicalGroupingId);
        }
    }
}