package com.persagy.dmp.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.dmp.common.PageResult;
import com.persagy.dmp.dao.ObjectClassConfigBlackDao;
import com.persagy.dmp.dao.ObjectClassConfigDao;
import com.persagy.dmp.dao.ObjectClassFuncRelDao;
import com.persagy.dmp.dao.ObjectGroupConfigDao;
import com.persagy.dmp.dao.ObjectGroupRelConfigDao;
import com.persagy.dmp.dto.InfoPointDictDTO;
import com.persagy.dmp.dto.objectclass.ModuleFuncObjClassDTO;
import com.persagy.dmp.entity.ObjectClassConfig;
import com.persagy.dmp.entity.ObjectClassConfigBlack;
import com.persagy.dmp.entity.ObjectClassFuncRel;
import com.persagy.dmp.entity.ObjectGroupConfig;
import com.persagy.dmp.entity.ObjectGroupRelConfig;
import com.persagy.dmp.service.DictService;
import com.persagy.dmp.service.ObjectClassFuncRelService;
import com.persagy.dmp.service.ObjectService;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service
public class ObjectClassFuncRelServiceImpl implements ObjectClassFuncRelService {
    private final ObjectClassFuncRelDao objectClassFuncRelDao;
    private final ObjectClassConfigDao objectClassConfigDao;
    private final ObjectClassConfigBlackDao objectClassConfigBlackDao;
    private final ObjectGroupConfigDao objectGroupConfigDao;
    private final ObjectGroupRelConfigDao objectGroupRelConfigDao;

    @Resource
    private DictService dictService;

    @Resource
    private ObjectService objectService;


    /**
     * 获取模块功能对应的对象类配置
     *
     * @param param
     * @return 平铺结构数据
     * @throws Exception
     */
    @Override
    public List<ObjectClassConfig> queryObjClassConfigMap(ModuleFuncObjClassDTO param) throws Exception {
        List<ObjectClassFuncRel> mapDataList = queryObjClassFuncMapRoot(param);

        if (!mapDataList.isEmpty()) {
            Set<String> idSet = new HashSet<>();
            for (ObjectClassFuncRel item : mapDataList) {
                idSet.add(item.getId());
            }

            QueryWrapper<ObjectClassConfig> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq(StringUtils.isNotBlank(param.getParentCode()), ObjectClassFuncRel.COL_PARENT_CODE, param.getParentCode())
                    .in(ObjectClassConfig.COL_FUNCTION_ID, idSet);

            List<ObjectClassConfig> records = objectClassConfigDao.list(queryWrapper);

            return records;
        }

        return new ArrayList<>();
    }

    /**
     * 获取模块配置根节点数据
     *
     * @param param
     * @return 平铺结构数据
     * @throws Exception
     */
    @Override
    public List<ObjectClassFuncRel> queryObjClassFuncMapRoot(ModuleFuncObjClassDTO param) throws Exception {
        QueryWrapper<ObjectClassFuncRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(param.getGroupCode()), ObjectClassFuncRel.COL_GROUP_CODE, param.getGroupCode())
                .eq(StringUtils.isNotBlank(param.getProjectId()), ObjectClassFuncRel.COL_PROJECT_ID, param.getProjectId())
                .eq(StringUtils.isNotBlank(param.getModuleCode()), ObjectClassFuncRel.COL_MODULE_CODE, param.getModuleCode())
                .eq(StringUtils.isNotBlank(param.getFunctionCode()), ObjectClassFuncRel.COL_FUNCTION_CODE, param.getFunctionCode())
                .eq(StringUtils.isNotBlank(param.getCode()), ObjectClassFuncRel.COL_CODE, param.getCode());
        List<ObjectClassFuncRel> records = objectClassFuncRelDao.list(queryWrapper);

        if (!ObjectUtil.isEmpty(records)) {
            return records;
        }

        return new ArrayList<>();
    }

    /**
     * 获取某一自定义对象类配置的信息点定义
     *
     * @param param 属性code不能为空
     * @return
     * @throws Exception
     */
    @Override
    public List<InfoPointDictDTO> queryObjectDict(ModuleFuncObjClassDTO param) throws Exception {

        List<ObjectClassConfig> records = getObjectClassConfigs(param);


        if (!ObjectUtil.isEmpty(records)) {
            // 设备类自定义字典编码，设备类数据字典编码
            String localCode = null, dicCode = null;

            // {设备类自定义字典编码: {信息点编码：信息点定义}}
            Map<String, Map<String, InfoPointDictDTO>> localCode_infoCode2InfoPointObj = new HashMap<>();

            for (ObjectClassConfig item : records) {
                dicCode = item.getDicCode();

                if (ObjectUtil.isEmpty(dicCode)) {
                    continue;
                }

                localCode = item.getCode();

                // 当前设备类localCode配置的需要用到的 {信息点编码：信息点定义}}
                Map<String, InfoPointDictDTO> infoCode2InfoPointDictDTO = localCode_infoCode2InfoPointObj.get(localCode);
                if (null == infoCode2InfoPointDictDTO) {
                    infoCode2InfoPointDictDTO = new HashMap<>();
                }

                // {信息点编码：信息点定义} 查询数据字典中，设备类的所有信息点定义
                Map<String, InfoPointDictDTO> infoPointDictDTOMap = dictService.queryInfoPointDictMap(param.getGroupCode(), param.getProjectId(), dicCode);

                // 本地对于标准信息点的配置
                JSONObject infoPointConf = item.getItemData();


                if (ObjectUtil.isEmpty(infoPointConf)) {
                    // 若无该配置，则默认使用数据字典的全部配置
                    infoCode2InfoPointDictDTO.putAll(infoPointDictDTOMap);
                } else {
                    // 使用本地配置过滤不使用的信息点，对已使用的信息点定义依据本地配置进行修改
                    for (String code : infoPointConf.keySet()) {
                        // 数据字典信息点编码code 对应的数据字典信息点标准定义
                        InfoPointDictDTO tempInfoPoint = infoPointDictDTOMap.get(code);

                        // 数据字典信息点编码code 对应的本地信息点配置
                        JSONObject localConf = infoPointConf.getJSONObject(code);
                        if (!ObjectUtil.isEmpty(localConf)) {
                            // 将本地配置应用到标准信息点定义上
                            BeanUtil.copyProperties(localConf, tempInfoPoint);
                        }

                        infoCode2InfoPointDictDTO.put(code, tempInfoPoint);
                    }
                }

                if (!infoPointDictDTOMap.isEmpty()) {
                    localCode_infoCode2InfoPointObj.put(localCode, infoCode2InfoPointDictDTO);
                }
            }

            Map<String, InfoPointDictDTO> combineResult = new HashMap<>();
            if (!localCode_infoCode2InfoPointObj.isEmpty()) {
                // 信息点合并，若信息点编码有相同的则覆盖
                for (String local_code : localCode_infoCode2InfoPointObj.keySet()) {
                    combineResult.putAll(localCode_infoCode2InfoPointObj.get(local_code));
                }

                return new ArrayList<>(combineResult.values());
            }
        }

        return new ArrayList<>();
    }

    /**
     * 获取 某一模块-功能-设备类分类关联的数据字典设备类配置记录
     *
     * @param param
     * @return 无数据返回空集
     * @throws Exception
     */
    private List<ObjectClassConfig> getObjectClassConfigs(ModuleFuncObjClassDTO param) throws Exception {
        String functionId = getFunctionId(param);
        if (null != param) {
            QueryWrapper<ObjectClassConfig> queryWrapper = new QueryWrapper<>();

            // 目前只支持二级结构
            queryWrapper.eq(true, ObjectClassConfig.COL_FUNCTION_ID, functionId)
                    .eq(true, ObjectClassConfig.COL_PARENT_CODE, param.getClassCategory());

            return objectClassConfigDao.list(queryWrapper);
        }

        return new ArrayList<>();
    }

    /**
     * 获取模块-功能所用的配置映射id
     *
     * @param param
     * @return
     * @throws Exception
     */
    private String getFunctionId(ModuleFuncObjClassDTO param) throws Exception {
        String functionId = null;

        // 先找到functionMapId TODO functionId 可以放缓存
        ModuleFuncObjClassDTO paramForFuncMapId = new ModuleFuncObjClassDTO();
        BeanUtil.copyProperties(param, paramForFuncMapId);
        paramForFuncMapId.setCode(null);

        List<ObjectClassFuncRel> objectClassFuncRels = queryObjClassFuncMapRoot(paramForFuncMapId);
        if (!objectClassFuncRels.isEmpty()) {
            functionId = objectClassFuncRels.get(0).getId();
        }
        return functionId;
    }

    /**
     * 获取某一自定义对象类下面的设备实例
     *
     * @param param 属性functionCode和classCategory不能为空
     * @return
     * @throws Exception
     */
    @Override
    public JSONArray queryObjectInstanceOfCategory(ModuleFuncObjClassDTO param) throws Exception {
        List<ObjectClassConfig> classConfigs = getObjectClassConfigs(param);

        JSONArray result = new JSONArray();

        if (!ObjectUtil.isEmpty(classConfigs)) {
            for (ObjectClassConfig item : classConfigs) {
                // 逐个数据字典对象类型从中台查询数据

                if (null == item.getDicCode()) {
                    continue;
                }

                JSONObject criteria = new JSONObject();
                criteria.put("classCode", item.getDicCode());
                if (null != item.getItemFilterData()) {
                    criteria.putAll(item.getItemFilterData());
                }

                JSONObject queryParam = queryDictObjectsParam(criteria, null == item.getItemData() ? null : item.getItemData().keySet(), false);

                PageResult<JSONArray> objPage = objectService.queryObjectList(param.getGroupCode(), param.getProjectId(), queryParam);
                JSONArray objArr = objPage.getData();

                if (!CollectionUtils.isEmpty(objArr)) {
                    // 黑名单过滤 TODO 可考虑缓存黑名单数据
                    Set<String> blackObjIds = getBlackIds(item.getId());

                    for (int i = 0; i < objArr.size(); i++) {
                        JSONObject objJson = objArr.getJSONObject(i);
                        if (!blackObjIds.contains(objJson.getString("id"))) {
                            result.add(objJson);
                        }
                    }
                }
            }
        }

        return result;
    }

    /**
     * 根据对象类配置信息的id获取下面的设备黑名单记录
     *
     * @param classConfigId
     * @return
     * @throws Exception
     */
    @Override
    public List<ObjectClassConfigBlack> queryObjBlackOfClassConfig(String classConfigId) throws Exception {
        if (null != classConfigId) {
            QueryWrapper<ObjectClassConfigBlack> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq(ObjectClassConfigBlack.COL_CLASS_CONFIG_ID, classConfigId);

            return objectClassConfigBlackDao.list(queryWrapper);
        }

        return new ArrayList<>();
    }

    /**
     * 根据对象类配置信息的id获取对象分组列表
     *
     * @param classConfigId
     * @return
     * @throws Exception
     */
    @Override
    public List<ObjectGroupConfig> queryObjectGroupConfig(String classConfigId) throws Exception {
        if (classConfigId != null) {
            QueryWrapper<ObjectGroupConfig> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq(ObjectGroupConfig.COL_CLASS_CONFIG_ID, classConfigId);

            return objectGroupConfigDao.list(queryWrapper);
        }

        return new ArrayList<>();
    }

    /**
     * 根据分组id获取对象与分组关系数据列表
     *
     * @param groupId
     * @return
     * @throws Exception
     */
    @Override
    public List<ObjectGroupRelConfig> queryObjectGroupRelConfig(String groupId) throws Exception {
        if (null != groupId) {
            QueryWrapper<ObjectGroupRelConfig> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq(ObjectGroupRelConfig.COL_GROUP_ID, groupId);

            return objectGroupRelConfigDao.list(queryWrapper);
        }

        return new ArrayList<>();
    }

    /**
     * 根据对象类配置id获取下面的黑名单对象id集合
     *
     * @param classConfigId
     * @return
     */
    private Set<String> getBlackIds(String classConfigId) throws Exception {
        HashSet<String> result = new HashSet<>();

        List<ObjectClassConfigBlack> blackList = queryObjBlackOfClassConfig(classConfigId);
        if (!CollectionUtils.isEmpty(blackList)) {
            blackList.stream().forEach(item -> result.add(item.getObjectId()));
        }
        return result;
    }

    /**
     * 按对象类型码查询中台对象实例的参数
     *
     * @param criteria    查询条件， 不为空
     * @param withColumns 可为空
     * @param onlyCount
     * @return
     */
    private JSONObject queryDictObjectsParam(JSONObject criteria, Set<String> withColumns, boolean onlyCount) {
        JSONObject param = new JSONObject();

        param.put("criteria", criteria);

        param.put("withColumns", withColumns);

        param.put("onlyCount", onlyCount);

        return param;
    }
}
