package com.persagy.dmp.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.dmp.dao.ObjectMeterFunctionDao;
import com.persagy.dmp.entity.ObjectMeterFunction;
import com.persagy.dmp.service.ObjectMeterFunctionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @description: 对象表号-功能号实现类
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2021/3/15 17:54
 * @version: V1.0
 **/
@Service
public class ObjectMeterFunctionServiceImpl implements ObjectMeterFunctionService {

    @Autowired
    private ObjectMeterFunctionDao objectMeterFunctionDao;

    @Override
    public void saveObjectMeterFunction(String groupCode, String projectId, String objectId, List<ObjectMeterFunction> objectMeterFunctionList) {
        QueryWrapper<ObjectMeterFunction> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObjectMeterFunction::getGroupCode, groupCode);
        queryWrapper.lambda().eq(ObjectMeterFunction::getProjectId, projectId);
        queryWrapper.lambda().eq(ObjectMeterFunction::getObjId, objectId);
        objectMeterFunctionDao.remove(queryWrapper);
        objectMeterFunctionDao.saveBatch(objectMeterFunctionList);
    }

    @Override
    @Cacheable(value = "meterFunction")
    public ObjectMeterFunction queryMeterFunctionByObj(String groupCode, String projectId, String objectId, String infoCode) throws Exception {
        QueryWrapper<ObjectMeterFunction> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObjectMeterFunction::getGroupCode, groupCode);
        queryWrapper.lambda().eq(ObjectMeterFunction::getProjectId, projectId);
        queryWrapper.lambda().eq(ObjectMeterFunction::getObjId, objectId);
        queryWrapper.lambda().eq(ObjectMeterFunction::getInfoCode, infoCode);
        List<ObjectMeterFunction> objectMeterFunctionList = objectMeterFunctionDao.list(queryWrapper);
        if (!CollectionUtils.isEmpty(objectMeterFunctionList)) {
            return objectMeterFunctionList.get(0);
        }
        return null;
    }

    @Override
    @Cacheable(value = "meterFunction")
    public List<ObjectMeterFunction> getMeterFunctionList(String groupCode, String projectId, List<String> objIds, String infoCode) throws Exception {
        QueryWrapper<ObjectMeterFunction> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObjectMeterFunction::getGroupCode, groupCode);
        queryWrapper.lambda().eq(ObjectMeterFunction::getProjectId, projectId);
        queryWrapper.lambda().in(ObjectMeterFunction::getObjId, objIds);
        queryWrapper.lambda().eq(ObjectMeterFunction::getInfoCode, infoCode);
        return objectMeterFunctionDao.list(queryWrapper);
    }

    @Override
    @Cacheable(value = "meterFunction")
    public List<ObjectMeterFunction> getMeterFunctionList(String groupCode, String projectId, String objId, List<String> infoCodes) throws Exception {
        QueryWrapper<ObjectMeterFunction> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObjectMeterFunction::getGroupCode, groupCode);
        queryWrapper.lambda().eq(ObjectMeterFunction::getProjectId, projectId);
        queryWrapper.lambda().eq(ObjectMeterFunction::getObjId, objId);
        queryWrapper.lambda().in(ObjectMeterFunction::getInfoCode, infoCodes);
        return objectMeterFunctionDao.list(queryWrapper);
    }

    @Override
    @Cacheable(value = "meterFunction")
    public ObjectMeterFunction queryObjectByMeter(String groupCode, String projectId, String meterId, Long funcId) {
        QueryWrapper<ObjectMeterFunction> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObjectMeterFunction::getGroupCode, groupCode);
        queryWrapper.lambda().eq(ObjectMeterFunction::getProjectId, projectId);
        queryWrapper.lambda().eq(ObjectMeterFunction::getMeter, meterId);
        queryWrapper.lambda().eq(ObjectMeterFunction::getFunction, funcId);
        List<ObjectMeterFunction> objectMeterFunctionList = objectMeterFunctionDao.list(queryWrapper);
        if (!CollectionUtils.isEmpty(objectMeterFunctionList)) {
            return objectMeterFunctionList.get(0);
        }
        return null;
    }
}
