package com.persagy.dmp.service.impl;


import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.google.common.collect.Lists;
import com.persagy.dmp.auth.AuthManager;
import com.persagy.dmp.client.PersonCenterClient;
import com.persagy.dmp.common.IntegratedResponseResult;
import com.persagy.dmp.constant.CommonConstant;
import com.persagy.dmp.constant.GeneralDictionaryConstant;
import com.persagy.dmp.dao.OperationLogDao;
import com.persagy.dmp.dto.DeptJobId;
import com.persagy.dmp.dto.GeneralDictionaryDTO;
import com.persagy.dmp.dto.OperationLogDTO;
import com.persagy.dmp.dto.QueryPersonDTO;
import com.persagy.dmp.entity.*;
import com.persagy.dmp.service.*;
import com.persagy.dmp.typeHandler.SaasOrgPersonJarHandler;
import com.persagy.dmp.utils.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static com.persagy.dmp.constant.CommonConstant.SYSTEM_ID;
import static com.persagy.dmp.constant.LogConstant.FUNCTION_LOG;

/**
 * 业务日志表（业务操作记录）服务接口实现
 *
 * @author luoguangyi
 * @description 由 Mybatisplus Code Generator 创建
 * @since 2020-12-03 15:18:07
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class OperationLogServiceImpl implements OperationLogService {
    private final OperationLogDao operationLogDao;

    @Autowired
    PersonCenterClient personCenterClient;
    @Autowired
    ClassInfoService classInfoService;
    @Autowired
    ObjectService objectService;
    @Autowired
    ZktModuleService zktModuleService;
    @Autowired
    GeneralDictionaryService generalDictionaryService;
    @Autowired
    SaasOrgPersonJarHandler saasOrgPersonHandler;
    @Autowired
    ZktJarRoleService zktRoleService;

    @Value("${user.login.admin.username:PERSAGYADMIN}")
    private String adminUsername;

    /**
     * @param operationLogDTO:
     * @description: 保存业务日志
     * @author: LuoGuangyi
     * @createTime: 2020/12/04 19:06
     * @return: com.persagy.dmp.dto.OperationLogDTO
     * @expression
     */
    @Override
    public OperationLogDTO save(OperationLogDTO operationLogDTO) throws Exception {
        log.debug("保存业务日志的请求参数：{}", JSONObject.toJSONString(operationLogDTO));
        String functionType = operationLogDTO.getFunctionType();
        Integer sourceType = operationLogDTO.getSourceType();
        String groupCode = operationLogDTO.getGroupCode();
        Assert.notBlank(functionType, "argument [{}] must not empty", OperationLog.COL_FUNCTION_TYPE);
        Assert.notNull(sourceType, "argument [{}] must not empty", OperationLog.COL_SOURCE_TYPE);
        Assert.notNull(groupCode, "argument [groupCode] must not empty");
        Map<String, String> dictDataCodeLabelMap = getLogCodeNameMap(groupCode, operationLogDTO.getProjectId());
        Assert.isTrue(dictDataCodeLabelMap.containsKey(functionType), "argument [{}]:{} can not find,please check!", OperationLog.COL_FUNCTION_TYPE, functionType);
        String userId = operationLogDTO.getUserId();
//        if (ObjectUtil.isNotEmpty(userId) && !SYSTEM_ID.equals(userId) && !ObjectUtil.isAllNotEmpty(operationLogDTO.getDeptJobIds(), operationLogDTO.getUserName())) {
        if (ObjectUtil.isNotEmpty(userId) && !SYSTEM_ID.equals(userId) && !adminUsername.equals(userId)) {
            try {
                JWTUserDto jwtUserDto = AuthManager.currentUser();
                if (jwtUserDto != null) {
                    log.debug("当前用户信息为", JSONObject.toJSONString(jwtUserDto));
                    DeptJobId deptJobId = new DeptJobId();
                    deptJobId.setDeptName(jwtUserDto.getRoleName());
                    operationLogDTO.setDeptJobIds(Lists.newArrayList(deptJobId));
                    operationLogDTO.setRoleName(jwtUserDto.getRoleName());
                    operationLogDTO.setUserId(jwtUserDto.getUserId());
                    operationLogDTO.setUserName(jwtUserDto.getUsername());
                } else if ("XG".equals(operationLogDTO.getGroupCode())) {
                    log.debug("香港置地保存日志");
                    JSONObject object = this.saasOrgPersonHandler.getPersonRole(operationLogDTO.getGroupCode(), operationLogDTO.getProjectId(), null, userId);
                    String ROLE_ID = "roleId";
                    String USER_NAME = "name";
                    if (object != null) {
                        log.debug("获取用户信息:{}", JSONObject.toJSONString(object));
                        operationLogDTO.setUserName(object.getString(USER_NAME));
                        if (StrUtil.isNotBlank(object.getString(ROLE_ID))) {
                            log.warn("获取角色名称,没有关联账户或没有设置角色,groupCode:{},projectId:{},userId:{}", operationLogDTO.getGroupCode(), operationLogDTO.getProjectId(), operationLogDTO.getProjectId());
                            ZktJarRole zktRole = this.zktRoleService.getRole(object.getString(ROLE_ID));
                            if (zktRole == null) {
                                log.warn("获取角色名称,角色ID无效,groupCode:{},projectId:{},userId:{},roleId:{}", operationLogDTO.getGroupCode(), operationLogDTO.getProjectId(), operationLogDTO.getProjectId(), object.getString(ROLE_ID));
                            } else {
                                operationLogDTO.setRoleName(zktRole.getRoleName());
                            }
                        }
                    } else {
                        log.debug("返回的用户信息是null");
                    }
                } else {
                    log.debug("token没有获取到用户信息，准备去person-center查询用户信息");
                    QueryPersonDTO personDTO = QueryPersonDTO.builder().id(userId).userId(userId).appId("0").groupCode(operationLogDTO.getGroupCode()).build();
                    IntegratedResponseResult<JSONObject> responseResult = personCenterClient.queryPersonDetailById(personDTO);
                    log.debug("responseResult:{}", JSONObject.toJSONString(responseResult));
                    if (ObjectUtil.isNotEmpty(responseResult.getContent())) {
                        log.debug("responseResult.getContent():{}", JSONObject.toJSONString(responseResult.getContent()));
                        //"[{\"deptId\":\"部门实例id\",\"deptName\":\"部门实例名称\",\"jobClassifyCode\":\"岗位分类编码\",\"jobClassifyName\":\"岗位分类名称\"}]";
                        JSONArray deptJobIds = responseResult.getContent().getJSONArray("deptJobIds");
                        operationLogDTO.setDeptJobIds(deptJobIds.toJavaList(DeptJobId.class));
                        if (ObjectUtil.isEmpty(operationLogDTO.getUserName())) {
                            operationLogDTO.setUserName(responseResult.getContent().getString("name"));
                        }
                    }
                }
            } catch (Exception e) {
                log.warn("查询人员信息失败", e);
            }
        } else if (SYSTEM_ID.equals(userId)) {
            operationLogDTO.setUserName(CommonConstant.SYSTEM_NAME);
            operationLogDTO.setRoleName(CommonConstant.SYSTEM_NAME);
        } else if (adminUsername.equals(userId)) {
            operationLogDTO.setUserName(adminUsername);
            operationLogDTO.setRoleName("超级管理员");
        }
        //系统名称赋值
        if (ObjectUtil.isNotEmpty(operationLogDTO.getSystemCode()) && ObjectUtil.isEmpty(operationLogDTO.getSystemName()) && ObjectUtil.isNotEmpty(operationLogDTO.getProjectId())) {
            Map<String, String> systemCodeNameMap = classInfoService.querySystemCodeNameMap(groupCode, operationLogDTO.getProjectId());
            operationLogDTO.setSystemName(systemCodeNameMap.get(operationLogDTO.getSystemCode()));
        }
        //对象名称,对象类型赋值
        if (ObjectUtil.isNotEmpty(operationLogDTO.getObjId()) && !ObjectUtil.isAllNotEmpty(operationLogDTO.getObjName(), operationLogDTO.getObjType(), operationLogDTO.getClassCode()) && ObjectUtil.isNotEmpty(operationLogDTO.getProjectId())) {
            JSONObject objJson = objectService.queryObjectInfo(groupCode, operationLogDTO.getProjectId(), operationLogDTO.getObjId());
            operationLogDTO.setObjName(objJson.getString("name"));
            operationLogDTO.setObjType(objJson.getString("objType"));
            operationLogDTO.setClassCode(objJson.getString("classCode"));
        }
        if (StringUtils.isNotBlank(operationLogDTO.getProjectId()) && StringUtils.isNotBlank(groupCode) && StringUtils.isNotBlank(operationLogDTO.getIbmsSceneCode())) {
            ZktModule zktModule = this.zktModuleService.getModule(operationLogDTO.getProjectId(), groupCode, operationLogDTO.getIbmsSceneCode());
            if (zktModule != null) {
                operationLogDTO.setModuleName(zktModule.getModuleName());
            }
        }
        OperationLog operationLog = CovertUtil.INSTANCE.convertOperationLog(operationLogDTO);
        log.debug("保存业务日志，获取的IP信息：{}", IpUtil.getIpAddress());
        operationLog.setIpAddress(IpUtil.getIpAddress());
        boolean save = operationLogDao.save(operationLog);
        Assert.isTrue(save, "保存日志失败");
        OperationLogDTO queryOperationLog = OperationLogDTO.builder().groupCode(groupCode).logId(operationLog.getLogId()).build();
        queryOperationLog.setPage(1);
        queryOperationLog.setSize(10);
        IPage<OperationLogDTO> operationLogDTOIPage = listOperationLog(queryOperationLog);
        Assert.isTrue(operationLogDTOIPage.getSize() > 0, "保存日志失败");
        return operationLogDTOIPage.getRecords().get(0);
    }

    /**
     * @param operationLogDTOs:
     * @description: 批量保存业务日志
     * @author: LuoGuangyi
     * @createTime: 2020/12/04 19:05
     * @return: java.util.List<com.persagy.dmp.dto.OperationLogDTO>
     * @expression
     */
    @Override
    public IPage<OperationLogDTO> saveBatch(List<OperationLogDTO> operationLogDTOs) throws Exception {
        JWTUserDto jwtUserDto = AuthManager.currentUser();
        log.debug("批量保存业务日志，从token解析的用户信息,{}", jwtUserDto);
        log.debug("批量保存业务日志，获取的IP信息：{}", IpUtil.getIpAddress());
        String superAdmin = "超级管理员";
        String hk = "XG";
        for (OperationLogDTO item : operationLogDTOs) {
            item.setIpAddress(IpUtil.getIpAddress());
            if (jwtUserDto != null) {
                item.setRoleName(jwtUserDto.getRoleName());
                item.setUserId(jwtUserDto.getUserId());
                item.setUserName(jwtUserDto.getName());
            } else if (SYSTEM_ID.equals(item.getUserId())) {
                log.debug("批量保存业务日志，userId是systemId,{}", JSONObject.toJSONString(item));
                item.setRoleName(CommonConstant.SYSTEM_NAME);
                item.setUserName(CommonConstant.SYSTEM_NAME);
            } else if (adminUsername.equals(item.getUserId())) {
                log.debug("批量保存业务日志，userId是{}，角色设置为超级管理员", item.getUserId());
                item.setUserName(adminUsername);
                item.setRoleName(superAdmin);
            } else if (hk.equals(item.getGroupCode())) {
                log.debug("香港置地批量保存日志");
                JSONObject object = this.saasOrgPersonHandler.getPersonRole(item.getGroupCode(), item.getProjectId(), null, item.getUserId());
                String ROLE_ID = "roleId";
                String USER_NAME = "name";
                if (object != null) {
                    log.debug("获取用户信息:{}", JSONObject.toJSONString(object));
                    item.setUserName(object.getString(USER_NAME));
                    if (StrUtil.isNotBlank(object.getString(ROLE_ID))) {
                        log.warn("获取角色名称,没有关联账户或没有设置角色,groupCode:{},projectId:{},userId:{}", item.getGroupCode(), item.getProjectId(), item.getProjectId());
                        ZktJarRole zktRole = this.zktRoleService.getRole(object.getString(ROLE_ID));
                        if (zktRole == null) {
                            log.warn("获取角色名称,角色ID无效,groupCode:{},projectId:{},userId:{},roleId:{}", item.getGroupCode(), item.getProjectId(), item.getProjectId(), object.getString(ROLE_ID));
                        } else {
                            item.setRoleName(zktRole.getRoleName());
                        }
                    }
                } else {
                    log.debug("返回的用户信息是null");
                }
            } else {
                log.debug("token没有获取到用户信息，准备去person-center查询用户信息");
                QueryPersonDTO personDTO = QueryPersonDTO.builder().id(item.getUserId()).userId(item.getUserId()).appId("0").groupCode(item.getGroupCode()).build();
                IntegratedResponseResult<JSONObject> responseResult = personCenterClient.queryPersonDetailById(personDTO);
                log.debug("responseResult:{}", JSONObject.toJSONString(responseResult));
                if (ObjectUtil.isNotEmpty(responseResult.getContent())) {
                    log.debug("responseResult.getContent():{}", JSONObject.toJSONString(responseResult.getContent()));
                    //"[{\"deptId\":\"部门实例id\",\"deptName\":\"部门实例名称\",\"jobClassifyCode\":\"岗位分类编码\",\"jobClassifyName\":\"岗位分类名称\"}]";
                    JSONArray deptJobIds = responseResult.getContent().getJSONArray("deptJobIds");
                    item.setDeptJobIds(deptJobIds.toJavaList(DeptJobId.class));
                    if (ObjectUtil.isEmpty(item.getUserName())) {
                        item.setUserName(responseResult.getContent().getString("name"));
                    }
                }
            }
            String functionType = item.getFunctionType();
            Integer sourceType = item.getSourceType();
            String groupCode = item.getGroupCode();
            Assert.notBlank(functionType, "argument [{}] must not empty", OperationLog.COL_FUNCTION_TYPE);
            Assert.notNull(sourceType, "argument [{}] must not empty", OperationLog.COL_SOURCE_TYPE);
            Assert.notNull(groupCode, "argument [groupCode] must not empty");
            Map<String, String> dictDataCodeLabelMap = getLogCodeNameMap(groupCode, item.getProjectId());
            Assert.isTrue(dictDataCodeLabelMap.containsKey(functionType), "argument [{}]:{} can not find,please check!", OperationLog.COL_FUNCTION_TYPE, functionType);
            if (StringUtils.isNotBlank(item.getProjectId()) && StringUtils.isNotBlank(groupCode) && StringUtils.isNotBlank(item.getIbmsSceneCode())) {
                ZktModule zktModule = this.zktModuleService.getModule(item.getProjectId(), groupCode, item.getIbmsSceneCode());
                if (zktModule != null) {
                    item.setModuleName(zktModule.getModuleName());
                }
            }
        }
        
        List<OperationLog> operationLogs = CovertUtil.INSTANCE.convertOperationLogs(operationLogDTOs);
        boolean save = operationLogDao.saveBatch(operationLogs);
        Assert.isTrue(save, "保存日志失败");
        List<String> ids = operationLogs.stream().map(OperationLog::getLogId).collect(Collectors.toList());
        String groupCode = operationLogDTOs.get(0).getGroupCode();
        OperationLogDTO queryOperationLog = OperationLogDTO.builder().groupCode(groupCode).logIds(ids).build();
        queryOperationLog.setPage(1);
        queryOperationLog.setSize(1000);
        IPage<OperationLogDTO> operationLogDTOIPage = listOperationLog(queryOperationLog);
        Assert.isTrue(operationLogDTOIPage.getSize() > 0, "保存日志失败");
        return operationLogDTOIPage;
    }

    /**
     * @param operationLogDTO:
     * @description: 查询业务日志
     * @author: LuoGuangyi
     * @createTime: 2020/12/04 19:05
     * @return: com.baomidou.mybatisplus.core.metadata.IPage<com.persagy.dmp.dto.OperationLogDTO>
     * @expression
     */
    @Override
    public IPage<OperationLogDTO> listOperationLog(OperationLogDTO operationLogDTO) throws Exception {
        String groupCode = operationLogDTO.getGroupCode();
        Assert.notBlank(groupCode, "argument [groupCode] must not empty");
        QueryWrapper<OperationLog> queryWrapper = new QueryWrapper<>();
        Sort sort;
        if (CollectionUtils.isEmpty(operationLogDTO.getOrders())) {
            sort = new Sort(OperationLog.COL_CREATE_TIME, false);
        } else {
            sort = operationLogDTO.getOrders().get(0);
            sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
        }
        if (StringUtils.isNotBlank(operationLogDTO.getTimeType()) && !Objects.isNull(operationLogDTO.getStartTime()) && !Objects.isNull(operationLogDTO.getEndTime())) {
            queryWrapper.between(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, operationLogDTO.getTimeType()), operationLogDTO.getStartTime(), operationLogDTO.getEndTime());
        }
        Assert.notNull(operationLogDTO.getPage(), "argument [{}] must not empty", PageAndOrder.COL_PAGE);
        Assert.notNull(operationLogDTO.getSize(), "argument [{}] must not empty", PageAndOrder.COL_SIZE);
        IPage<OperationLog> memberPage = new Page<>(operationLogDTO.getPage(), operationLogDTO.getSize(), true);
        queryWrapper.eq(StringUtils.isNotBlank(operationLogDTO.getClassCode()), OperationLog.COL_CLASS_CODE, operationLogDTO.getClassCode())
                .eq(StringUtils.isNotBlank(operationLogDTO.getGroupCode()), OperationLog.COL_GROUP_CODE, operationLogDTO.getGroupCode())
                .eq(StringUtils.isNotBlank(operationLogDTO.getProjectId()), OperationLog.COL_PROJECT_ID, operationLogDTO.getProjectId())
                .eq(StringUtils.isNotBlank(operationLogDTO.getFunctionType()), OperationLog.COL_FUNCTION_TYPE, operationLogDTO.getFunctionType())
                .eq(StringUtils.isNotBlank(operationLogDTO.getOperateType()), OperationLog.COL_OPERATE_TYPE, operationLogDTO.getOperateType())
                .eq(StringUtils.isNotBlank(operationLogDTO.getLogId()), OperationLog.COL_LOG_ID, operationLogDTO.getLogId())
                .in(CollectionUtil.isNotEmpty(operationLogDTO.getLogIds()), OperationLog.COL_LOG_ID, operationLogDTO.getLogIds())
                .eq(StringUtils.isNotBlank(operationLogDTO.getObjId()), OperationLog.COL_OBJ_ID, operationLogDTO.getObjId())
                .eq(!Objects.isNull(operationLogDTO.getSourceType()), OperationLog.COL_SOURCE_TYPE, operationLogDTO.getSourceType());
        if (StringUtils.isNotBlank(operationLogDTO.getIbmsSceneCode())) {
        	queryWrapper.and(wrapper -> wrapper.eq(OperationLog.IBMS_SCENE_CODE, operationLogDTO.getIbmsSceneCode())
        			.or().eq(OperationLog.COL_SYSTEM_CODE, operationLogDTO.getIbmsSceneCode()));
		} else if (StringUtils.isNotBlank(operationLogDTO.getSystemCode())) {
			queryWrapper.and(wrapper -> wrapper.eq(OperationLog.IBMS_SCENE_CODE, operationLogDTO.getSystemCode())
        			.or().eq(OperationLog.COL_SYSTEM_CODE, operationLogDTO.getSystemCode()));
		}
        if (StringUtils.isNotBlank(operationLogDTO.getOperateDetail())) {
        	if (operationLogDTO.getSourceType() != null && 1 == operationLogDTO.getSourceType()) {
            	queryWrapper.and(wrapper -> wrapper.like(OperationLog.COL_USER_NAME, operationLogDTO.getOperateDetail())
                		.or().like(OperationLog.COL_OPERATE_DETAIL, operationLogDTO.getOperateDetail())
                	);
    		} else if (operationLogDTO.getSourceType() != null && 2 == operationLogDTO.getSourceType()) {
            	queryWrapper.and(wrapper -> wrapper.like(OperationLog.COL_OBJ_NAME, operationLogDTO.getOperateDetail())
                		.or().like(OperationLog.COL_OPERATE_DETAIL, operationLogDTO.getOperateDetail())
                	);
    		}
		}
        queryWrapper.orderBy(true, sort.getAsc(), sort.getColumn());
        
        IPage<OperationLog> operationLogIPage = operationLogDao.getBaseMapper().selectPage(memberPage, queryWrapper);
        IPage<OperationLogDTO> operationLogDTOIPage = BeanDtoVoUtils.pageVo(operationLogIPage, OperationLogDTO.class);
        Map<String, String> dictDataCodeLabelMap = getLogCodeNameMap(groupCode, operationLogDTO.getProjectId());
        log.info("dictDataCodeLabelMap : {}", JSONObject.toJSONString(dictDataCodeLabelMap));
        operationLogDTOIPage.getRecords().forEach(item -> {
            item.setFunctionName(dictDataCodeLabelMap.get(item.getFunctionType()));
        });
        return operationLogDTOIPage;
    }

    /**
     * @Description 导出日志
     * @ClassName OperationLogServiceImpl.java
     * @author LuoGuangyi
     * @version 1.0.0
     * @createTime 2021/02/24 14:41
     * @Since
     */
    @Override
    public void exportOperationLog(OperationLogDTO operationLogDTO, HttpServletResponse response) throws Exception {
        Assert.notNull(operationLogDTO.getSourceType(), "argument [sourceType] must not empty");
        IPage<OperationLogDTO> operationLog = listOperationLog(operationLogDTO);
        GeneralDictionaryDTO operationLogHeadParam = new GeneralDictionaryDTO();
        operationLogHeadParam.setDictType(GeneralDictionaryConstant.OPERATION_LOG_FIELDS);
        operationLogHeadParam.setCode(StringUtil.getString(operationLogDTO.getSourceType()));
        operationLogHeadParam.setGroupCode(operationLogDTO.getGroupCode());
        List<OperationLogDTO> records = operationLog.getRecords();
        List<GeneralDictionaryDTO> dictionaries = generalDictionaryService.listGeneralDictionary(operationLogHeadParam);
        Assert.isTrue(ObjectUtil.isNotEmpty(dictionaries), "请配置日志表头");
        JSONObject generalFields = dictionaries.get(0).getItemData();
        JSONArray headers = dictionaries.get(0).getArrayData();
        String name = dictionaries.get(0).getName();
        LocalDateTime now = LocalDateTime.now();
        String xlsFileName = StrUtil.format("{}-{}.xlsx", name, DateUtils.format(now, "yyyy-MM-dd HHmmss"));
        ExcelWriter writer = ExcelUtil.getBigWriter();
        // 设置所有列为自动宽度，不考虑合并单元格
        SXSSFSheet sheet = (SXSSFSheet) writer.getSheet();
        sheet.trackAllColumnsForAutoSizing();
        List<String> rowHead = new ArrayList<>();
        for (int i = 0; i < headers.size(); i++) {
            String headerCode = headers.getString(i);
            String headerName = StringUtil.getString(generalFields.getString(headerCode));
            writer.addHeaderAlias(headerCode, headerName);
            rowHead.add(headerName);
        }
        //isWriteKeyAsHead为true的时候自动添加表头，与writeHeadRow不能同时存在，否则为双表头
        writer.writeHeadRow(rowHead);
        //表头冻结
        writer.setFreezePane(1);
        writer.setOnlyAlias(true);
        //setOnlyAlias必须在Write之前添加，否则总会在第一列多产生一个表头，也就是是说事后诸葛亮是不生效的
        writer.write(records, false);
        // 此方法必须在指定列数据完全写出后调用才有效
        //   writer.autoSizeColumnAll();  write方法中列数量变成了-1，导致autoSizeColumnAll不生效
        for (int i = 0; i < headers.size(); i++) {
            // 调整每一列宽度
            sheet.autoSizeColumn((short) i);
            // 解决自动设置列宽中文失效的问题
            int columnWidth = (sheet.getColumnWidth(i) * 16 / 10) > 25000 ? 25000 : (sheet.getColumnWidth(i) * 16 / 10);
            sheet.setColumnWidth(i, columnWidth);
        }
        xlsFileName = URLEncoder.encode(xlsFileName, "UTF-8");
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8");
        response.setHeader("Content-Disposition", "attachment;filename=" + xlsFileName);
        ServletOutputStream out = response.getOutputStream();
        writer.flush(out, true);
        writer.close();
        IoUtil.close(out);
    }

    private Map<String, String> getCodeNameMap(String dictType, String groupCode, String projectId) throws Exception {
        //FunctionName 赋值
        GeneralDictionaryDTO logParam = new GeneralDictionaryDTO();
        logParam.setDictType(dictType);
        logParam.setGroupCode(groupCode);
        logParam.setProjectId(projectId);
        HashMap<String, String> codeNameMap = generalDictionaryService.queryGDCodeNameMap(logParam);
        return codeNameMap;
    }

    private Map<String, String> getLogCodeNameMap(String groupCode, String projectId) throws Exception {
        return getCodeNameMap(FUNCTION_LOG, groupCode, projectId);
    }

}