package com.persagy.dmp.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.persagy.dmp.dao.InstanceDao;
import com.persagy.dmp.dto.ObjRelationDTO;
import com.persagy.dmp.dto.ObjectInfoDTO;
import com.persagy.dmp.dto.relation.BatchRelationCriteria;
import com.persagy.dmp.dto.relation.CreateRelation;
import com.persagy.dmp.dto.relation.RelationCriteria;
import com.persagy.dmp.service.ObjectService;
import com.persagy.dmp.service.RelationService;
import lombok.NonNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @description: 关系接口实现类
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/9/22 15:12
 * @version: V1.0
 **/
@Service
public class RelationServiceImpl implements RelationService {

    @Autowired
    private InstanceDao instanceDao;

    @Autowired
    private ObjectService objectService;

    @Override
    public List<String> queryRelationByFromId(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull String fromId) throws Exception {
        //查询关系
        List<ObjRelationDTO> objRelationDTOList = this.queryRelationByFromIds(groupCode, projectId, graphCode, relCode, Arrays.asList(fromId));
        if (CollectionUtils.isEmpty(objRelationDTOList)) {
            return new ArrayList<>();
        }
        return objRelationDTOList.stream().map(ObjRelationDTO::getObjTo).collect(Collectors.toList());
    }

    @Override
    public List<String> queryRelationByFromId(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull String fromId, @NonNull String toClassCode) throws Exception {
        List<String> toIds = this.queryRelationByFromId(groupCode, projectId, graphCode, relCode, fromId);
        if (CollectionUtils.isEmpty(toIds)) {
            return new ArrayList<>();
        }
        return this.filterObjectIds(groupCode, projectId, toIds, toClassCode);
    }

    @Override
    public List<ObjRelationDTO> queryRelationByFromIds(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull List<String> fromIds) throws Exception {
        //查询关系
        if (CollectionUtils.isEmpty(fromIds)) {
            return new ArrayList<>();
        }
        return instanceDao.queryRelationByFromIds(groupCode, projectId, graphCode, relCode, fromIds);
    }

    @Override
    public List<ObjRelationDTO> queryRelationByFromIds(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull List<String> fromIds, @NonNull String toClassCode) throws Exception {
        //查询关系
        List<ObjRelationDTO> objRelationDTOList = this.queryRelationByFromIds(groupCode, projectId, graphCode, relCode, fromIds);
        if (CollectionUtils.isEmpty(objRelationDTOList)) {
            return new ArrayList<>();
        }
        List<String> toIds = objRelationDTOList.stream().map(ObjRelationDTO::getObjTo).collect(Collectors.toList());
        toIds = this.filterObjectIds(groupCode, projectId, toIds, toClassCode);
        List<String> finalToIds = toIds;
        return objRelationDTOList.stream().filter(o -> finalToIds.stream().anyMatch(id -> id.equals(o.getObjTo()))).collect(Collectors.toList());
    }

    @Override
    public List<String> queryRelationByToId(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull String toId) throws Exception {
        //查询关系
        List<ObjRelationDTO> objRelationDTOList = this.queryRelationByToIds(groupCode, projectId, graphCode, relCode, Arrays.asList(toId));
        if (CollectionUtils.isEmpty(objRelationDTOList)) {
            return new ArrayList<>();
        }
        return objRelationDTOList.stream().map(ObjRelationDTO::getObjFrom).collect(Collectors.toList());
    }

    @Override
    public List<String> queryRelationByToId(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull String toId, @NonNull String fromClassCode) throws Exception {
        //查询关系
        List<String> fromIds = this.queryRelationByToId(groupCode, projectId, graphCode, relCode, toId);
        if (CollectionUtils.isEmpty(fromIds)) {
            return new ArrayList<>();
        }
        return this.filterObjectIds(groupCode, projectId, fromIds, fromClassCode);
    }

    @Override
    public List<ObjRelationDTO> queryRelationByToIds(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull List<String> toIds) throws Exception {
        //查询关系
        if (CollectionUtils.isEmpty(toIds)) {
            return new ArrayList<>();
        }
        return instanceDao.queryRelationByToIds(groupCode, projectId, graphCode, relCode, toIds);
    }

    @Override
    public List<ObjRelationDTO> queryRelationByToIds(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, @NonNull List<String> toIds, @NonNull String fromClassCode) throws Exception {
        //查询关系
        List<ObjRelationDTO> objRelationDTOList = this.queryRelationByToIds(groupCode, projectId, graphCode, relCode, toIds);
        if (CollectionUtils.isEmpty(objRelationDTOList)) {
            return new ArrayList<>();
        }
        List<String> fromIds = objRelationDTOList.stream().map(ObjRelationDTO::getObjFrom).collect(Collectors.toList());
        fromIds = this.filterObjectIds(groupCode, projectId, fromIds, fromClassCode);
        List<String> finalToIds = fromIds;
        return objRelationDTOList.stream().filter(o -> finalToIds.stream().anyMatch(id -> id.equals(o.getObjFrom()))).collect(Collectors.toList());
    }

    @Override
    public List<ObjRelationDTO> queryRelation(@NonNull String groupCode, @NonNull String projectId, @NonNull String graphCode, @NonNull String relCode, List<String> fromIds, List<String> toIds) throws Exception {
        //查询关系
        if (ObjectUtil.isAllEmpty(fromIds, toIds)) {
            return new ArrayList<>();
        }
        return instanceDao.queryRelation(groupCode, projectId, graphCode, relCode, fromIds, toIds);
    }

    @Override
    public Boolean createRelation(@NonNull String groupCode, @NonNull String projectId, @NonNull CreateRelation createRelation) {
        return instanceDao.createRelation(groupCode, projectId, createRelation);
    }

    @Override
    public Boolean createRelation(@NonNull String groupCode, @NonNull String projectId, @NonNull List<CreateRelation> createRelationList) {
        return instanceDao.createRelation(groupCode, projectId, createRelationList);
    }

    @Override
    public Boolean deleteRelation(@NonNull String groupCode, @NonNull String projectId, @NonNull RelationCriteria criteria) {
        return instanceDao.deleteRelation(groupCode, projectId, criteria);
    }

    @Override
    public Boolean batchDeleteRelation(@NonNull String groupCode, @NonNull String projectId, @NonNull BatchRelationCriteria criteria) {
        return instanceDao.batchDeleteRelation(groupCode, projectId, criteria);
    }

    private List<String> filterObjectIds(@NonNull String groupCode, @NonNull String projectId, @NonNull List<String> objIds, @NonNull String classCode) throws Exception {
        JSONArray objArr = objectService.queryObjectList(groupCode, projectId, objIds);
        if (CollectionUtils.isEmpty(objArr)) {
            return new ArrayList<>();
        }
        List<ObjectInfoDTO> objectInfoDTOList = objArr.toJavaList(ObjectInfoDTO.class);
        if (!CollectionUtils.isEmpty(objectInfoDTOList)) {
            return objIds.stream().filter(id ->
                    objectInfoDTOList.stream().anyMatch(o ->
                            id.equals(o.getId()) && classCode.equals(o.getClassCode())))
                    .collect(Collectors.toList());
        }
        return new ArrayList<>();
    }
}
