package com.persagy.dmp.typeHandler;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.dmp.constant.RequestUrlConstant;
import com.persagy.dmp.constant.UserConstant;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.entity.ContentType;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * @author zhangqiankun
 * @version 1.0.0
 * @company persagy
 * @date 2021年12月15日 上午10:53:06
 */
@Slf4j
@Component
public class SaasOrgPersonJarHandler {

    @Value("${saas.org.person.http.prefix:}")
    private String saasOrgPerson;


    /**
     * 获取指定用户的角色人员信息
     *
     * @param groupCode
     * @param projectId
     * @param username  其与personId 不同时生效
     * @param personId
     * @return
     * @date 2021年12月15日 下午6:01:26
     */
    public JSONObject getPersonRole(String groupCode, String projectId, String username, String personId) {
        // 获取FM人员列表
        JSONObject body = new JSONObject();
        body.put("userId", UserConstant.USER_ID);
        body.put("projectId", projectId);
        if (StrUtil.isNotBlank(username)) {
            body.put("keyword", username);
        } else if (StrUtil.isNotBlank(personId)) {
            body.put("keyword", personId);
        }
        body.put("page", 1);
        body.put("pageSize", 100);
        String queryUrl = saasOrgPerson + RequestUrlConstant.QUERY_FM_SAAS_PERSON;
//        String queryUrl = "http://183.230.202.111:30000/saas-org-person" + RequestUrlConstant.QUERY_FM_SAAS_PERSON;
        String result = postHttp(queryUrl, groupCode, body.toJSONString());
        if (StrUtil.isBlank(result)) {
            log.error(UserConstant.SERVER_EXCEPTION + "FM查询人员，响应为空");
        }
        log.info("FM响应数据：{}", result);
        JSONObject resultObj = JSONObject.parseObject(result);
        String resultCode = resultObj.getString("Result");
        if (!UserConstant.SUCCESS.equals(resultCode)) {
            log.error(UserConstant.SERVER_EXCEPTION + resultObj.getString("ResultMsg"));
        }

        JSONArray jsonArray = resultObj.getJSONArray("Content");
        if (jsonArray == null || jsonArray.size() == 0) {
            return null;
        }
        return jsonArray.getJSONObject(0);
    }

    /**
     * 发送post json请求
     *
     * @param url
     * @param groupCode
     * @param body
     * @return
     * @date 2021年12月15日 上午10:41:35
     */
    private static String postHttp(String url, String groupCode, String body) {
        Map<String, String> headers = new HashMap<String, String>();
        headers.put("groupCode", groupCode);
        return HttpUtil.createPost(url).addHeaders(headers).body(body, ContentType.APPLICATION_JSON.toString())
                .timeout(10 * 1000).execute().body();
    }

}
