package com.persagy.dmp.utils;

import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.dmp.client.AlarmNewServerClient;
import com.persagy.dmp.common.url.InstanceUrlParam;
import com.persagy.dmp.dto.alarm.ResultQueryVo;
import org.springframework.util.CollectionUtils;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 报警查询的公共util
 * @author 易涛
 * @version 1.0
 * @date 2021/6/23 17:12
 */
public class AlarmQueryUtil {

    private static final AlarmNewServerClient alarmNewServerClient = SpringUtil.getBean(AlarmNewServerClient.class);

    /**
     * 根据1级二级菜单code查询对象
     * @param projectId 项目id
     * @param groupCode 集团编号
     * @param ibmsSceneCode 1级菜单code
     * @param ibmsClassCode 2级菜单code
     * @author 易涛
     * @date 2021/7/9 10:28
     * @return com.alibaba.fastjson.JSONArray
     */
    public static JSONArray queryObjectList(String projectId,String groupCode,String ibmsSceneCode,
                                            String ibmsClassCode){
        InstanceUrlParam instanceUrlParam = InstanceUrlParam
                .builder()
                .groupCode(groupCode)
                .projectId(projectId)
                .build();
        //封装post请求体
        JSONObject jsonCriteria = new JSONObject();
        JSONObject jsonObjectItem = new JSONObject();
        jsonObjectItem.put("ibmsSceneCode",ibmsSceneCode);
        jsonObjectItem.put("ibmsClassCode",ibmsClassCode);
        jsonCriteria.put("criteria",jsonObjectItem);

        JSONObject jsonObject = alarmNewServerClient.queryObject(instanceUrlParam,jsonCriteria);
        ResultQueryVo resultQueryVo = JSONObject.toJavaObject(jsonObject,ResultQueryVo.class);
        if(resultQueryVo==null){
            return null;
        }
        return resultQueryVo.getData();
    }

    public static JSONArray queryRelationObjectList(String projectId, String groupCode, String graphCode, String relCode,
                                                    List<String> objFrom, List<String> objTo, String ibmsSceneCode, String ibmsClassCode){
        InstanceUrlParam instanceUrlParam = InstanceUrlParam
                .builder()
                .groupCode(groupCode)
                .projectId(projectId)
                .build();
        //封装post请求体
        JSONObject jsonCriteria = new JSONObject();

        JSONObject jsonObjectItem = new JSONObject();
        jsonObjectItem.put("graphCode",graphCode);
        jsonObjectItem.put("relCode",relCode);
        if(!CollectionUtils.isEmpty(objTo)){
            jsonObjectItem.put("objTo",objTo);
        }
        if(!CollectionUtils.isEmpty(objFrom)){
            jsonObjectItem.put("objFrom",objFrom);
        }

        JSONObject objectCriteria = new JSONObject();
        objectCriteria.put("ibmsSceneCode",ibmsSceneCode);
        objectCriteria.put("ibmsClassCode",ibmsClassCode);
        objectCriteria.put("objectCriteria",objectCriteria);
        jsonCriteria.put("criteria",jsonObjectItem);

        JSONObject jsonObject = alarmNewServerClient.queryRelation(instanceUrlParam,jsonCriteria);
        ResultQueryVo resultQueryVo = JSONObject.toJavaObject(jsonObject,ResultQueryVo.class);
        if(resultQueryVo==null){
            return null;
        }
        return resultQueryVo.getData();
    }

    /**
     * 根据1级菜单编号查询报警对象类
     * @param ibmsSceneCode 1级菜单code
     * @author 易涛
     * @date 2021/7/9 10:50 
     * @return com.alibaba.fastjson.JSONArray
     */
    public static JSONArray queryClassList(@NotNull String projectId,@NotNull String groupCode,@NotNull String ibmsSceneCode){
        InstanceUrlParam instanceUrlParam = InstanceUrlParam
                .builder()
                .groupCode(groupCode)
                .projectId(projectId)
                .build();
        //封装post请求体
        JSONObject jsonCriteria = new JSONObject();
        JSONObject jsonObjectItem = new JSONObject();
        jsonObjectItem.put("ibmsSceneCode",ibmsSceneCode);
        jsonCriteria.put("criteria",jsonObjectItem);
        JSONObject jsonObject = alarmNewServerClient.queryObjectClass(instanceUrlParam,jsonCriteria);
        ResultQueryVo resultQueryVo = JSONObject.toJavaObject(jsonObject,ResultQueryVo.class);
        if(resultQueryVo==null){
            return null;
        }
        return resultQueryVo.getData();
    }
}