package com.persagy.dmp.utils;

import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @description: Dto，Vo，entity 转换工具类
 * @author:LuoGuanhyi
 * @company:PersagyTechnologyCo.,Ltd
 * @since:2020/10/10 001021:18
 * @version:V1.0
 **/
public class BeanDtoVoConvert {
    /**
     * TODO  dot ,Do ,entity 相互转换
     * 同：BeanUtils.copyProperties(dtoEntity, newInstance);
     *
     * @param oldClass 原数据--Dto，Vo，entity
     * @param newClass 转换为--Dto，Vo，entity
     * @return
     */
    public static <E> E convert(Object oldClass, Class<E> newClass) {
        // 判断oldClass 是否为空!
        if (oldClass == null) {
            return null;
        }
        // 判断newClass 是否为空
        if (newClass == null) {
            return null;
        }
        try {
            // 创建新的对象实例
            E newInstance = newClass.newInstance();
            // 把原对象数据拷贝到新的对象
            BeanUtils.copyProperties(oldClass, newInstance);
            // 返回新对象
            return newInstance;
        } catch (Exception e) {
            return null;
        }
    }


    //TODO  list<Entity> 集合对象转list<Vo> ( list 循环)
    public static <T, V> List<V> listVo(List<T> oldList, Class<V> v) {
        List<V> voList = new ArrayList<>();
        for (T t : oldList) {
            voList.add(BeanDtoVoConvert.convert(t, v));
        }
        return voList;
    }


    //TODO  list<Entity> 集合对象转list<Vo> （Stream 方式）
    public static <T, V> List<V> listVoStream(List<T> oldList, Class<V> v) {
        return oldList.stream().map(item ->
                BeanDtoVoConvert.convert(item, v)
        ).collect(Collectors.toList());
    }

    //TODO  Page<Entity> 分页对象转 Page<Vo>  ( list 循环)
    public static <T, V> Page<V> pageVo(Page<T> page, Class<V> v) {
        List<T> tList = page.getContent();
        List<V> voList = new ArrayList<>();
        for (T t : tList) {
            voList.add(BeanDtoVoConvert.convert(t, v));
        }
        return new PageImpl<>(voList, page.getPageable(), page.getTotalElements());
    }


    //TODO  Page<Entity> 分页对象转 Page<Vo> （Stream 方式）
    public static <T, V> Page<V> pageVoStream(Page<T> page, Class<V> v) {
        List<V> voList = page.getContent()
                .stream()
                .map(item -> BeanDtoVoConvert.convert(item, v))
                .collect(Collectors.toList());

        return new PageImpl<>(voList, page.getPageable(), page.getTotalElements());
    }

    //TODO  Page<Entity> 分页对象转 Page<Vo>  ( list 循环)
    public static <T, V> IPage<V> pageVo(IPage<T> page, Class<V> v) {
        return page.convert(item -> BeanDtoVoConvert.convert(item, v));
    }

}
