package com.persagy.dmp.utils;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalUnit;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;

/**
 * 时间相关的工具类
 *
 * @author feng
 */
public class DateUtils {
    public static final String SECOND = "0";
    public static final String MINUTE = "1";
    public static final String HOUR = "2";
    public static final String DAY = "3";
    public static final String MONTH = "4";
    public static final String YEAR = "5";
    public static final String SDF_SECOND = "yyyy-MM-dd HH:mm:ss";
    public static final String SDF_MINUTE = "yyyy-MM-dd HH:mm";
    public static final String SDF_HOUR = "yyyy-MM-dd HH";
    public static final String SDF_DAY = "yyyy-MM-dd";
    public static final String SDFDAY = "yyyyMMdd";
    public static final String SDF_DOT_DAY = "yyyy.MM.dd";
    public static final String SDFDAY2 = "yyyy.MM.dd";
    public static final String SDFSECOND = "yyyyMMddHHmmss";
    public static final String SDF_MONTH = "yyyy-MM-01";
    public static final String SDF_DOT_MONTH = "yyyy.MM";
    public static final String SDFMONTH2 = "yyyy.MM";
    public static final String SDF_YEAR = "yyyy-01-01";
    public static final String SDF_ONLY_HOUR_MINUTE = "HH:mm";
    public static final String sdfDay = "yyyyMMdd";
    public static final String sdfMonth = "yyyyMM";
    public static final String sdfTime = "yyyyMMddHHmmss";
    public static final String sdfTimeNotDate = "HHmmss";
    public static final String sdfHour = "yyyyMMddHH";
    public static final String sdfMinute = "yyyyMMddHHmm";
    // 时间格式-显示
    public final static String date_format_show = "yyyy-MM-dd HH:mm:ss";
    public final static String date_format_show_minute = "yyyy-MM-dd HH:mm";
    private final static DateTimeFormatter FORMATTER = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");
    /**
     * 东八区时区偏移量
     */
    public final static int ASIA_SHANGHAI = 8;

    /**
     * 获取当前的UTC时间,精确到毫秒
     */
    public static Long getUtcTimeNow() {
        Date dateNow = new Date();
        Long lRes = dateNow.getTime();
        return lRes;
    }

    public static String formatStr(String pattern, Date date) {
        String str = new SimpleDateFormat(pattern).format(date);
        return str;
    }

    public static Date parseDate(String pattern, String str) throws ParseException {
        Date date = new SimpleDateFormat(pattern).parse(str);
        return date;
    }

    /**
     * 获取指定年月的第一天
     *
     * @param year
     * @param month
     * @return
     */
    public static String getFirstDayOfMonth(int year, int month) {
        Calendar cal = Calendar.getInstance();
        //设置年份
        cal.set(Calendar.YEAR, year);
        //设置月份
        cal.set(Calendar.MONTH, month - 1);
        //获取某月最小天数
        int firstDay = cal.getMinimum(Calendar.DATE);
        //设置日历中月份的最小天数
        cal.set(Calendar.DAY_OF_MONTH, firstDay);
        //格式化日期
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        return sdf.format(cal.getTime());
    }

    /**
     * 获取指定年月的最后一天
     *
     * @param year
     * @param month
     * @return
     */
    public static String getLastDayOfMonth(int year, int month) {
        Calendar cal = Calendar.getInstance();
        //设置年份
        cal.set(Calendar.YEAR, year);
        //设置月份
        cal.set(Calendar.MONTH, month - 1);
        //获取某月最大天数
        int lastDay = cal.getActualMaximum(Calendar.DATE);
        //设置日历中月份的最大天数
        cal.set(Calendar.DAY_OF_MONTH, lastDay);
        //格式化日期
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        return sdf.format(cal.getTime());
    }

    /**
     * 将pattern格式的字符串日期转换为Date类型日期
     *
     * @param source  pattern格式的字符串日期
     * @param pattern 日期格式如yyyy-MM-dd HH:mm:ss
     * @return
     * @throws Exception
     */
    public static Date str2Date(String source, String pattern) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        return sdf.parse(source);
    }

    /**
     * Description:返回两个日期间相差多少天
     *
     * @param start
     * @param end
     * @return Integer
     * @author cuixubin
     * @since 2018年9月18日: 下午12:12:38
     * Update By cuixubin 2018年9月18日: 下午12:12:38
     */
    public static Integer getDayNum(Date start, Date end) {
        if (null == start || end == null) {
            return null;
        }

        if (start.getTime() == end.getTime()) {
            return 0;
        }

        int num = 0;
        Date temp = start.before(end) ? start : end;
        Date endDate = start.before(end) ? end : start;
        while (temp.before(endDate)) {
            num++;
            temp = getDayOff(temp, 1);
        }
        return num;
    }

    /**
     * 将Date类型的日期转换成pattern格式的字符串日期
     *
     * @param date
     * @param pattern
     * @return
     * @throws Exception
     */
    public static String date2Str(Date date, String pattern) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        return sdf.format(date);
    }

    /**
     * 获取时间按年偏移offset个单位后的时间
     *
     * @param date
     * @param offset
     * @return
     */
    public static Date getYearOff(Date date, int offset) {
        return getOffset(date, offset, 5);
    }

    /**
     * 获取时间按月偏移offset个单位后的时间
     *
     * @param date
     * @param offset
     * @return
     */
    public static Date getMonthOff(Date date, int offset) {
        return getOffset(date, offset, 4);
    }

    /**
     * 获取时间按日偏移offset个单位后的时间
     *
     * @param date
     * @param offset
     * @return
     */
    public static Date getDayOff(Date date, int offset) {
        return getOffset(date, offset, 2);
    }

    /**
     * 获取时间按周偏移offset个单位后的时间
     *
     * @param date
     * @param offset
     * @return
     */
    public static Date getWeekOff(Date date, int offset) {
        return getOffset(date, offset, 3);
    }

    /**
     * 获取时间按小时偏移offset个单位后的时间
     *
     * @param date
     * @param offset
     * @return
     */
    public static Date getHourOff(Date date, int offset) {
        return getOffset(date, offset, 1);
    }

    /**
     * description 获取时间按分钟偏移offset个单位后的时间
     *
     * @param date
     * @param offset
     * @return java.util.Date
     * @author feng
     * @since 18:51 2018/12/10
     */
    public static Date getMinuteOff(Date date, int offset) {
        return getOffset(date, offset, 0);
    }

    /**
     * 获取指定时间粒度偏移量偏移后的时间
     *
     * @param date
     * @param offset int 偏移量
     * @param type   int 时间粒度0-分，1-时，2-天，3-周，4-月，5-年
     * @return
     */
    private static Date getOffset(Date date, int offset, int type) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        switch (type) {
            case 0:
                cal.add(Calendar.MINUTE, offset);
                break;
            case 1:
                cal.add(Calendar.HOUR_OF_DAY, offset);
                break;
            case 2:
                cal.add(Calendar.DAY_OF_MONTH, offset);
                break;
            case 3:
                cal.add(Calendar.DAY_OF_MONTH, offset * 7);
                break;
            case 4:
                cal.add(Calendar.MONTH, offset);
                break;
            case 5:
                cal.add(Calendar.YEAR, offset);
                break;
            default:
                break;
        }
        return cal.getTime();
    }
	
    /**
     * 得到某月最后一天的结束时间 即2017-01-31 23:59:59.999
     *
     * @param date 时间参数
     * @return
     */
    public static Date getLastDayOfMonth(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int year = c.get(Calendar.YEAR);
        int month = c.get(Calendar.MONTH);
        c.set(Calendar.HOUR_OF_DAY, 23);
        c.set(Calendar.DAY_OF_MONTH, 1);
        c.set(Calendar.MINUTE, 59);
        c.set(Calendar.SECOND, 59);
        c.set(Calendar.MILLISECOND, 999);
        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, month);
        // 月加1，日减1 得到当前月最后1天
        c.add(Calendar.MONTH, 1);
        c.add(Calendar.DAY_OF_YEAR, -1);
        return c.getTime();
    }

    public static Date getEndTimeOfDay(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int year = c.get(Calendar.YEAR);
        int month = c.get(Calendar.MONTH);
        int day = c.get(Calendar.DAY_OF_MONTH);
        c.set(Calendar.HOUR_OF_DAY, 23);
        c.set(Calendar.MINUTE, 59);
        c.set(Calendar.SECOND, 59);
        c.set(Calendar.MILLISECOND, 999);
        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, month);
        c.set(Calendar.DAY_OF_MONTH, day);
        return c.getTime();
    }

    public static Date getLastDayStartTimeOfMonth(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int year = c.get(Calendar.YEAR);
        int month = c.get(Calendar.MONTH);
        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.DAY_OF_MONTH, 1);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        c.set(Calendar.MILLISECOND, 0);
        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, month);
        // 月加1，日减1 得到当前月最后1天
        c.add(Calendar.MONTH, 1);
        c.add(Calendar.DAY_OF_YEAR, -1);
        return c.getTime();
    }

    /**
     * 得到某年最后一天的结束时间 即2017-12-31 23:59:59.999
     *
     * @param date 时间参数
     * @return
     */
    public static Date getLastDayStartTimeOfYear(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int year = c.get(Calendar.YEAR);
        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.DAY_OF_MONTH, 31);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        c.set(Calendar.MILLISECOND, 0);
        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, 11);
        return c.getTime();
    }

    /**
     * 获取某一时间所在月某月第一天的开始时间 即2017-01-01 00:00:00.000
     *
     * @param date 时间参数
     * @return
     */
    public static Date getFirstDayOfMonth(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int year = c.get(Calendar.YEAR);
        int month = c.get(Calendar.MONTH);
        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.DAY_OF_MONTH, 1);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        c.set(Calendar.MILLISECOND, 0);
        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, month);
        return c.getTime();
    }

    public static Date getFirstDayOfYear(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int year = c.get(Calendar.YEAR);
        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.DAY_OF_MONTH, 1);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        c.set(Calendar.MILLISECOND, 0);
        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, 0);
        return c.getTime();
    }

    /**
     * 获取某一天的开始时间 即2017-01-01 00:00:00
     *
     * @param date 时间参数
     * @return
     * @throws ParseException
     */
    public static Date getStartTimeOfDay(Date date) throws ParseException {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int year = c.get(Calendar.YEAR);
        int month = c.get(Calendar.MONTH);
        int day = c.get(Calendar.DAY_OF_MONTH);
        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        c.set(Calendar.MILLISECOND, 0);
        c.set(Calendar.DAY_OF_MONTH, day);
        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, month);
        return c.getTime();
    }

    /**
     * 获取某一天的开始时间 即2017-01-01 00:00:00
     *
     * @param date 时间参数
     * @return
     * @throws ParseException
     */
    public static String getStartTimeOfDayStr(Date date) throws ParseException {
        if (date == null) {
            return null;
        }
        SimpleDateFormat sdfDay = new SimpleDateFormat(SDF_DAY);
        return sdfDay.format(getStartTimeOfDay(date));
    }

    /**
     * endDate比beginDate多的天数
     *
     * @param beginDate
     * @param endDate
     * @return
     */
    public static int differentDays(Date beginDate, Date endDate) {
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(beginDate);
        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(endDate);
        int day1 = cal1.get(Calendar.DAY_OF_YEAR);
        int day2 = cal2.get(Calendar.DAY_OF_YEAR);
        int year1 = cal1.get(Calendar.YEAR);
        int year2 = cal2.get(Calendar.YEAR);
        if (year1 != year2) {
            int timeDistance = 0;
            for (int i = year1; i < year2; i++) {
                // 闰年
                if (i % 4 == 0 && i % 100 != 0 || i % 400 == 0) {
                    timeDistance += 366;
                } else {
                    timeDistance += 365;
                }
            }
            return timeDistance + (day2 - day1);
        } else {
            return day2 - day1;
        }
    }

    /**
     * 获取周几(周一为1,周二为2,周日为7)
     *
     * @param date
     * @return
     */
    public static int getDayOfWeek(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int day = calendar.get(Calendar.DAY_OF_WEEK);
        if (day > 1) {
            day = day - 1;
        } else {
            day = 7;
        }
        return day;
    }

    /**
     * 获取月份
     *
     * @param date
     * @return
     */
    public static int getMonthOfYear(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int month = calendar.get(Calendar.MONTH);
        return month + 1;
    }

    /**
     * 获取年份
     *
     * @param date
     * @return
     */
    public static int getYear(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int year = calendar.get(Calendar.YEAR);
        return year;
    }

    /**
     * 返回小时 0~23
     *
     * @param date
     * @return
     */
    public static int getHour(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int hour = calendar.get(Calendar.HOUR_OF_DAY);
        return hour;
    }

    /**
     * Description: 返回0-59
     *
     * @param date
     * @return int
     * @author cuixubin
     * @since 2018年12月18日: 下午6:29:43
     */
    public static int getMinute(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int minute = calendar.get(Calendar.MINUTE);
        return minute;
    }

    /**
     * 获取某一月的总天数
     *
     * @param date
     * @return
     */
    public static int getTotalDaysOfMonth(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        return calendar.getActualMaximum(Calendar.DAY_OF_MONTH);
    }

    public static int getTotalDaysOfYear(Date date) {
        return ifRunYear(date) ? 366 : 365;
    }

    /**
     * 获得本周星期一的日期
     *
     * @param time
     * @return
     */
    public static Date getCurrentMonday(Date time) {
        int mondayPlus = 0;
        Calendar cd = Calendar.getInstance();
        cd.setTime(time);
        // 获得今天是一周的第几天，星期日是第一天，星期二是第二天......
        int dayOfWeek = cd.get(Calendar.DAY_OF_WEEK);
        if (dayOfWeek == 1) {
            mondayPlus = -6;
        } else {
            mondayPlus = 2 - dayOfWeek;
        }
        GregorianCalendar currentDate = new GregorianCalendar();
        currentDate.add(GregorianCalendar.DATE, mondayPlus);
        Date monday = currentDate.getTime();
        return monday;
    }

    /**
     * 获取日期所在周的周一
     *
     * @param date
     * @return
     */
    public static Date getMonday(Date date) {
        int daynum = getDayOfWeek(date);
        try {
            date = getShortDate(date, SDF_DAY);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return getDayOff(date, 1 - daynum);
    }

    /**
     * 获取日期所在周周六0点
     *
     * @param date
     * @return
     */
    public static Date getWeekDay(Date date) {
        int daynum = getDayOfWeek(date);
        try {
            date = getShortDate(date, SDF_DAY);
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        if (daynum == 6) {
            try {
                return date;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        if (daynum == 7) {
            return getDayOff(date, -1);
        } else {
            return getDayOff(date, 6 - daynum);
        }
    }

    /**
     * 将日期精度更改到年, 月，天或秒
     *
     * @param date
     * @param pattern DateUtils.SDF_SECOND、DateUtils.SDF_DAY、DateUtils.SDF_HOUR...
     * @return
     * @throws Exception
     */
    public static Date getShortDate(Date date, String pattern) throws Exception {
        String dateStr = null;
        Date dateR = null;
        switch (pattern) {
            case SDF_SECOND:
                dateStr = date2Str(date, SDF_SECOND);
                dateR = str2Date(dateStr, SDF_SECOND);
                break;
            case SDF_DAY:
                dateStr = date2Str(date, SDF_DAY);
                dateR = str2Date(dateStr, SDF_DAY);
                break;
            case SDF_MONTH:
                dateStr = date2Str(date, SDF_MONTH);
                dateR = str2Date(dateStr, SDF_MONTH);
                break;
            case SDF_HOUR:
                dateStr = date2Str(date, SDF_HOUR);
                dateR = str2Date(dateStr, SDF_HOUR);
                break;
            case SDF_YEAR:
                dateStr = date2Str(date, SDF_YEAR);
                dateR = str2Date(dateStr, SDF_YEAR);
                break;
            default:
                break;
        }

        return dateR;
    }

    /**
     * 判断两个日期是否为同一年/月/日
     *
     * @param date1
     * @param date2
     * @param type  3-日；4-月；5-年
     * @return
     */
    public static boolean ifSameDate(Date date1, Date date2, String type) {
        boolean result = false;
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(date1);
        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(date2);

        int year1 = cal1.get(Calendar.YEAR);
        int year2 = cal2.get(Calendar.YEAR);
        int month1 = cal1.get(Calendar.MONTH);
        int month2 = cal2.get(Calendar.MONTH);
        int day1 = cal1.get(Calendar.DAY_OF_MONTH);
        int day2 = cal2.get(Calendar.DAY_OF_MONTH);

        switch (type) {
            case YEAR:
                result = year1 == year2;
                break;
            case MONTH:
                result = (year1 == year2) && (month1 == month2);
                break;
            case DAY:
                result = (year1 == year2) && (month1 == month2) && (day1 == day2);
                break;
            default:
                break;
        }

        return result;
    }

    /**
     * 返回同一年中两个时间点相差的天数，两个时间点若不是同一年则返回0
     *
     * @param fromDate
     * @param toDate
     * @return
     */
    public static int dayNumber(Date fromDate, Date toDate) {
        int num = 0;
        if (fromDate.after(toDate)) {
            Date temp = fromDate;
            fromDate = toDate;
            toDate = temp;
        }

        Calendar cal = Calendar.getInstance();
        cal.setTime(fromDate);
        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(toDate);


        if (cal.get(Calendar.YEAR) == cal2.get(Calendar.YEAR)) {
            num = cal2.get(Calendar.DAY_OF_YEAR) - cal.get(Calendar.DAY_OF_YEAR);
        }

        return num;
    }

    /**
     * 获取某月的第几天 如month="2017-11-23 00:00:00",返回23
     *
     * @param month
     * @return
     */
    public static int getDayOfMonth(Date month) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(month);
        return cal.get(Calendar.DAY_OF_MONTH);
    }

    /**
     * 判断某一时间所在年是否为润年
     *
     * @param date
     * @return
     */
    public static boolean ifRunYear(Date date) {
        int year = getYear(date);
        return year % 4 == 0 && year % 100 != 0 || year % 400 == 0;
    }

    /**
     * 获取日期所在周为本年第几周
     *
     * @param date
     * @return
     */
    public static int getWeekNum(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        return cal.get(Calendar.WEEK_OF_YEAR);
    }

    public static Date getWeekStart(Date date, int weekNumOfYear) {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.WEEK_OF_YEAR, weekNumOfYear);

        return cal.getTime();
    }

    /**
     * 获取某时间所在周（非）工作日的开始结束时间
     *
     * @param date
     * @param offSet 0-date所在周，1-下一周，-1-上一周，n-下n周
     * @return {"startTime":null,"endTime":null,"daynum":2} (非)工作日的开始时间、结束时间、天数
     */
    public static Map<String, Object> getWorkTime(Date date, int offSet) {
        Map<String, Object> returnMap = new HashMap<>();

        int weekDay = getDayOfWeek(date);
        Date startDate = getMonday(date);
        try {
            startDate = getShortDate(startDate, SDF_DAY);
        } catch (Exception e) {
            e.printStackTrace();
        }

        if (weekDay < 6) {
            returnMap.put("daynum", 5);
            returnMap.put("startTime", getDayOff(startDate, offSet * 7));
            returnMap.put("endTime", getDayOff(startDate, 5 + offSet * 7));
        } else {
            returnMap.put("daynum", 2);
            returnMap.put("startTime", getDayOff(startDate, 5 + offSet * 7));
            returnMap.put("endTime", getDayOff(startDate, 7 + offSet * 7));
        }
        return returnMap;
    }

    /**
     * 获取日期信息
     *
     * @param date
     * @param field 0-秒，1-分，2-时（24），3-天（月），4-月，5-年
     * @return
     */
    public static int getFieldOfDate(Date date, int field) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int num = 0;
        switch (field) {
            case 0:
                num = cal.get(Calendar.MILLISECOND);
                break;
            case 1:
                num = cal.get(Calendar.MILLISECOND);
                break;
            case 2:
                num = cal.get(Calendar.HOUR_OF_DAY);
                break;
            case 3:
                num = cal.get(Calendar.DAY_OF_MONTH);
                break;
            case 4:
                num = cal.get(Calendar.MONTH);
                break;
            case 5:
                num = cal.get(Calendar.YEAR);
                break;
            case 21:
                num = cal.get(Calendar.HOUR);
                break;
            case 22:
                num = cal.get(Calendar.DAY_OF_YEAR);
                break;
            default:
                break;
        }
        return num;
    }

    /**
     * 获取当前时间所处于本年第几
     *
     * @param date
     * @return
     */
    public static int getWeeksOfYear(Date date) {
        Calendar c = new GregorianCalendar();
        c.setFirstDayOfWeek(Calendar.MONDAY);
        c.setMinimalDaysInFirstWeek(7);
        c.setTime(date);

        return c.get(Calendar.WEEK_OF_YEAR);
    }

    /**
     * 获取某一年共有多少周
     *
     * @param year
     * @return
     */
    public static int getMaxWeekNumOfYear(int year) {
        Calendar c = new GregorianCalendar();
        c.set(year, Calendar.DECEMBER, 31, 23, 59, 59);

        return getWeeksOfYear(c.getTime());
    }

    /**
     * 获取某年的第几周的开始日期
     *
     * @param year
     * @param week 从1开始
     * @return
     */
    public static Date getFirstDayOfWeek(int year, int week) {
        week--;

        Date startDate = null;
        try {
            startDate = str2Date(year + "-01-01", SDF_DAY);
        } catch (Exception e) {
            e.printStackTrace();
        }

        // 周几
        int weekDayNum = getDayOfWeek(startDate);
        int offset = 0;
        if (weekDayNum > 1) {
            offset = 8 - weekDayNum;
        }

        Date returnDay = getDayOff(startDate, offset);

        return getDayOff(returnDay, 7 * week);
    }

    /**
     * 是否当前月
     *
     * @param date
     * @return
     * @throws Exception
     */
    public static boolean isCurrentMonth(Date date) throws Exception {
        return (getFirstDayOfMonth(date).getTime() == getFirstDayOfMonth(new Date()).getTime());
    }

	/*public static String transTimeFormat(String timeStr, String srcFormat, String targetFormat) throws Exception {
		Date time = str2Date(timeStr, srcFormat);
		return date2Str(time, targetFormat);
	}*/

    public static boolean isCurrentYear(Date date) throws Exception {
        return (getFirstDayOfYear(date).getTime() == getFirstDayOfYear(new Date()).getTime());
    }

    public static boolean isCurrentDay(Date date) throws Exception {
        return (getStartTimeOfDay(date).getTime() == getStartTimeOfDay(new Date()).getTime());
    }

    /**
     * description 获取一天15分钟的时间列表
     *
     * @param date
     * @return
     * @author feng
     * @since 18:42 2018/12/10
     */
    public static List<Date> get15MinDateList(Date date) throws Exception {
        List<Date> resultDateList = new ArrayList<>();
        Date min15Date = getStartTimeOfDay(date);
        for (int i = 0; i < 96; i++) {
            resultDateList.add(min15Date);
            min15Date = getMinuteOff(min15Date, 15);
        }
        return resultDateList;
    }

    /**
     * Description: 获取某一时间所在15min的向下取整时间点，如2018-01-01 20:17:15 对应的为 2018-01-01 20:15:00;2018-01-01 20:30:02 对应的为 2018-01-01 20:30:00
     *
     * @param timeNow
     * @return Date
     * @throws Exception
     * @author cuixubin
     * @since 2018年12月18日: 下午6:24:52
     */
    public static Date get15MinLite(Date timeNow) throws Exception {
        if (null == timeNow) {
            return null;
        }

        int hourMinute = getMinute(timeNow);
        if (hourMinute < 15) {
            hourMinute = 0;
        } else if (hourMinute < 30) {
            hourMinute = 15;
        } else if (hourMinute < 45) {
            hourMinute = 30;
        } else if (hourMinute < 60) {
            hourMinute = 45;
        }

        Date resultDate = getShortDate(timeNow, SDF_HOUR);
        resultDate = getMinuteOff(resultDate, hourMinute);

        return resultDate;
    }

    /**
     * description 获取时间段的月份列表
     *
     * @param startDate 开始时间
     * @param endDate   结束时间
     * @return java.util.List<java.util.Date>
     * @author feng
     * @since 12:16 2019/4/22
     */
    public static List<Date> getMonthList(Date startDate, Date endDate) throws Exception {
        List<Date> resultList = new ArrayList<>();
        while (startDate.before(endDate)) {
            resultList.add(startDate);
            startDate = getMonthOff(startDate, 1);
        }
        return resultList;
    }

    public static List<Date> getYearList(Date startDate, Date endDate) throws Exception {
        List<Date> resultList = new ArrayList<>();
        while (startDate.before(endDate)) {
            resultList.add(startDate);
            startDate = getYearOff(startDate, 1);
        }
        return resultList;
    }

    /**
     * description 获取时间段的
     *
     * @param startDate
     * @param endDate
     * @return java.util.List<java.util.Date>
     * @author feng
     * @since 17:29 2019/4/23
     */
    public static List<Date> getDayList(Date startDate, Date endDate) throws Exception {
        List<Date> resultList = new ArrayList<>();
        while (startDate.before(endDate)) {
            resultList.add(startDate);
            startDate = getDayOff(startDate, 1);
        }
        return resultList;
    }

    public static boolean isSameMonth(Date date1, Date date2) throws Exception {
        return getFirstDayOfMonth(date1).getTime() == getFirstDayOfMonth(date2).getTime();
    }

    public static Double getYearPassedTimeRatio(Date startTime, Date endTime) {
        Date now = new Date();
        Date nowYearStartTime = DateUtils.getFirstDayOfYear(now);
        Date nextYearStartTime = DateUtils.getYearOff(nowYearStartTime, 1);
        if (!endTime.after(nowYearStartTime)) {
            return 1.0;
        }
        if (!startTime.before(nextYearStartTime)) {
            return 0.0;
        }

        return (endTime.getTime() - nowYearStartTime.getTime()) * 1.0 / (nextYearStartTime.getTime() - nowYearStartTime.getTime());
    }

    public static Double getMonthPassedTimeRatio(Date startTime, Date endTime) {
        Date now = new Date();
        Date nowMonthStartTime = DateUtils.getFirstDayOfMonth(now);
        Date nextMonthStartTime = DateUtils.getMonthOff(nowMonthStartTime, 1);
        if (!endTime.after(nowMonthStartTime)) {
            return 1.0;
        }
        if (!startTime.before(nextMonthStartTime)) {
            return 0.0;
        }
        return (endTime.getTime() - nowMonthStartTime.getTime()) * 1.0 / (nextMonthStartTime.getTime() - nowMonthStartTime.getTime());
    }

    /**
     * LocalDate类型转为Date
     *
     * @param localDate LocalDate object
     * @return Date object
     */
    public static Date localDate2Date(LocalDate localDate) {

        ZonedDateTime zonedDateTime = localDate.atStartOfDay(ZoneId.systemDefault());
        return Date.from(zonedDateTime.toInstant());
    }

    /**
     * LocalDateTime类型转为Date
     *
     * @param localDateTime LocalDateTime object
     * @return Date object
     */
    public static Date localDateTime2Date(LocalDateTime localDateTime) {
        return Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
    }

    /**
     * Description: Date转换为LocalDateTime
     *
     * @param date
     * @return LocalDateTime
     * @author luoguangyi
     * @since 2019年8月5日: 下午5:25:27 Update By luoguangyi 2019年8月5日: 下午5:25:27
     */
    public static LocalDateTime date2LocalDateTime(Date date) {
        return LocalDateTime.ofInstant(date.toInstant(), ZoneId.systemDefault());
    }

    // 获取指定日期的毫秒
    public static Long getMilliByLocalDateTime(LocalDateTime time) {
        return time.atZone(ZoneId.systemDefault()).toInstant().toEpochMilli();
    }

    // 时间戳转LocalDateTime(LocalDate)
    public static LocalDateTime getLocalDateTimeFromMilli(Long timestamp) {
        //LocalDate localDate = Instant.ofEpochMilli(timestamp).atZone(ZoneOffset.ofHours(8)).toLocalDate();
        return Instant.ofEpochMilli(timestamp).atZone(ZoneId.systemDefault()).toLocalDateTime();
    }


    /**
     * 获取时间戳
     *
     * @param dateTime
     * @return
     */
    public static Long getMilliByLocalDateTime(String dateTime) {
        return parse(dateTime).atZone(ZoneId.systemDefault()).toInstant().toEpochMilli();
    }

    // 获取指定日期的秒
    public static Long getSecondsByLocalDateTime(LocalDateTime time) {
        return time.atZone(ZoneId.systemDefault()).toInstant().getEpochSecond();
    }

    /**
     * 格式化LocalDateTime为指定格式 字符串
     *
     * @param pattern 格式
     * @return 日期字符串
     */
    public static String format(LocalDateTime time, String pattern) {
        return time.format(DateTimeFormatter.ofPattern(pattern));
    }

    public static String format(LocalDateTime time, DateTimeFormatter pattern) {
        return time.format(pattern);
    }

    public static String format(LocalDateTime time) {
        return time.format(FORMATTER);
    }

    public static String formatDate(Date date) {
        return format(date2LocalDateTime(date));
    }

    public static String formatDate(Date date, String pattern) {
        return format(date2LocalDateTime(date), pattern);
    }

    /**
     * 解析字符串日期为Date
     *
     * @param dateStr 日期字符串
     * @param pattern 格式
     * @return Date
     */
    public static LocalDateTime parse(String dateStr, String pattern) {
        return LocalDateTime.parse(dateStr, DateTimeFormatter.ofPattern(pattern));
    }

    public static LocalDateTime parse(String dateStr, DateTimeFormatter pattern) {
        return LocalDateTime.parse(dateStr, pattern);
    }

    public static LocalDateTime parse(String dateStr) {
        return LocalDateTime.parse(dateStr, FORMATTER);
    }

    public static Date parseDate(String dateStr) {
        return localDateTime2Date(LocalDateTime.parse(dateStr, FORMATTER));
    }

    // 获取当前时间的指定格式
    public static String formatNow(String pattern) {
        return format(LocalDateTime.now(), pattern);
    }

    // 日期加上一个数,根据field不同加不同值,field为ChronoUnit.*
    public static LocalDateTime plus(LocalDateTime time, long number, TemporalUnit field) {
        return time.plus(number, field);
    }

    // 日期减去一个数,根据field不同减不同值,field参数为ChronoUnit.*
    public static LocalDateTime minus(LocalDateTime time, long number, TemporalUnit field) {
        return time.minus(number, field);
    }

    /**
     * 获取两个日期的差 field参数为ChronoUnit.* 默认累加的方式 重要说明： 计算月份差 计算的是否满月，年份同理，
     * 区别1：Period只考虑到日期，ChronoUnit.between()计算到具体时分秒
     * 区别2：Period只计算月份差，不考虑年份是否一样，ChronoUnit.between()计算考虑年份
     * :Period只考虑到日期，field.between(startTime, endTime)会计算到具体时间 period.getMonths();
     * field.between 比如：6月6号8点到8月6号7点： 2 1 比如：6月6号8点到8月6号9点： 2 2 比如：6月5号到8月6号： 2 2
     * 比如：6月7号到8月6号： 1 1
     *
     * @param startTime
     * @param endTime
     * @param field     单位(年月日时分秒)
     * @return
     */
    public static long betweenTwoTime(LocalDateTime startTime, LocalDateTime endTime, ChronoUnit field) {

        // Period period = Period.between(LocalDate.from(startTime),
        // LocalDate.from(endTime));
        // startTime : 1993-10-19
        // endTime : 2019-08-06
        // 年龄 : 25 年 9 月 18 日
        // period.getYears(); period.getMonths(); period.getDays();
        // 25 年 9 月 18 日
        // field.between(startTime, endTime);
        // 25 年 25*12+9 （ 25*12+9）个月+18 日

        return field.between(startTime, endTime);
    }

    /**
     * 获取间隔时间
     *
     * @param startTime
     * @param endTime
     * @return
     */
    public static long betweenTwoTime(String startTime, String endTime, ChronoUnit field) {
        return betweenTwoTime(parse(startTime), parse(endTime), field);
    }

    /**
     * 获取间隔秒数
     *
     * @param startTime
     * @param endTime
     * @return
     */
    public static long betweenTwoTimeSecond(String startTime, String endTime) {
        return betweenTwoTime(startTime, endTime, ChronoUnit.SECONDS);
    }

    /**
     * Description:获得当前时间的字符串.
     *
     * @return String 当前时间的字符串格式.格式是yyyyMMddHHmmss
     * @author lijie
     */
    public static String getNowTimeStr() {
        return LocalDateTime.now().format(FORMATTER);
    }

    public static String getTimeStr(LocalDateTime date) {
        return date.format(FORMATTER);
    }

    public static String getMinusHour(String dateTime, long hour) {
        LocalDateTime parse = parse(dateTime).minusHours(hour);
        return format(parse);
    }

    public static String getSdfTimeNotDate(String dateTime) {
        return dateTime.substring(8, 14);
    }

    public static String getPlusHour(String dateTime, long hour) {
        LocalDateTime parse = parse(dateTime).plusHours(hour);
        return format(parse);
    }

    public static String getPlusDay(String dateTime, long days) {
        LocalDateTime parse = parse(dateTime).plusDays(days);
        return format(parse);
    }

    public static String getPlusHourNow(long hour) {
        LocalDateTime parse = LocalDateTime.now().plusHours(hour);
        return format(parse);
    }

    /**
     * 获取年
     *
     * @return 年
     */
    public static int getYear(LocalDateTime ldt) {
        return ldt.getYear();
    }

    /**
     * 获取月份
     */
    public static int getMonth(LocalDateTime ldt) {
        return ldt.getMonthValue();
    }

    /**
     * 获取月份差 不算满月的月份差（比如7月31号到8月1号仍然算一个月份差）
     */
    public static int getBetweenMonth(LocalDateTime startLdt, LocalDateTime endLdt) {
        int minusMonth = (endLdt.getYear() - startLdt.getYear()) * 12 - endLdt.getMonthValue()
                - startLdt.getMonthValue();
        return minusMonth;
    }

    /**
     * 获取月份最大天数
     */
    public static int getDaylengthOfMonth() {
        LocalDateTime localTime = LocalDateTime.now();
        return localTime.toLocalDate().lengthOfMonth();
    }

    /**
     * 获取年份最大天数
     */
    public static int getDaylengthOfYear() {
        LocalDateTime localTime = LocalDateTime.now();
        return localTime.toLocalDate().lengthOfYear();
    }

    /**
     * Description: 日期格式转换
     *
     * @param srcDateStr      源日期字符串
     * @param srcDatePattern  源日期字符串格式
     * @param destDatePattern 目标日期字符串格式
     * @return String
     * @author luoguangyi
     * @since 2019年10月9日: 下午2:29:23 Update By luoguangyi 2019年10月9日: 下午2:29:23
     */
    public static String transferDateFormat(String srcDateStr, String srcDatePattern, String destDatePattern) {
        DateTimeFormatter srcFommater = DateTimeFormatter.ofPattern(srcDatePattern);
        DateTimeFormatter destFommater = DateTimeFormatter.ofPattern(destDatePattern);
        return LocalDateTime.parse(srcDateStr, srcFommater).format(destFommater);
    }


    //获取时间差方法
    public static String getTimeDiff(Date startDate, Date endDate) {
        long diffMS = DateUtil.betweenMs(startDate, endDate);
        //(1000 * 60 * 60 * 24)
        Long dayMSRate = 86400000L;
        //(1000 * 60 * 60)
        Long hourMSRate = 3600000L;
        //(1000 * 60)
        Long minuteMSRate = 60000L;
        //(1000)
        Long secondMSRate = 1000L;
        long days = diffMS / dayMSRate;
        //获取时
        long hours = diffMS % dayMSRate / hourMSRate;
        //获取分钟
        long minutes = diffMS % hourMSRate / minuteMSRate;
        //获取秒
        long seconds = diffMS % minuteMSRate / secondMSRate;
        String format = "{}天{}小时{}分{}秒";
        return StrUtil.format(format, days, hours, minutes, seconds);
    }
    
    //获取时间差方法
    public static double getTimeDiffToDay(Date startDate, Date endDate) {
        long diffMS = DateUtil.betweenMs(startDate, endDate);
        //(1000 * 60 * 60 * 24)
        Long dayMSRate = 86400000L;
        //(1000 * 60 * 60)
        Long hourMSRate = 3600000L;
        
        long days = diffMS / dayMSRate;
        //获取时
        long hours = diffMS % dayMSRate / hourMSRate;
        double hour = hours/2.4;
        BigDecimal b = new BigDecimal(hour * 0.1);
		double hour1 = b.setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
        return days + hour1;
    }
    
    //获取时间差方法
    public static String getTimeDiff(Long startTimestamp, Long endTimestamp) {
        Date startDate = new Date(startTimestamp);
        Date endDate = new Date(endTimestamp);
        return getTimeDiff(startDate, endDate);
    }

    public static void main(String[] args) {
    	DateTime date = DateUtil.date();
		DateTime date1 = DateUtil.date();
		date1.offset(DateField.HOUR_OF_DAY, 23);
		System.out.println(getTimeDiffToDay(date, date1));
	}
    
}
