package com.persagy.zkt.exception;


import com.persagy.zkt.commons.response.ResponseMsg;
import com.persagy.zkt.commons.response.ResponseUtil;
import com.persagy.zkt.enumeration.EnumResultCode;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

@RestControllerAdvice
@Order(Ordered.HIGHEST_PRECEDENCE)
public class GlobalExceptionHandler {
    private static final Logger log = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    @ResponseBody
    @ExceptionHandler(value = Exception.class)
    ResponseMsg handleException(Exception e, HttpServletRequest request, HttpServletResponse response) {
    	log.error("请求异常", e);
        if (e instanceof CommonException) {
            CommonException commonException = (CommonException) e;
            return ResponseUtil.errorMsg(commonException.getErrCode(), commonException.getErrMsg());
        } else if (e instanceof ParameterIsNullException) {
            return ResponseUtil.errorMsg(
                    EnumResultCode.RCC_PARAMETER_IS_NULL.getCode(), EnumResultCode.RCC_PARAMETER_IS_NULL.getMsg());
        } else {
            return ResponseUtil.errorMsg(
                    EnumResultCode.RCC_REQUEST_EXCEPTION.getCode(), EnumResultCode.RCC_REQUEST_EXCEPTION.getMsg());
        }
    }

    /**
     * Validator 参数校验异常处理
     */
    @ResponseBody
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public ResponseMsg handleMethodVoArgumentNotValidException(MethodArgumentNotValidException ex) {
        List<ObjectError> allErrors = ex.getBindingResult().getAllErrors();
        StringBuffer errorMessage = new StringBuffer("");
        allErrors.forEach(objectError -> errorMessage.append(objectError.getDefaultMessage()).append("！"));
        return ResponseUtil.errorMsg(EnumResultCode.RCC_PARAMETER_INVALID.getCode(),errorMessage.toString());
    }

    /**
     * 自定义异常处理，参数输入有误
     * 例如：身份证号重复
     */
    @ResponseBody
    @ExceptionHandler(value = ParamsInputException.class)
    public ResponseMsg handleParamsInputException(ParamsInputException ex) {
        return ResponseUtil.errorMsg(EnumResultCode.RCC_PARAMETER_INVALID.getCode(), EnumResultCode.RCC_PARAMETER_INVALID.getMsg());
    }
}
